package hu.oroscom.c1801;

import com.vaadin.server.VaadinSession;
import hu.oroscom.c1801.domain.User;
import hu.oroscom.c1801.service.ContextService;
import hu.oroscom.c1801.service.ScheduledTasks;
import hu.oroscom.c1801.service.TokenService;
import hu.oroscom.c1801.service.UserService;
import hu.oroscom.c1801.util.UserUtil;
import org.springframework.beans.BeansException;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.actuate.health.Health;
import org.springframework.boot.actuate.health.HealthIndicator;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.builder.SpringApplicationBuilder;
import org.springframework.boot.context.web.SpringBootServletInitializer;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.context.ApplicationEvent;
import org.springframework.context.ApplicationListener;
import org.springframework.context.annotation.*;
import org.springframework.context.event.ContextRefreshedEvent;
import org.springframework.security.authentication.event.AuthenticationSuccessEvent;
import org.springframework.security.core.userdetails.UserDetails;
import org.vaadin.spring.events.EventBus;
import org.vaadin.spring.events.EventScope;
import org.vaadin.spring.events.annotation.EventBusScope;
import org.vaadin.spring.events.internal.ScopedEventBus;
import org.vaadin.spring.i18n.annotation.EnableVaadinI18N;
import org.vaadin.spring.internal.VaadinSessionScope;
import org.vaadin.spring.internal.VaadinUIScope;

import javax.inject.Inject;

@SpringBootApplication
@Import({SecurityConfig.class, ScheduledTasks.class})
@EnableVaadinI18N
public class Application extends SpringBootServletInitializer implements ApplicationListener, ApplicationContextAware, HealthIndicator {
	public static final String contextPath = "/app";
	private static ApplicationContext applicationContext = null;
	private static Class<Application> applicationClass = Application.class;

	@Inject
	TokenService tokenService;

	@Inject
	UserService userService;

	@Inject
	ContextService contextService;

    public static void main(String[] args) {
//	    Locale locale = new Locale("hu", "HU");
//	    Locale.setDefault(locale);
		SpringApplication.run(applicationClass, args);
	}

	public static ApplicationContext getApplicationContext() {
		return applicationContext;
	}

	@Override
	public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
		this.applicationContext = applicationContext;
	}

	@Override
	protected SpringApplicationBuilder configure(SpringApplicationBuilder application) {
		return application.sources(applicationClass);
	}

	@Override
	public void onApplicationEvent(ApplicationEvent applicationEvent) {
		if (applicationEvent instanceof AuthenticationSuccessEvent)
		{
			AuthenticationSuccessEvent event = (AuthenticationSuccessEvent) applicationEvent;
			UserDetails userDetails = (UserDetails) event.getAuthentication().getPrincipal();
			User user = userService.findUserByUsername(userDetails.getUsername());
			user = userService.userLoggedIn(user);
			UserUtil.deleteSession();
			UserUtil.setUser(user);
			if (VaadinSession.getCurrent() != null) {
				VaadinSession.getCurrent().setAttribute(User.class, user);
			}
		} else if (applicationEvent instanceof ContextRefreshedEvent) {
			contextService.initAuthorities();
			tokenService.generateRSAKeyPair();
		}
	}

	@Bean
	@EventBusScope(EventScope.APPLICATION)
	EventBus applicationEventBus() {
		return new ScopedEventBus(EventScope.APPLICATION);
	}

	@Bean
	@Scope(value = VaadinSessionScope.VAADIN_SESSION_SCOPE_NAME, proxyMode = ScopedProxyMode.INTERFACES)
	@EventBusScope(value = EventScope.SESSION, proxy = true)
	EventBus proxiedSessionEventBus() {
		return sessionEventBus();
	}

	@Bean
	@Scope(value = VaadinSessionScope.VAADIN_SESSION_SCOPE_NAME, proxyMode = ScopedProxyMode.NO)
	@EventBusScope(EventScope.SESSION)
	EventBus sessionEventBus() {
		return new ScopedEventBus(EventScope.SESSION, applicationEventBus());
	}

	@Bean
	@Scope(value = VaadinUIScope.VAADIN_UI_SCOPE_NAME, proxyMode = ScopedProxyMode.INTERFACES)
	@EventBusScope(value = EventScope.UI, proxy = true)
	EventBus proxiedUiEventBus() {
		return uiEventBus();
	}

	@Bean
	@Scope(value = VaadinUIScope.VAADIN_UI_SCOPE_NAME, proxyMode = ScopedProxyMode.NO)
	@Primary
	@EventBusScope(EventScope.UI)
	EventBus uiEventBus() {
		return new ScopedEventBus(EventScope.UI, sessionEventBus());
	}

	@Override
	public Health health() {
		return Health.up().withDetail("hello", "world").build();
	}
}
