package hu.oroscom.c1801;

import hu.oroscom.c1801.jparepository.DefultRepositoryFactoryBean;
import org.eclipse.persistence.config.FlushClearCache;
import org.eclipse.persistence.config.PersistenceUnitProperties;
import org.springframework.context.ApplicationContext;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.env.Environment;
import org.springframework.core.io.Resource;
import org.springframework.dao.annotation.PersistenceExceptionTranslationPostProcessor;
import org.springframework.data.jpa.repository.config.EnableJpaRepositories;
import org.springframework.jdbc.datasource.DriverManagerDataSource;
import org.springframework.jdbc.datasource.init.DatabasePopulatorUtils;
import org.springframework.jdbc.datasource.init.ResourceDatabasePopulator;
import org.springframework.orm.jpa.JpaTransactionManager;
import org.springframework.orm.jpa.JpaVendorAdapter;
import org.springframework.orm.jpa.LocalContainerEntityManagerFactoryBean;
import org.springframework.orm.jpa.vendor.EclipseLinkJpaDialect;
import org.springframework.orm.jpa.vendor.EclipseLinkJpaVendorAdapter;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.annotation.EnableTransactionManagement;
import org.vaadin.spring.i18n.ResourceBundleMessageProvider;

import javax.inject.Inject;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.sql.DataSource;
import java.beans.PropertyVetoException;
import java.io.IOException;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Properties;

@Configuration
@EnableJpaRepositories(basePackages = "hu.oroscom.c1801.jparepository", repositoryFactoryBeanClass = DefultRepositoryFactoryBean.class)
@EnableTransactionManagement
public class JPAConfig {
	@Inject
	private ApplicationContext applicationContext;

	@Bean
	public DataSource dataSource() throws SQLException {

		DriverManagerDataSource dataSource = new DriverManagerDataSource();
		dataSource.setDriverClassName("com.mysql.jdbc.Driver");
		dataSource.setUrl("jdbc:mysql://localhost:3306/c1801?noAccessToProcedureBodies=true&amp;autoReconnect=true&amp;useUnicode=yes&amp;characterEncoding=UTF-8&amp;characterSetResults=UTF-8");
		dataSource.setUsername("root");
		dataSource.setPassword("codeaccepted");

		return dataSource;
	}

	@Inject
	private Environment environment;

	/**
	 * Base
	 * @return
	 */
//	@Bean
//	DataSource dataSource() {
//		DataSource dataSource = null;
//		JndiTemplate jndi = new JndiTemplate();
//		try {
//			dataSource = (DataSource) jndi.lookup("java:/comp/env/jdbc/c1801");
//		} catch (NamingException e) {
//			e.printStackTrace();
//		}
//		return dataSource;
//	}

	/**
	 * Teszt
	 *
	 * @return
	 */
//	@Bean
//	DataSource dataSource() {
//		DataSource dataSource = null;
//		JndiTemplate jndi = new JndiTemplate();
//		try {
//			dataSource = (DataSource) jndi.lookup("java:/comp/env/jdbc/c1801teszt");
//		} catch (NamingException e) {
//			e.printStackTrace();
//		}
//		return dataSource;
//	}

	JpaVendorAdapter jpaAdapter() {
		EclipseLinkJpaVendorAdapter vendorAdapter = new EclipseLinkJpaVendorAdapter();
		vendorAdapter.setGenerateDdl(true);
		return vendorAdapter;
	}

	Properties jpaProperties() {
		Properties properties = new Properties();
		properties.put("eclipselink.weaving", "static");
//		properties.put("eclipselink.weaving", "false");
		properties.put("jpaDialect", "org.springframework.orm.jpa.vendor.EclipseLinkJpaDialect");
		properties.put("eclipselink.ddl-generation", "create-or-extend-tables");
//		properties.put("eclipselink.ddl-generation", "drop-and-create-tables");
//		properties.put("eclipselink.logging.level", "FINE");
//		properties.put("eclipselink.logging.level.sql", "FINE");
		properties.put("eclipselink.logging.parameters", "true");
//		properties.put("eclipselink.cache.shared.default", "false");
		properties.put("javax.persistence.cache.storeMode", "REFRESH");
		properties.put(PersistenceUnitProperties.FLUSH_CLEAR_CACHE, FlushClearCache.Drop);
		return properties;
	}

	@Bean
	public EntityManagerFactory entityManagerFactory() throws ClassNotFoundException, PropertyVetoException, SQLException {
		LocalContainerEntityManagerFactoryBean emf = new LocalContainerEntityManagerFactoryBean();
		emf.setDataSource(dataSource());
		emf.setPackagesToScan("hu.oroscom.c1801.domain");
		emf.setJpaVendorAdapter(jpaAdapter());
		emf.setJpaProperties(jpaProperties());
		emf.afterPropertiesSet();
		return emf.getObject();
	}

	@Bean
	public EntityManager entityManager(EntityManagerFactory entityManagerFactory) {
		return entityManagerFactory.createEntityManager();
	}

	@Bean
	public PlatformTransactionManager transactionManager() throws ClassNotFoundException, PropertyVetoException, SQLException {
		JpaTransactionManager txManager = new JpaTransactionManager();
		txManager.setEntityManagerFactory(entityManagerFactory());

		return txManager;
	}

	@Bean
	public PersistenceExceptionTranslationPostProcessor persistenceExceptionTranslationPostProcessor() {
		return new PersistenceExceptionTranslationPostProcessor();
	}

	@Bean
	public EclipseLinkJpaDialect eclipseLinkJpaDialect() {
		return new EclipseLinkJpaDialect();
	}

	private void runDataScript(DataSource dataSource) {

		String scripts = "classpath*:data.sql";
		Boolean exists = false;

		List<Resource> resources = new ArrayList<>();

		try {
			resources.addAll(Arrays.asList(applicationContext
					.getResources(scripts)));
		} catch (IOException ex) {
			throw new IllegalStateException("Unable to load resource from "
					+ scripts, ex);
		}

		ResourceDatabasePopulator populator = new ResourceDatabasePopulator();
		for (Resource resource : resources) {
			if (resource.exists()) {
				exists = true;
				populator.addScript(resource);
				populator.setContinueOnError(false);
			}
		}

		if (exists) {
			DatabasePopulatorUtils.execute(populator, dataSource);
		}

	}

	@Bean
	ResourceBundleMessageProvider resourceBundleMessageProvider() {
		return new ResourceBundleMessageProvider("messages");
	}

}
