package hu.oroscom.c1801;

import hu.oroscom.c1801.service.SecurityService;
import hu.oroscom.c1801.webservice.AjaxAuthenticationFilter;
import hu.oroscom.c1801.webservice.AjaxAuthenticationSuccessHandler;
import hu.oroscom.c1801.webservice.CustomTokenAuthenticationFilter;
import hu.oroscom.c1801.webservice.TokenAuthenticationSuccessHandler;
import org.springframework.context.annotation.Bean;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.config.annotation.authentication.builders.AuthenticationManagerBuilder;
import org.springframework.security.config.annotation.method.configuration.EnableGlobalMethodSecurity;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.builders.WebSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.config.annotation.web.configuration.WebSecurityConfigurerAdapter;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.web.authentication.AuthenticationFailureHandler;
import org.springframework.security.web.authentication.AuthenticationSuccessHandler;
import org.springframework.security.web.authentication.LoginUrlAuthenticationEntryPoint;
import org.springframework.security.web.authentication.UsernamePasswordAuthenticationFilter;
import org.springframework.security.web.authentication.www.BasicAuthenticationFilter;
import org.springframework.security.web.header.writers.frameoptions.XFrameOptionsHeaderWriter;
import org.springframework.security.web.util.matcher.AntPathRequestMatcher;
import org.vaadin.spring.security.annotation.EnableVaadinSecurity;

import javax.inject.Inject;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

@EnableWebSecurity(debug = false)
@EnableGlobalMethodSecurity(prePostEnabled = true, securedEnabled = true, jsr250Enabled = true)
@EnableVaadinSecurity
public class SecurityConfig extends WebSecurityConfigurerAdapter {

	@Inject
	SecurityService securityService;

	@Override
	protected void configure(AuthenticationManagerBuilder registry) throws Exception {
		registry.userDetailsService(securityService);
	}

	@Override
	public void configure(WebSecurity web) throws Exception {
		web
				.ignoring()
				.antMatchers("/VAADIN/**");
	}

	@Override
	protected void configure(HttpSecurity http) throws Exception {
		http
				.addFilter(usernamePasswordAuthenticationFilter())/*.csrf().disable()*/.authorizeRequests().antMatchers("/client/**").fullyAuthenticated().and()
				.csrf().disable()
				.authorizeRequests()
				.antMatchers(
						Application.contextPath + "/UIDL/**",
						Application.contextPath + "/HEARTBEAT/**",
						Application.contextPath + "/login",
						"/api/**",
						"/client/j_spring_security_check",
						"/client/error"
				)
				.permitAll()
				.anyRequest().fullyAuthenticated()
				.and()
				.exceptionHandling()
				.authenticationEntryPoint(new LoginUrlAuthenticationEntryPoint(Application.contextPath + "/login"));

//		http.addFilter(usernamePasswordAuthenticationFilter())
//				.csrf().disable().authorizeRequests().antMatchers("/client/**").fullyAuthenticated().anyRequest().permitAll().and()
//				.exceptionHandling()
//				.authenticationEntryPoint(new LoginUrlAuthenticationEntryPoint("/client/error"));

		http.addFilterBefore(ajaxAuthenticationFilter(), UsernamePasswordAuthenticationFilter.class);

		http.addFilterBefore(authenticationTokenProcessingFilter(), BasicAuthenticationFilter.class);

		http.headers().addHeaderWriter(new XFrameOptionsHeaderWriter(XFrameOptionsHeaderWriter.XFrameOptionsMode.SAMEORIGIN));
	}

	@Bean
	AjaxAuthenticationFilter ajaxAuthenticationFilter() {
		return new AjaxAuthenticationFilter();
	}

	@Override
	@Bean
	public AuthenticationManager authenticationManagerBean() throws Exception {
		return super.authenticationManagerBean();
	}

	@Bean
	public UsernamePasswordAuthenticationFilter usernamePasswordAuthenticationFilter() throws Exception {
		UsernamePasswordAuthenticationFilter filter = new UsernamePasswordAuthenticationFilter();
		filter.setAllowSessionCreation(true);
		filter.setAuthenticationManager(authenticationManagerBean());
		filter.setAuthenticationSuccessHandler(authenticationSuccessHandler());
		filter.setAuthenticationFailureHandler(new AuthenticationFailureHandler() {
			@Override
			public void onAuthenticationFailure(HttpServletRequest request, HttpServletResponse response, AuthenticationException exception) throws IOException, ServletException {
				response.sendError(HttpServletResponse.SC_OK, "Unauthorized");
			}
		});
		filter.setPostOnly(false);
		filter.setRequiresAuthenticationRequestMatcher(new AntPathRequestMatcher("/client/j_spring_security_check"));
		return filter;
	}

	@Bean
	public AuthenticationSuccessHandler authenticationSuccessHandler() {
		AjaxAuthenticationSuccessHandler successHandler = new AjaxAuthenticationSuccessHandler();
		successHandler.setAlwaysUseDefaultTargetUrl(false);
		successHandler.setUseReferer(true);
		successHandler.setTargetUrlParameter("targetUrl");
		return successHandler;
	}

	@Bean
	public CustomTokenAuthenticationFilter authenticationTokenProcessingFilter() throws Exception {
		CustomTokenAuthenticationFilter filter = new CustomTokenAuthenticationFilter(new AntPathRequestMatcher("/api/**"));
		filter.setAllowSessionCreation(false);
		filter.setAuthenticationManager(authenticationManagerBean());
		filter.setAuthenticationSuccessHandler(new TokenAuthenticationSuccessHandler());
		filter.setAuthenticationFailureHandler(new AuthenticationFailureHandler() {
			@Override
			public void onAuthenticationFailure(HttpServletRequest request, HttpServletResponse response, AuthenticationException exception) throws IOException, ServletException {
				response.sendError(HttpServletResponse.SC_UNAUTHORIZED, "Unauthorized");
			}
		});
		return filter;
	}
}
