package hu.oroscom.c1801.component;

import com.vaadin.server.FontAwesome;
import com.vaadin.shared.ui.label.ContentMode;
import com.vaadin.ui.Button;
import com.vaadin.ui.HorizontalLayout;
import com.vaadin.ui.Label;
import com.vaadin.ui.themes.ValoTheme;
import hu.oroscom.c1801.Application;
import hu.oroscom.c1801.domain.Doctor;
import hu.oroscom.c1801.domain.User;
import hu.oroscom.c1801.domain.filter.EatenFoodFilter;
import hu.oroscom.c1801.domain.filter.Filter;
import hu.oroscom.c1801.domain.filter.StudyEntryFilter;
import hu.oroscom.c1801.enumeration.*;
import hu.oroscom.c1801.service.BaseService;
import hu.oroscom.c1801.service.DoctorService;
import hu.oroscom.c1801.service.UserService;
import hu.oroscom.c1801.util.EventBusAction;
import hu.oroscom.c1801.util.FilterItemBean;
import hu.oroscom.c1801.util.FilterUtil;
import hu.oroscom.c1801.util.MessagesUtil;
import org.vaadin.spring.events.EventBus;
import org.vaadin.spring.i18n.I18N;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * Created by Norbert on 2015.07.09..
 */
public class FilterInfo<F extends Filter, S extends BaseService> extends HorizontalLayout implements Button.ClickListener {
	private I18N i18n;
	private EventBus eventBus;
	private F filter;
	private S[] services;
	private int entityCount = 0;
	private Label filterLabel = new Label();

	public FilterInfo(F filter, S... services) {
		this.filter = filter;
		this.services = services;
		this.i18n = (I18N) Application.getApplicationContext().getBean("i18n");
		this.eventBus = (EventBus) Application.getApplicationContext().getBean("uiEventBus");

		setHeight(30, Unit.PIXELS);
		setWidth(100, Unit.PERCENTAGE);
		addStyleName("filter-info");

		if (FilterType.GLOBAL.equals(filter.getFilterType())) {
			addStyleName("global");
		} else if (FilterType.SAVED.equals(filter.getFilterType())) {
			addStyleName("saved");
		}

		filterLabel.setValue(i18n.get("FilterInfo.filterLabel", entityCount));
		filterLabel.setWidth(120, Unit.PIXELS);
		filterLabel.addStyleName("filter-label");
		filterLabel.addStyleName(ValoTheme.LABEL_BOLD);
		filterLabel.addStyleName(ValoTheme.LABEL_TINY);
		addComponent(filterLabel);

		Label filterStringLabel = new Label(generateFilterString(filter), ContentMode.HTML);
		filterStringLabel.addStyleName("filter-string-label");
		filterStringLabel.addStyleName("ellipsis");
		filterStringLabel.addStyleName(ValoTheme.LABEL_TINY);
		addComponent(filterStringLabel);

		Label expandingGap = new Label();
		expandingGap.setWidth("100%");

		Button deleteButton = new Button(FontAwesome.TIMES);
		deleteButton.addClickListener(this);
		deleteButton.addStyleName("delete-button");
		deleteButton.addStyleName(ValoTheme.BUTTON_LARGE);
		deleteButton.addStyleName(ValoTheme.BUTTON_LINK);
		deleteButton.addStyleName(ValoTheme.BUTTON_ICON_ONLY);
		deleteButton.setWidth(22, Unit.PIXELS);
		deleteButton.setHeight(22, Unit.PIXELS);
		addComponent(deleteButton);

		setExpandRatio(filterStringLabel, 1);
	}

	private String generateFilterString(F filter) {
		String filterString = "";
		try {
			Field filterNameField = filter.getClass().getSuperclass().getDeclaredField("filterName");
			filterNameField.setAccessible(true);
			Object value = filterNameField.get(filter);
			if (value != null) {
				filterString = value + " &rarr; ";
			}
		} catch (NoSuchFieldException | IllegalAccessException e) {
			e.printStackTrace();
		}

		List<Field> declaredFields = new ArrayList<>(Arrays.asList(filter.getClass().getDeclaredFields()));
		try {
			declaredFields.add(0, filter.getClass().getSuperclass().getDeclaredField("ids"));
			declaredFields.add(0, filter.getClass().getSuperclass().getDeclaredField("quickSearch"));
		} catch (NoSuchFieldException e) {
			e.printStackTrace();
		}

		for (Field field : declaredFields) {
			field.setAccessible(true);
			try {
				Object value = field.get(filter);
				if (value != null) {
					if (List.class.equals(field.getType())) {
						filterString += "<strong>" + MessagesUtil.get(i18n, "FilterInfo." + field.getName()) + "</strong> ";
						List<String> filterItemStrings = (List<String>) value;
						int i = 0;
						for (String filterItemString : filterItemStrings) {
							FilterItemBean filterItemBean = FilterUtil.getFilterItem(filterItemString);
							String filterValue;
							switch (field.getName()) {
								case "studyEntryType":
								case "entryType":
									filterValue = StudyEntryType.valueOf(filterItemBean.getFilterValue()).getCaption();
									break;
								case "connectedUser":
									filterValue = filterItemBean.getFilterValue().isEmpty() ? "" : ((User) getService(UserService.class).findOne(Long.valueOf(filterItemBean.getFilterValue()))).getDisplayName();
									break;
								case "doctor":
									filterValue = filterItemBean.getFilterValue().isEmpty() ? "" : ((Doctor) getService(DoctorService.class).findOne(Long.valueOf(filterItemBean.getFilterValue()))).getDisplayName();
									break;
								case "bookingDateTime":
								case "bookingStartDate":
								case "bookingInsertedAt":
								case "bookingDeletedAt":
								case "exportDate":
								case "entryDateTime":
								case "dateTime":
									if (FilterOperand.INTERVAL.equals(filterItemBean.getFilterOperand())) {
										String[] dateArray = filterItemBean.getFilterValue().split("sep");
										filterValue = dateArray[0] + " " + i18n.get("FilterInfo.AND") + " " + dateArray[1];
									} else {
										filterValue = filterItemBean.getFilterValue();
									}
									break;
								case "approved":
								case "online":
									filterValue = filterItemBean.getFilterValue().equals("1") ? i18n.get("FilterPanel.true") : i18n.get("FilterPanel.false");
									break;
								case "serviceManagementNature":
									filterValue = ServiceManagementNature.valueOf(filterItemBean.getFilterValue()).getCaption();
									break;
								case "serviceManagementStatus":
									filterValue = ServiceManagementStatus.valueOf(filterItemBean.getFilterValue()).getCaption();
									break;
								case "baseStatus":
								case "onlineStatus":
									filterValue = EntityStatus.valueOf(filterItemBean.getFilterValue()).getCaption();
									break;
								default:
									filterValue = filterItemBean.getFilterValue();
									break;
							}
							if (!"ids".equals(field.getName())) {
								filterString += filterItemBean.getFilterOperand().getCaption() + " " + filterValue + " ";
								if (i < (filterItemStrings.size() - 1)) {
									filterString += i18n.get("FilterInfo.OR") + " ";
								}
							}
							i++;
						}
					} else if (String.class.equals(field.getType())) {
						String filterValue = (String) value;
						String filterCaption = i18n.get("FilterInfo.name");
						if (filter instanceof EatenFoodFilter || filter instanceof StudyEntryFilter) {
							filterCaption = i18n.get("FilterInfo.description");
						}
						filterString += filterCaption + " " + FilterOperand.CONTAINS.getCaption() + " " + filterValue + " ";
					}
				}
			} catch (IllegalAccessException e) {
				e.printStackTrace();
			}
		}

		filterString = filterString.replaceFirst(i18n.get("FilterInfo.AND"), "");
		return filterString;
	}

	private S getService(Class aClass) {
		for (S availableService : services) {
			if (availableService.getClass().getName().contains(aClass.getName())) {
				return availableService;
			}
		}

		return null;
	}

	public void setEntityCount(int entityCount) {
		this.entityCount = entityCount;
		filterLabel.setValue(i18n.get("FilterInfo.filterLabel", entityCount));
	}

	@Override
	public void buttonClick(Button.ClickEvent event) {
		try {
			eventBus.publish(this, new EventBusAction<>(Action.DELETE_FILTER, filter.getClass().getDeclaredConstructor().newInstance()));
		} catch (InstantiationException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			e.printStackTrace();
		} catch (NoSuchMethodException e) {
			e.printStackTrace();
		}
	}

}
