package hu.oroscom.c1801.component.cell;

import com.vaadin.server.Resource;
import com.vaadin.ui.Button;
import com.vaadin.ui.HorizontalLayout;
import com.vaadin.ui.Label;
import com.vaadin.ui.Table;
import com.vaadin.ui.themes.ValoTheme;


/**
 * Created by Norbert on 2014.11.24..
 */
public class DoubleButtonCell extends AbstractCell {

	private Button.ClickListener firstButtonClickListener;
	private Button.ClickListener secondButtonClickListener;
	private Resource firstButtonIcon;
	private Resource secondButtonIcon;

	public DoubleButtonCell(Resource firstButtonIcon, Button.ClickListener firstButtonClickListener, Resource secondButtonIcon, Button.ClickListener secondButtonClickListener) {
		this.firstButtonIcon = firstButtonIcon;
		this.firstButtonClickListener = firstButtonClickListener;
		this.secondButtonIcon = secondButtonIcon;
		this.secondButtonClickListener = secondButtonClickListener;
	}

	@Override
	public Object generateCell(Table source, Object itemId, Object columnId) {
		HorizontalLayout horizontalLayout = new HorizontalLayout();
		boolean isFirstButtonEnabled = isEnabled(source, itemId, "isFirstButtonEnabled");
		boolean isSecondButtonEnabled = isEnabled(source, itemId, "isSecondButtonEnabled");

		if (isFirstButtonEnabled || isSecondButtonEnabled) {
			if (isFirstButtonEnabled) {
				horizontalLayout.addComponent(generateButton(itemId, firstButtonIcon, firstButtonClickListener));
			}
			if (isSecondButtonEnabled) {
				horizontalLayout.addComponent(generateButton(itemId, secondButtonIcon, secondButtonClickListener));
			}
		} else {
			horizontalLayout.addComponent(new Label(""));
		}

		return horizontalLayout;
	}

	private Button generateButton(Object itemId, Resource icon, Button.ClickListener clickListener) {
		Button button = new Button(icon);
		button.setData(itemId);
		button.addClickListener(clickListener);
		button.addStyleName(ValoTheme.BUTTON_ICON_ONLY);
		button.addStyleName(ValoTheme.BUTTON_SMALL);
		return button;
	}
}
