package hu.oroscom.c1801.component.cell;

import com.vaadin.server.Page;
import com.vaadin.ui.Button;
import com.vaadin.ui.Notification;
import com.vaadin.ui.Table;
import com.vaadin.ui.UI;
import com.vaadin.ui.themes.ValoTheme;
import hu.oroscom.c1801.Application;
import hu.oroscom.c1801.enumeration.Action;
import hu.oroscom.c1801.service.BaseService;
import hu.oroscom.c1801.util.EventBusAction;
import org.vaadin.dialogs.ConfirmDialog;

/**
 * Created by Norbert on 2014.11.24..
 */
public class RemoveCell<S extends BaseService> extends AbstractCell {

	private S service;

	public RemoveCell(Class<S> sClass) {
		String serviceName = sClass.getSimpleName().substring(0, 1).toLowerCase() + sClass.getSimpleName().substring(1);
		service = (S) Application.getApplicationContext().getBean(serviceName);
	}

	@Override
	public Object generateCell(Table source, Object itemId, Object columnId) {
		if (!isVisible(source, itemId)) {
			return null;
		}

		if (isEnabled(source, itemId, "isRemoveCellEnabled")) {
			final Long id = (Long) source.getItem(itemId).getItemProperty("id").getValue();
			Button buttonRemove = new Button("x");
			buttonRemove.addStyleName("buttonRemove");
			buttonRemove.setData(itemId);
			buttonRemove.addClickListener(new Button.ClickListener() {
				@Override
				public void buttonClick(final Button.ClickEvent clickEvent) {
					ConfirmDialog.show(UI.getCurrent(), i18n.get("RemoveCell.confirmationMessage"), i18n.get("RemoveCell.areYouSureMessage"),
							i18n.get("BaseForm.yesButton"), i18n.get("BaseForm.noButton"), new ConfirmDialog.Listener() {

								public void onClose(ConfirmDialog dialog) {
									if (dialog.isConfirmed()) {
										Long id = (Long) clickEvent.getButton().getData();
										try {
											service.delete(id);
											Notification.show(i18n.get("RemoveCell.deletedSuccessfullyMessage"), Notification.Type.TRAY_NOTIFICATION);
											eventBus.publish(this, new EventBusAction<>(Action.DELETE));
										} catch (Exception e) {
											String msg = i18n.get("RemoveCell.integrityConstraintViolationException");
											Notification notification = new Notification(null, msg, Notification.Type.ERROR_MESSAGE, true);
											notification.show(Page.getCurrent());
										}
									}
								}
							});
				}
			});
			buttonRemove.addStyleName(ValoTheme.BUTTON_LINK);
			return buttonRemove;
		} else {
			return null;
		}
	}
}
