package hu.oroscom.c1801.component.form;

import com.vaadin.ui.*;
import com.vaadin.ui.themes.ValoTheme;
import hu.oroscom.c1801.Application;
import hu.oroscom.c1801.domain.StudyEntry;
import hu.oroscom.c1801.domain.StudyEntryBaseData;
import hu.oroscom.c1801.domain.StudyEntryData;
import hu.oroscom.c1801.enumeration.Action;
import hu.oroscom.c1801.enumeration.Operation;
import hu.oroscom.c1801.enumeration.StudyEntryType;
import hu.oroscom.c1801.service.StudyEntryBaseDataService;
import hu.oroscom.c1801.service.StudyEntryDataService;
import hu.oroscom.c1801.service.StudyEntryService;
import hu.oroscom.c1801.util.EventBusAction;
import hu.oroscom.c1801.util.UserUtil;
import org.vaadin.spring.events.EventBus;
import org.vaadin.spring.i18n.I18N;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

/**
 * Created by Norbert on 2015.06.24..
 */
public class AssignmentPanel extends VerticalLayout implements Button.ClickListener {

	private I18N i18n = (I18N) Application.getApplicationContext().getBean("i18n");
	private EventBus eventBus = (EventBus) Application.getApplicationContext().getBean("uiEventBus");

	private Button applyButton;
	private Button cancelButton;
	private ComboBox operationComboBox;
	private OptionGroup studyEntryTypeOptionGroup;

	private Long[] selectedStudyEntryDataIds;
	private StudyEntryBaseDataService studyEntryBaseDataService;
	private StudyEntryDataService studyEntryDataService;
	private StudyEntryService studyEntryService;

	public AssignmentPanel(Set<Object> selectedStudyEntryDataIds, StudyEntryService studyEntryService, StudyEntryDataService studyEntryDataService, StudyEntryBaseDataService studyEntryBaseDataService) {
		this.selectedStudyEntryDataIds = selectedStudyEntryDataIds.toArray(new Long[selectedStudyEntryDataIds.size()]);
		this.studyEntryBaseDataService = studyEntryBaseDataService;
		this.studyEntryDataService = studyEntryDataService;
		this.studyEntryService = studyEntryService;

		setMargin(true);
		addStyleName("manage-targeting");

		applyButton = new Button(i18n.get("AssignmentPanel.applyButton"), this);
		cancelButton = new Button(i18n.get("AssignmentPanel.cancelButton"), this);

		Label assignmentHeadingLabel = new Label(i18n.get("AssignmentPanel.assignmentHeadingLabel", i18n.get(StudyEntryData.class.getSimpleName())));
		assignmentHeadingLabel.addStyleName("manage-targeting-heading");
		assignmentHeadingLabel.addStyleName(ValoTheme.LABEL_H2);

		Label selectedStudyEntryDataLabel = new Label(i18n.get("AssignmentPanel.selectedStudyEntryDataLabel", selectedStudyEntryDataIds.size()));

		studyEntryTypeOptionGroup = new OptionGroup(i18n.get("AssignmentPanel.valueOptionGroup"));
		studyEntryTypeOptionGroup.setMultiSelect(true);
		for (StudyEntryType studyEntryType : StudyEntryType.values()) {
			studyEntryTypeOptionGroup.addItem(studyEntryType);
			studyEntryTypeOptionGroup.setItemCaption(studyEntryType, studyEntryType.getCaption());
		}

		operationComboBox = new ComboBox(i18n.get("AssignmentPanel.operationComboBox"));
		operationComboBox.addStyleName(ValoTheme.COMBOBOX_SMALL);
		operationComboBox.setTextInputAllowed(false);
		operationComboBox.setNullSelectionAllowed(false);
		operationComboBox.addItem(Operation.ADD);
		operationComboBox.setItemCaption(Operation.ADD, Operation.ADD.getCaption());
		operationComboBox.addItem(Operation.DELETE);
		operationComboBox.setItemCaption(Operation.DELETE, Operation.DELETE.getCaption());
		operationComboBox.addItem(Operation.OVERWRITE);
		operationComboBox.setItemCaption(Operation.OVERWRITE, Operation.OVERWRITE.getCaption());
		operationComboBox.setValue(Operation.ADD);

		HorizontalLayout buttonBarHorizontalLayout = new HorizontalLayout();
		buttonBarHorizontalLayout.addStyleName("margin-top");
		buttonBarHorizontalLayout.setSpacing(true);
		buttonBarHorizontalLayout.setWidth(100, Unit.PERCENTAGE);
		Label spacer = new Label();
		buttonBarHorizontalLayout.addComponent(spacer);
		buttonBarHorizontalLayout.addComponent(applyButton);
		buttonBarHorizontalLayout.addComponent(cancelButton);
		buttonBarHorizontalLayout.setExpandRatio(spacer, 1);

		addComponent(assignmentHeadingLabel);
		addComponent(selectedStudyEntryDataLabel);
		addComponent(new FormLayout(studyEntryTypeOptionGroup, operationComboBox));
		addComponent(buttonBarHorizontalLayout);
	}

	@Override
	public void buttonClick(Button.ClickEvent event) {
		if (event.getButton().equals(applyButton)) {
			Set<StudyEntryType> selectedStudyEntryTypes = (Set<StudyEntryType>) studyEntryTypeOptionGroup.getValue();
			if (selectedStudyEntryTypes.isEmpty()) {
				Notification.show(i18n.get("AssignmentPanel.noEntitySelected"), Notification.Type.ERROR_MESSAGE);
				return;
			}
			StudyEntryType[] studyEntryTypes = (selectedStudyEntryTypes).toArray(new StudyEntryType[selectedStudyEntryTypes.size()]);
			Set<StudyEntryBaseData> selectedStudyEntryBaseData = studyEntryBaseDataService.findAllByStudyEntryType(studyEntryTypes);
			Set<StudyEntry> selectedStudyEntries = studyEntryService.findAllByStudyEntryType(studyEntryTypes);
			Set<StudyEntryData> selectedStudyEntryData = studyEntryDataService.findAll(selectedStudyEntryDataIds);

			switch ((Operation)operationComboBox.getValue()) {
				case ADD:
					for (StudyEntry selectedStudyEntry : selectedStudyEntries) {
						selectedStudyEntry.getStudyEntryData().addAll(selectedStudyEntryData);
					}

					createStudyEntryBaseDataIfNotExists(selectedStudyEntryBaseData, studyEntryTypes);

					for (StudyEntryBaseData studyEntryBaseData : selectedStudyEntryBaseData) {
						studyEntryBaseData.getStudyEntryData().addAll(selectedStudyEntryData);
					}
					break;
				case DELETE:
					for (StudyEntry selectedStudyEntry : selectedStudyEntries) {
						selectedStudyEntry.getStudyEntryData().removeAll(selectedStudyEntryData);
					}

					for (StudyEntryBaseData studyEntryBaseData : selectedStudyEntryBaseData) {
						studyEntryBaseData.getStudyEntryData().removeAll(selectedStudyEntryData);
					}
					break;
				case OVERWRITE:
					for (StudyEntry selectedStudyEntry : selectedStudyEntries) {
						selectedStudyEntry.setStudyEntryData(new HashSet<StudyEntryData>());
						selectedStudyEntry.getStudyEntryData().addAll(selectedStudyEntryData);
					}

					createStudyEntryBaseDataIfNotExists(selectedStudyEntryBaseData, studyEntryTypes);

					for (StudyEntryBaseData studyEntryBaseData : selectedStudyEntryBaseData) {
						studyEntryBaseData.setStudyEntryData(new HashSet<StudyEntryData>());
						studyEntryBaseData.getStudyEntryData().addAll(selectedStudyEntryData);
					}
					break;
			}

			studyEntryService.save(selectedStudyEntries);
			studyEntryBaseDataService.save(selectedStudyEntryBaseData);
			Notification.show(i18n.get("AssignmentPanel.assignmentSaved", i18n.get(StudyEntryData.class.getSimpleName())), Notification.Type.TRAY_NOTIFICATION);
			eventBus.publish(this, new EventBusAction<>(Action.ASSIGNMENT, StudyEntryData.class));
		}
		Collection<Window> windows = UI.getCurrent().getWindows();
		for (Window window : windows) {
			UI.getCurrent().removeWindow(window);
		}
	}

	private void createStudyEntryBaseDataIfNotExists(Set<StudyEntryBaseData> selectedStudyEntryBaseData, StudyEntryType[] studyEntryTypes) {
		if (selectedStudyEntryBaseData.size() != studyEntryTypes.length) {
			for (StudyEntryType selectedStudyEntryType : studyEntryTypes) {
				boolean isStudyEntryTypeNotIn = selectedStudyEntryBaseData.isEmpty();
				for (StudyEntryBaseData selectedStudyEntryBaseData1 : selectedStudyEntryBaseData) {
					if (selectedStudyEntryType.equals(selectedStudyEntryBaseData1.getStudyEntryType())) {
						isStudyEntryTypeNotIn = true;
						break;
					}
				}

				if (isStudyEntryTypeNotIn) {
					StudyEntryBaseData studyEntryBaseData = new StudyEntryBaseData();
					studyEntryBaseData.setCustomer(UserUtil.getCustomer());
					studyEntryBaseData.setStudyEntryType(selectedStudyEntryType);
					studyEntryBaseData.setStudyEntryData(new HashSet<StudyEntryData>());
					selectedStudyEntryBaseData.add(studyEntryBaseData);
				}
			}
		}
	}
}
