package hu.oroscom.c1801.component.form;

import com.vaadin.data.Container;
import com.vaadin.data.Validator;
import com.vaadin.data.fieldgroup.BeanFieldGroup;
import com.vaadin.data.fieldgroup.FieldGroup;
import com.vaadin.data.fieldgroup.PropertyId;
import com.vaadin.data.util.BeanItemContainer;
import com.vaadin.ui.ComboBox;
import com.vaadin.ui.Label;
import com.vaadin.ui.PasswordField;
import com.vaadin.ui.TextArea;
import hu.oroscom.c1801.component.combobox.VenueComboBox;
import hu.oroscom.c1801.domain.Assistant;
import hu.oroscom.c1801.domain.User;
import hu.oroscom.c1801.enumeration.EntityStatus;
import hu.oroscom.c1801.enumeration.EntryPoint;
import hu.oroscom.c1801.enumeration.RoleName;
import hu.oroscom.c1801.service.AssistantService;
import hu.oroscom.c1801.util.UserUtil;

/**
 * Created by Norbert on 2014.10.15..
 */
public class AssistantForm extends BaseForm<Assistant, AssistantService> {
	private Assistant assistant;

	@PropertyId("status")
	private ComboBox entityStatusComboBox;

	@PropertyId("venue")
	private VenueComboBox venueComboBox;

	private PasswordField passwordField;
	private PasswordField confirmPasswordField;

	public AssistantForm(Assistant entity, Container... container) {
		super(entity, container);
	}

	public AssistantForm(Assistant entity, boolean isWindowedForm, Container... container) {
		super(entity, isWindowedForm, container);
	}

	@Override
	public void initMemberFields(final BeanFieldGroup<Assistant> beanFieldGroup, Container... container) {
		assistant = beanFieldGroup.getItemDataSource().getBean();
		User sessionUser = UserUtil.getUser();

		formLayout.addComponent(beanFieldGroup.buildAndBind(i18n.get("User.firstName"), "firstName"));
		formLayout.addComponent(beanFieldGroup.buildAndBind(i18n.get("User.lastName"), "lastName"));
		formLayout.addComponent(beanFieldGroup.buildAndBind(i18n.get("User.userName"), "userName"));
		formLayout.addComponent(beanFieldGroup.buildAndBind(i18n.get("User.shortName"), "shortName"));
		formLayout.addComponent(beanFieldGroup.buildAndBind(i18n.get("User.company"), "company"));

		venueComboBox = new VenueComboBox(i18n.get("UserForm.venue"));
		formLayout.addComponent(venueComboBox);

		final BeanItemContainer<EntityStatus> entityStatusBeanItemContainer = new BeanItemContainer<>(EntityStatus.class);
		for (EntityStatus entityStatus : EntityStatus.values()) {
			if (entityStatus != EntityStatus.DELETED) {
				entityStatusBeanItemContainer.addBean(entityStatus);
			}
		}
		entityStatusComboBox = new ComboBox(i18n.get("Service.status"), entityStatusBeanItemContainer);
		entityStatusComboBox.setItemCaptionPropertyId("caption");
		entityStatusComboBox.setRequired(true);
		entityStatusComboBox.setNullSelectionAllowed(false);
		formLayout.addComponent(entityStatusComboBox);

		formLayout.addComponent(beanFieldGroup.buildAndBind(i18n.get("User.shortDescription"), "description"));
		formLayout.addComponent(beanFieldGroup.buildAndBind(i18n.get("User.description"), "shortDescription", TextArea.class));

		Label changePasswordLabel = new Label(i18n.get("UserForm.changePasswordLabel"));
		formLayout.addComponent(changePasswordLabel);

		passwordField = new PasswordField(i18n.get("User.password"));
		passwordField.setRequired(true);
		passwordField.setNullRepresentation("");
		formLayout.addComponent(passwordField);

		confirmPasswordField = new PasswordField(i18n.get("UserForm.confirmPasswordField"));
		formLayout.addComponent(confirmPasswordField);
		confirmPasswordField.addValidator(new Validator() {
			@Override
			public void validate(Object value) throws InvalidValueException {
				String password = (String) value;
				if (password != null && !password.equals(passwordField.getValue())) {
					throw new InvalidValueException(i18n.get("UserForm.confirmPasswordFieldDoesntMatchErrorMessage"));
				}
			}
		});
		beanFieldGroup.bind(confirmPasswordField, "password");
		passwordField.setValue(confirmPasswordField.getValue());

		beanFieldGroup.bindMemberFields(this);

		beanFieldGroup.addCommitHandler(new FieldGroup.CommitHandler() {
			@Override
			public void preCommit(FieldGroup.CommitEvent commitEvent) throws FieldGroup.CommitException {
				Assistant assistant = beanFieldGroup.getItemDataSource().getBean();
				assistant.setRole(service.findRoleByRoleName(RoleName.ROLE_ASSISTANT));
				assistant.setEntryPoint(EntryPoint.DASHBOARD_VIEW);
			}

			@Override
			public void postCommit(FieldGroup.CommitEvent commitEvent) throws FieldGroup.CommitException {
				Assistant assistant = beanFieldGroup.getItemDataSource().getBean();
				if (assistant.getPassword().length() < 128) {
					assistant.setPassword(UserUtil.encryptPassword(assistant.getPassword()));
					passwordField.setValue(assistant.getPassword());
					confirmPasswordField.setValue(assistant.getPassword());
				}
				assistant.setCustomer(UserUtil.getCustomer());
			}
		});
	}
}
