package hu.oroscom.c1801.component.form;

import com.vaadin.data.Container;
import com.vaadin.data.fieldgroup.BeanFieldGroup;
import com.vaadin.data.fieldgroup.FieldGroup;
import com.vaadin.server.Page;
import com.vaadin.shared.ui.MarginInfo;
import com.vaadin.shared.ui.label.ContentMode;
import com.vaadin.ui.*;
import com.vaadin.ui.themes.ValoTheme;
import hu.oroscom.c1801.domain.BaseEntity;
import hu.oroscom.c1801.domain.EntityBase;
import hu.oroscom.c1801.domain.User;
import hu.oroscom.c1801.enumeration.Action;
import hu.oroscom.c1801.enumeration.FormMode;
import hu.oroscom.c1801.service.BaseService;
import hu.oroscom.c1801.util.ErrorUtils;
import hu.oroscom.c1801.util.EventBusAction;
import org.vaadin.dialogs.ConfirmDialog;

import java.util.List;

/**
 * Created by Norbert on 2014.10.17..
 */
public abstract class BaseForm<T extends EntityBase, E extends BaseService> extends FormBase<T, E> implements Button.ClickListener {
	private T entity;

	protected VerticalLayout formVerticalLayout;

	protected FormLayout formLayout;

	protected Button saveButton, cancelButton, deleteButton;

	protected HorizontalLayout buttonBarHorizontalLayout;

	protected BaseForm(T entity) {
		super(entity);
	}

	protected BaseForm(T entity, boolean isWindowedForm) {
		super(entity, isWindowedForm);
	}

	protected BaseForm(T entity, Container... container) {
		super(entity, container);
	}

	protected BaseForm(T entity, boolean isWindowedForm, Container... container) {
		super(entity, isWindowedForm, container);
	}

	protected BaseForm(List<T> entities) {
		super(entities);
	}

	protected BaseForm(List<T> entities, boolean isWindowedForm) {
		super(entities, isWindowedForm);
	}

	protected BaseForm(List<T> entities, Container... container) {
		super(entities, container);
	}

	@Override
	protected void init(List<T> entities, boolean isWindowedForm, Container... container) {
		//TODO: ezt a részt majd szebben!
		if (entities.size() > 0) {
			initGlobalVariables(entities.get(0));
		}

		configureWindow();

		formLayout = new FormLayout();

		if (isWindowedForm) {
			formLayout.setStyleName("windowed-form");
			formLayout.setMargin(new MarginInfo(false, true, false, false));
		}

		for (T entity : entities) {
			BeanFieldGroup<T> tBeanFieldGroup = new BeanFieldGroup<>((Class<T>) entity.getClass());
			tBeanFieldGroup.setItemDataSource(entity);
			tBeanFieldGroupList.add(tBeanFieldGroup);
			if (entity instanceof BaseEntity && ((BaseEntity) entity).getId() != null) {
				formMode = FormMode.EDIT;
			}
		}

		formVerticalLayout = new VerticalLayout(formLayout);
		formVerticalLayout.setMargin(true);

		this.setContent(formVerticalLayout);
//		setCompositionRoot(formLayout);
		for (BeanFieldGroup<T> beanFieldGroup : tBeanFieldGroupList) {
			initMemberFields(beanFieldGroup, container);
		}

		buttonBarHorizontalLayout = new HorizontalLayout();
		buttonBarHorizontalLayout.setSpacing(true);
//		buttonBarHorizontalLayout.setMargin(true);
		saveButton = new Button(i18n.get("BaseForm.saveButton"));
		saveButton.addClickListener(this);
		cancelButton = new Button(i18n.get("BaseForm.cancelButton"));
		cancelButton.addClickListener(this);
		buttonBarHorizontalLayout.addComponent(saveButton);
		if (isWindowedForm) {
			buttonBarHorizontalLayout.addComponent(cancelButton);
		}

		if (formMode == FormMode.EDIT && isWindowedForm) {
			deleteButton = new Button(i18n.get("BaseForm.deleteButton"));
			deleteButton.addClickListener(this);
			buttonBarHorizontalLayout.addComponent(deleteButton);
		}
		formVerticalLayout.addComponent(buttonBarHorizontalLayout);

		boolean isFirst = true;
		for (BeanFieldGroup<T> beanFieldGroup : tBeanFieldGroupList) {
			postManageFields(beanFieldGroup, isFirst);
			isFirst = false;
			beforeShow(beanFieldGroup.getItemDataSource().getBean());
		}
	}

	protected Component generateHistoryData(String changeLog) {
		if (changeLog != null) {
			changeLog = changeLog.replace("\n", "<br>");
		}
		Label label = new Label(changeLog, ContentMode.HTML);
		label.addStyleName(ValoTheme.TEXTAREA_TINY);

		Panel panel = new Panel();
		panel.setCaption(i18n.get("BaseForm.change"));
		panel.addStyleName("historyData");
		panel.setScrollTop(Integer.MAX_VALUE);
		panel.setWidth(100, Unit.PERCENTAGE);
		panel.setContent(label);
		return panel;
	}

	private boolean saveButtonClick() {
		boolean isValid = true;
		for (BeanFieldGroup<T> tBeanFieldGroup : tBeanFieldGroupList) {
			try {
				beforeSave(tBeanFieldGroup.getItemDataSource().getBean());
				tBeanFieldGroup.commit();
				//beforeSave(tBeanFieldGroup.getItemDataSource().getBean());
				entity = (T) service.save(tBeanFieldGroup.getItemDataSource().getBean());
				tBeanFieldGroup.setItemDataSource(entity);
			} catch (FieldGroup.CommitException e) {
				if (!e.getMessage().isEmpty() && !"Commit failed".equals(e.getMessage())) {
					Notification notification = new Notification(i18n.get("BaseForm.saveFailedErrorMessage"), e.getMessage(), Notification.Type.ERROR_MESSAGE, true);
					notification.show(Page.getCurrent());
				} else {
					ErrorUtils.showComponentErrors(i18n.get("BaseForm.saveFailedErrorMessage"), tBeanFieldGroup.getFields());
				}
				isValid = false;
				break;
			}
		}

		return isValid;
	}

	private void cancelButtonClick() {
		for (BeanFieldGroup<T> tBeanFieldGroup : tBeanFieldGroupList) {
			tBeanFieldGroup.discard();
		}
	}

	private void deleteButtonClick() {
		ConfirmDialog.show(UI.getCurrent(), i18n.get("RemoveCell.confirmationMessage"), i18n.get("RemoveCell.areYouSureMessage"),
				i18n.get("BaseForm.yesButton"), i18n.get("BaseForm.noButton"), new ConfirmDialog.Listener() {

					public void onClose(ConfirmDialog dialog) {
						if (dialog.isConfirmed()) {
							for (BeanFieldGroup<T> tBeanFieldGroup : tBeanFieldGroupList) {
								tBeanFieldGroup.discard();
								try {
									entity = tBeanFieldGroup.getItemDataSource().getBean();
									service.delete(entity);
								} catch (Exception e) {
									String msg;
									if (entity instanceof User) {
										msg = i18n.get("RemoveCell.integrityConstraintViolationException");
									} else {
										msg = i18n.get("BaseForm.integrityConstraintViolationExceptionMessage");
									}
									Notification notification = new Notification(null, msg, Notification.Type.ERROR_MESSAGE, true);
									notification.show(Page.getCurrent());
								}
							}
							eventBus.publish(this, new EventBusAction<>(Action.DELETE, entity));
						}
					}
				});
	}

	@Override
	public void buttonClick(Button.ClickEvent clickEvent) {
		if (clickEvent.getSource() == saveButton) {
			if (saveButtonClick()) {
				Notification.show(i18n.get("BaseForm.savedMessage", i18n.get(aClass.getSimpleName())), Notification.Type.TRAY_NOTIFICATION);
				for (Window window : UI.getCurrent().getWindows()) {
					if (Boolean.TRUE.equals(window.getData()) && window.isVisible()) {
						window.setData(entity);
					}
				}
				eventBus.publish(this, new EventBusAction<>(Action.SAVE, entity));
			}
		} else if (clickEvent.getSource() == cancelButton) {
			cancelButtonClick();
			eventBus.publish(this, new EventBusAction<>(Action.CANCEL));
		} else {
			deleteButtonClick();
//			eventBus.publish(this, new EventBusAction<>(Action.DELETE, entity));
		}
	}
}
