package hu.oroscom.c1801.component.form;

import com.vaadin.data.Container;
import com.vaadin.data.fieldgroup.BeanFieldGroup;
import com.vaadin.data.fieldgroup.FieldGroup;
import com.vaadin.data.util.converter.Converter;
import com.vaadin.server.*;
import com.vaadin.shared.ui.datefield.Resolution;
import com.vaadin.ui.*;
import com.vaadin.ui.themes.ValoTheme;
import hu.oroscom.c1801.component.datefield.JodaDateField;
import hu.oroscom.c1801.converter.DateToJodaDateConverter;
import hu.oroscom.c1801.domain.EatenFood;
import hu.oroscom.c1801.service.EatenFoodService;

import java.io.*;
import java.nio.file.Files;
import java.util.Date;

public class EatenFoodForm extends BaseForm<EatenFood, EatenFoodService> {
	private EatenFood eatenFood;
	private byte[] fileBytes;
	private String fileName;
	private String mimeType;
	private Button fileButton;
	private FileDownloader fileDownloader;

	public EatenFoodForm(EatenFood entity, boolean isWindowedForm, Container... container) {
		super(entity, isWindowedForm, container);
	}

	@Override
	public void initMemberFields(final BeanFieldGroup<EatenFood> beanFieldGroup, Container... container) {
		eatenFood = beanFieldGroup.getItemDataSource().getBean();
		JodaDateField dateTimeJodaDateField = new JodaDateField(i18n.get("EatenFood.dateTime")) {
			@Override
			protected Date handleUnparsableDateString(String dateString) throws Converter.ConversionException {
				Notification.show(i18n.get("DateField.invalidDateFormat"), Notification.Type.WARNING_MESSAGE);
				throw new Converter.ConversionException(i18n.get("DateField.invalidDateFormat"));
			}
		};
		dateTimeJodaDateField.setResolution(Resolution.MINUTE);
		dateTimeJodaDateField.setConverter(new DateToJodaDateConverter());
		beanFieldGroup.bind(dateTimeJodaDateField, "dateTime");
		formLayout.addComponent(dateTimeJodaDateField);

		formLayout.addComponent(beanFieldGroup.buildAndBind(i18n.get("EatenFood.description"), "description", TextArea.class));

		class FileUploader implements Upload.Receiver, Upload.SucceededListener {
			public File file;

			@Override
			public OutputStream receiveUpload(String filename, String mimeType) {
				FileOutputStream fos;
				try {
					file = new File(System.getProperty("java.io.tmpdir") + File.separator + filename);
					fos = new FileOutputStream(file);
				} catch (FileNotFoundException e) {
					new Notification("Could not open file", e.getMessage(), Notification.Type.ERROR_MESSAGE).show(Page.getCurrent());
					return null;
				}
				return fos;
			}

			@Override
			public void uploadSucceeded(Upload.SucceededEvent event) {
				try {
					fileBytes = Files.readAllBytes(file.toPath());
					fileName = event.getFilename();
					mimeType = event.getMIMEType();
					fileButton.setEnabled(true);
					fileButton.setVisible(true);

					StreamResource.StreamSource streamSource = new StreamResource.StreamSource() {
						@Override
						public InputStream getStream() {
							byte[] decodedBytes = fileBytes;
							return (decodedBytes == null) ? null : new ByteArrayInputStream(decodedBytes);
						}
					};
					fileButton.setCaption(fileName);
					StreamResource resource = new StreamResource(streamSource, fileName);
					resource.setMIMEType(mimeType);
					fileDownloader.setFileDownloadResource(resource);
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}

		FileUploader receiver = new FileUploader();
		Upload upload = new Upload(null, receiver);
		upload.setButtonCaption(i18n.get("EatenFoodForm.uploadFile"));
		upload.addSucceededListener(receiver);

		fileButton = new Button(eatenFood.getAttachmentFileName());
		fileButton.addStyleName(ValoTheme.BUTTON_LINK);
		fileButton.setVisible(eatenFood.getAttachment() != null);
		fileButton.setEnabled(eatenFood.getAttachment() != null);

		StreamResource.StreamSource streamSource = new StreamResource.StreamSource() {
			@Override
			public InputStream getStream() {
				byte[] decodedBytes = eatenFood.getAttachment();
				return (decodedBytes == null) ? null : new ByteArrayInputStream(decodedBytes);
			}
		};
		StreamResource resource = new StreamResource(streamSource, eatenFood.getAttachmentFileName());
		resource.setMIMEType(eatenFood.getAttachmentMimeType());

		fileDownloader = new FileDownloader(resource);
		fileDownloader.extend(fileButton);
		formLayout.addComponent(fileButton);

		Layout panelContent = new VerticalLayout();
		panelContent.setCaption(i18n.get("EatenFoodForm.fileUpload"));
		panelContent.addComponents(upload, fileButton);
		formLayout.addComponent(panelContent);

		/* Árpa */
		String basepath = VaadinService.getCurrent().getBaseDirectory().getAbsolutePath();
		FileResource arpaResource = new FileResource(new File(basepath + "/WEB-INF/images/1-arpa.jpg"));

		Image arpaImage = new Image("Árpa");
		arpaImage.setWidth(100, Unit.PIXELS);
		arpaImage.setSource(arpaResource);

		Slider arpaSlider = new Slider();
		arpaSlider.setResolution(0);
		arpaSlider.setMin(0);
		arpaSlider.setMax(100);
		arpaSlider.setWidth(200, Unit.PIXELS);

		HorizontalLayout arpaHorizontalLayout = new HorizontalLayout(arpaImage, arpaSlider);
		arpaHorizontalLayout.setComponentAlignment(arpaImage, Alignment.MIDDLE_CENTER);
		arpaHorizontalLayout.setComponentAlignment(arpaSlider, Alignment.MIDDLE_CENTER);
		arpaHorizontalLayout.setSpacing(true);

		formLayout.addComponent(arpaHorizontalLayout);

		/* Búza */
		FileResource buzaResource = new FileResource(new File(basepath + "/WEB-INF/images/2-buza.jpg"));

		Image buzaImage = new Image("Búza");
		buzaImage.setWidth(100, Unit.PIXELS);
		buzaImage.setSource(buzaResource);

		Slider buzaSlider = new Slider();
		buzaSlider.setResolution(0);
		buzaSlider.setMin(0);
		buzaSlider.setMax(100);
		buzaSlider.setWidth(200, Unit.PIXELS);

		HorizontalLayout buzaHorizontalLayout = new HorizontalLayout(buzaImage, buzaSlider);
		buzaHorizontalLayout.setComponentAlignment(buzaImage, Alignment.MIDDLE_CENTER);
		buzaHorizontalLayout.setComponentAlignment(buzaSlider, Alignment.MIDDLE_CENTER);
		buzaHorizontalLayout.setSpacing(true);

		formLayout.addComponent(buzaHorizontalLayout);

		/* Sörélesztő */
		FileResource elesztoResource = new FileResource(new File(basepath + "/WEB-INF/images/3-soreleszto.jpg"));

		Image elesztoImage = new Image("Élesztő (sör)");
		elesztoImage.setWidth(100, Unit.PIXELS);
		elesztoImage.setSource(elesztoResource);

		Slider elesztoSlider = new Slider();
		elesztoSlider.setResolution(0);
		elesztoSlider.setMin(0);
		elesztoSlider.setMax(100);
		elesztoSlider.setWidth(200, Unit.PIXELS);

		HorizontalLayout elesztoHorizontalLayout = new HorizontalLayout(elesztoImage, elesztoSlider);
		elesztoHorizontalLayout.setComponentAlignment(elesztoImage, Alignment.MIDDLE_CENTER);
		elesztoHorizontalLayout.setComponentAlignment(elesztoSlider, Alignment.MIDDLE_CENTER);
		elesztoHorizontalLayout.setSpacing(true);

		formLayout.addComponent(elesztoHorizontalLayout);

		/* Tehéntej */
		FileResource tehentejResource = new FileResource(new File(basepath + "/WEB-INF/images/4-tehentej.jpg"));

		Image tehentejImage = new Image("Tehéntej");
		tehentejImage.setWidth(100, Unit.PIXELS);
		tehentejImage.setSource(tehentejResource);

		Slider tehentejSlider = new Slider();
		tehentejSlider.setResolution(0);
		tehentejSlider.setMin(0);
		tehentejSlider.setMax(100);
		tehentejSlider.setWidth(200, Unit.PIXELS);

		HorizontalLayout tehentejHorizontalLayout = new HorizontalLayout(tehentejImage, tehentejSlider);
		tehentejHorizontalLayout.setComponentAlignment(tehentejImage, Alignment.MIDDLE_CENTER);
		tehentejHorizontalLayout.setComponentAlignment(tehentejSlider, Alignment.MIDDLE_CENTER);
		tehentejHorizontalLayout.setSpacing(true);

		formLayout.addComponent(tehentejHorizontalLayout);

		/* Tojásfehérja */
		FileResource tojasfeherjeResource = new FileResource(new File(basepath + "/WEB-INF/images/5-tojasfeherje.jpg"));

		Image tojasfeherjeImage = new Image("Tojásfehérje");
		tojasfeherjeImage.setWidth(100, Unit.PIXELS);
		tojasfeherjeImage.setSource(tojasfeherjeResource);

		Slider tojasfeherjeSlider = new Slider();
		tojasfeherjeSlider.setResolution(0);
		tojasfeherjeSlider.setMin(0);
		tojasfeherjeSlider.setMax(100);
		tojasfeherjeSlider.setWidth(200, Unit.PIXELS);

		HorizontalLayout tojasfeherjeHorizontalLayout = new HorizontalLayout(tojasfeherjeImage, tojasfeherjeSlider);
		tojasfeherjeHorizontalLayout.setComponentAlignment(tojasfeherjeImage, Alignment.MIDDLE_CENTER);
		tojasfeherjeHorizontalLayout.setComponentAlignment(tojasfeherjeSlider, Alignment.MIDDLE_CENTER);
		tojasfeherjeHorizontalLayout.setSpacing(true);

		formLayout.addComponent(tojasfeherjeHorizontalLayout);

		beanFieldGroup.addCommitHandler(new FieldGroup.CommitHandler() {
			@Override
			public void preCommit(FieldGroup.CommitEvent commitEvent) throws FieldGroup.CommitException {

			}

			@Override
			public void postCommit(FieldGroup.CommitEvent commitEvent) throws FieldGroup.CommitException {
				if (fileBytes != null) {
					eatenFood.setAttachment(fileBytes);
					eatenFood.setAttachmentFileName(fileName);
					eatenFood.setAttachmentMimeType(mimeType);
				}
			}
		});
	}
}
