package hu.oroscom.c1801.component.form;

import com.vaadin.data.Container;
import com.vaadin.data.fieldgroup.BeanFieldGroup;
import com.vaadin.data.fieldgroup.FieldGroup;
import com.vaadin.server.VaadinSession;
import com.vaadin.ui.*;
import hu.oroscom.c1801.Application;
import hu.oroscom.c1801.domain.BaseEntity;
import hu.oroscom.c1801.domain.EntityBase;
import hu.oroscom.c1801.enumeration.FormMode;
import hu.oroscom.c1801.service.BaseService;
import org.hibernate.validator.constraints.NotEmpty;
import org.joda.time.format.DateTimeFormat;
import org.joda.time.format.DateTimeFormatter;
import org.vaadin.spring.events.EventBus;
import org.vaadin.spring.i18n.I18N;

import javax.validation.constraints.NotNull;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

/**
 * Created by kollerk on 14. 11. 28..
 */
public abstract class FormBase<T extends EntityBase, E extends BaseService> extends Panel {

	protected static DateTimeFormatter dateTimeFormatter;
	protected static DateTimeFormatter timeFormatter;
	protected static NumberFormat numberFormat;
	protected static DecimalFormat decimalFormat;
	protected final List<BeanFieldGroup<T>> tBeanFieldGroupList = new ArrayList<>();
	protected I18N i18n = (I18N) Application.getApplicationContext().getBean("i18n");
	protected Class<?> aClass;
	protected EventBus eventBus = (EventBus) Application.getApplicationContext().getBean("uiEventBus");
	protected E service;
	protected FormMode formMode = FormMode.NEW;
	protected Window window = new Window();

	protected FormBase(T entity) {
		if (entity != null) {
			List<T> entities = new ArrayList<>();
			entities.add(entity);
			init(entities, false, null);
		}
	}

	protected FormBase(T entity, boolean isWindowedForm) {
		if (entity != null) {
			List<T> entities = new ArrayList<>();
			entities.add(entity);
			init(entities, isWindowedForm, null);
		}
	}

	protected FormBase(T entity, Container... container) {
		if (entity != null) {
			List<T> entities = new ArrayList<>();
			entities.add(entity);
			init(entities, false, container);
		}
	}

	protected FormBase(T entity, boolean isWindowedForm, Container... container) {
		if (entity != null) {
			List<T> entities = new ArrayList<>();
			entities.add(entity);
			init(entities, isWindowedForm, container);
		}
	}

	protected FormBase(List<T> entities) {
		this(entities, false, null);
	}

	protected FormBase(List<T> entities, boolean isWindowedForm) {
		this(entities, isWindowedForm, null);
	}

	protected FormBase(List<T> entities, Container... container) {
		this(entities, false, container);
	}

	protected FormBase(List<T> entities, boolean isWindowedForm, Container... container) {
		if (!entities.isEmpty()) {
			init(entities, isWindowedForm, container);
		}
	}

	protected void beforeShow(T entity) {
	}

	//TODO ez kiváltható saját CommitHandler -el
	protected void beforeSave(T entity) throws FieldGroup.CommitException {
	}

	protected void init(List<T> entities, boolean isWindowedForm, Container... container) {
		dateTimeFormatter = DateTimeFormat.shortDateTime().withLocale(VaadinSession.getCurrent().getLocale());
		timeFormatter = DateTimeFormat.shortTime().withLocale(VaadinSession.getCurrent().getLocale());
		numberFormat = NumberFormat.getNumberInstance(VaadinSession.getCurrent().getLocale());
		decimalFormat = (DecimalFormat) numberFormat;
		if (entities.size() > 0) {
			initGlobalVariables(entities.get(0));
		}

		configureWindow();

		for (T entity : entities) {
			BeanFieldGroup<T> tBeanFieldGroup = new BeanFieldGroup<>((Class<T>) entity.getClass());
			tBeanFieldGroup.setItemDataSource(entity);
			tBeanFieldGroupList.add(tBeanFieldGroup);
			if (entity instanceof BaseEntity && ((BaseEntity) entity).getId() != null) {
				formMode = FormMode.EDIT;
			}
		}

		beforeInitMemberFields();

		for (BeanFieldGroup<T> beanFieldGroup : tBeanFieldGroupList) {
			initMemberFields(beanFieldGroup, container);
		}

		boolean isFirst = true;
		for (BeanFieldGroup<T> beanFieldGroup : tBeanFieldGroupList) {
			postManageFields(beanFieldGroup, isFirst);
			isFirst = false;
			beforeShow(beanFieldGroup.getItemDataSource().getBean());
		}
	}

	protected void beforeInitMemberFields() {
	}

	protected void initGlobalVariables(T entity) {
		aClass = entity.getClass();
		String entityName = aClass.getSimpleName().substring(0, 1).toLowerCase() + aClass.getSimpleName().substring(1);
		this.service = (E) Application.getApplicationContext().getBean(entityName + "Service");
	}

	private java.lang.reflect.Field getDeclaredField(Class<?> aClass, String propertyId) throws ClassNotFoundException, NoSuchFieldException {
		if (propertyId.indexOf(".") > -1) {
			String embeddedClassName = "hu.oroscom.c1801.domain." + propertyId.substring(0, 1).toUpperCase() + propertyId.substring(1, propertyId.indexOf("."));
			Class<?> embeddedClass = Class.forName(embeddedClassName);
			return getDeclaredField(embeddedClass, propertyId.substring(propertyId.indexOf(".") + 1));
		}
		Class<?> superClass = aClass.getSuperclass();
		if (superClass != null) {
			try {
				return getDeclaredField(superClass, propertyId);
			} catch (NoSuchFieldException e) {
				return aClass.getDeclaredField(propertyId);
			}
		}
		return aClass.getDeclaredField(propertyId);
	}

	protected void postManageFields(BeanFieldGroup<T> tBeanFieldGroup, boolean needFocusing) {
		int i = 1;
		for (Field<?> field : tBeanFieldGroup.getFields()) {
			if (field instanceof TextField) {
				((TextField) field).setNullRepresentation("");
				((TextField) field).setImmediate(false);
			} else if (field instanceof TextArea) {
				((TextArea) field).setNullRepresentation("");
				((TextArea) field).setImmediate(false);
			} else if (field instanceof PasswordField) {
				((PasswordField) field).setNullRepresentation("");
				((PasswordField) field).setImmediate(false);
			}
			if (needFocusing && i++ == 1) {
				field.focus();
			}

			try {
				String propertyId = tBeanFieldGroup.getPropertyId(field).toString();
				java.lang.reflect.Field entityField = this.getDeclaredField(aClass, propertyId);
				if (entityField.getAnnotation(NotNull.class) != null || entityField.getAnnotation(NotEmpty.class) != null) {
					field.setRequired(true);
					field.setRequiredError(field.getCaption() + ": " + i18n.get("FormBase.requiredMessage"));
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	public abstract void initMemberFields(BeanFieldGroup<T> beanFieldGroup, Container... container);

	public void resizeWindow() {
		Collection<Window> windows = UI.getCurrent().getWindows();
		for (Window window : windows) {
			window.center();
		}
	}

	protected void configureWindow() {
		int browserWindowWidth = UI.getCurrent().getPage().getBrowserWindowWidth();
		int browserWindowHeight = UI.getCurrent().getPage().getBrowserWindowHeight();

		window.setModal(true);
		window.center();
		window.setClosable(true);
		window.setResizable(false);
		window.setWidth(browserWindowWidth < 1000 ? browserWindowWidth : 1000, Unit.PIXELS);

//		Page.Styles styles = Page.getCurrent().getStyles();
//		styles.add(".v-app .v-window .v-panel {max-height:" + browserWindowHeight + "px;border:none;box-shadow:none;}");
	}

	protected void addContentToWindow(Component component) {
		window.setContent(component);
		if (!window.isAttached()) {
			UI.getCurrent().addWindow(window);
		}
		window.center();
	}

	protected void removeWindow(Window window) {
		if (UI.getCurrent().getWindows().contains(window)) {
			UI.getCurrent().removeWindow(window);
		}
	}

	protected void removeWindow() {
		for (Window window : UI.getCurrent().getWindows()) {
			UI.getCurrent().removeWindow(window);
		}
	}

	public FormMode getFormMode() {
		return formMode;
	}

	public void setFormMode(FormMode formMode) {
		this.formMode = formMode;
	}
}
