package hu.oroscom.c1801.component.form;

import com.vaadin.ui.*;
import com.vaadin.ui.themes.ValoTheme;
import hu.oroscom.c1801.Application;
import hu.oroscom.c1801.domain.Doctor;
import hu.oroscom.c1801.domain.Patient;
import hu.oroscom.c1801.domain.StudyEntryData;
import hu.oroscom.c1801.enumeration.Action;
import hu.oroscom.c1801.enumeration.Operation;
import hu.oroscom.c1801.service.DoctorService;
import hu.oroscom.c1801.service.PatientService;
import hu.oroscom.c1801.util.EventBusAction;
import org.vaadin.spring.events.EventBus;
import org.vaadin.spring.i18n.I18N;

import java.util.Collection;
import java.util.Set;

/**
 * Created by Norbert on 2015.06.24..
 */
public class PatientAssignmentPanel extends VerticalLayout implements Button.ClickListener {

	private I18N i18n = (I18N) Application.getApplicationContext().getBean("i18n");
	private EventBus eventBus = (EventBus) Application.getApplicationContext().getBean("uiEventBus");

	private Button applyButton;
	private Button cancelButton;
	private ComboBox operationComboBox;
	private OptionGroup doctorOptionGroup;

	private Long[] selectedPatientIds;
	private DoctorService doctorService;
	private PatientService patientService;

	public PatientAssignmentPanel(Set<Object> selectedPatientIds, PatientService patientService, DoctorService doctorService) {
		this.selectedPatientIds = selectedPatientIds.toArray(new Long[selectedPatientIds.size()]);
		this.doctorService = doctorService;
		this.patientService = patientService;

		setMargin(true);
		addStyleName("manage-targeting");

		applyButton = new Button(i18n.get("AssignmentPanel.applyButton"), this);
		cancelButton = new Button(i18n.get("AssignmentPanel.cancelButton"), this);

		Label assignmentHeadingLabel = new Label(i18n.get("AssignmentPanel.assignmentHeadingLabel", i18n.get(StudyEntryData.class.getSimpleName())));
		assignmentHeadingLabel.addStyleName("manage-targeting-heading");
		assignmentHeadingLabel.addStyleName(ValoTheme.LABEL_H2);

		Label selectedStudyEntryDataLabel = new Label(i18n.get("PatientAssignmentPanel.selectedPatientLabel", selectedPatientIds.size()));

		doctorOptionGroup = new OptionGroup(i18n.get("AssignmentPanel.valueOptionGroup"));
		doctorOptionGroup.setMultiSelect(false);
		for (Doctor doctor : doctorService.findAllByCustomer()) {
			doctorOptionGroup.addItem(doctor);
			doctorOptionGroup.setItemCaption(doctor, doctor.getDisplayName());
		}

		operationComboBox = new ComboBox(i18n.get("AssignmentPanel.operationComboBox"));
		operationComboBox.addStyleName(ValoTheme.COMBOBOX_SMALL);
		operationComboBox.setTextInputAllowed(false);
		operationComboBox.setNullSelectionAllowed(false);
		operationComboBox.addItem(Operation.DELETE);
		operationComboBox.setItemCaption(Operation.DELETE, Operation.DELETE.getCaption());
		operationComboBox.addItem(Operation.OVERWRITE);
		operationComboBox.setItemCaption(Operation.OVERWRITE, Operation.OVERWRITE.getCaption());
		operationComboBox.setValue(Operation.OVERWRITE);

		HorizontalLayout buttonBarHorizontalLayout = new HorizontalLayout();
		buttonBarHorizontalLayout.addStyleName("margin-top");
		buttonBarHorizontalLayout.setSpacing(true);
		buttonBarHorizontalLayout.setWidth(100, Unit.PERCENTAGE);
		Label spacer = new Label();
		buttonBarHorizontalLayout.addComponent(spacer);
		buttonBarHorizontalLayout.addComponent(applyButton);
		buttonBarHorizontalLayout.addComponent(cancelButton);
		buttonBarHorizontalLayout.setExpandRatio(spacer, 1);

		addComponent(assignmentHeadingLabel);
		addComponent(selectedStudyEntryDataLabel);
		addComponent(new FormLayout(doctorOptionGroup, operationComboBox));
		addComponent(buttonBarHorizontalLayout);
	}

	@Override
	public void buttonClick(Button.ClickEvent event) {
		if (event.getButton().equals(applyButton)) {
			Doctor selectedDoctor = (Doctor) doctorOptionGroup.getValue();
			if (selectedDoctor == null) {
				Notification.show(i18n.get("AssignmentPanel.noEntitySelected"), Notification.Type.ERROR_MESSAGE);
				return;
			}
			Set<Patient> selectedPatients = patientService.findAll(selectedPatientIds);

			switch ((Operation)operationComboBox.getValue()) {
				case DELETE:
					for (Patient patient : selectedPatients) {
						if (selectedDoctor.equals(patient.getDoctor())) {
							patient.setDoctor(null);
						}
					}
					selectedDoctor.getPatients().removeAll(selectedPatients);
					break;
				case OVERWRITE:
					for (Patient patient : selectedPatients) {
						patient.setDoctor(selectedDoctor);
					}
					break;
			}

			patientService.save(selectedPatients);
			Notification.show(i18n.get("AssignmentPanel.assignmentSaved", i18n.get(Doctor.class.getSimpleName())), Notification.Type.TRAY_NOTIFICATION);
			eventBus.publish(this, new EventBusAction<>(Action.ASSIGNMENT, Doctor.class));
		}
		Collection<Window> windows = UI.getCurrent().getWindows();
		for (Window window : windows) {
			UI.getCurrent().removeWindow(window);
		}
	}
}
