package hu.oroscom.c1801.component.form;

import com.vaadin.data.Container;
import com.vaadin.data.Validator;
import com.vaadin.data.fieldgroup.BeanFieldGroup;
import com.vaadin.data.fieldgroup.FieldGroup;
import com.vaadin.data.fieldgroup.PropertyId;
import com.vaadin.data.util.BeanItemContainer;
import com.vaadin.ui.ComboBox;
import com.vaadin.ui.Label;
import com.vaadin.ui.PasswordField;
import com.vaadin.ui.TextArea;
import hu.oroscom.c1801.component.combobox.GenderComboBox;
import hu.oroscom.c1801.component.combobox.PatientStatusComboBox;
import hu.oroscom.c1801.component.combobox.VenueComboBox;
import hu.oroscom.c1801.component.combobox.YesNoComboBox;
import hu.oroscom.c1801.converter.StringToLongConverter;
import hu.oroscom.c1801.domain.Patient;
import hu.oroscom.c1801.domain.User;
import hu.oroscom.c1801.enumeration.EntityStatus;
import hu.oroscom.c1801.enumeration.EntryPoint;
import hu.oroscom.c1801.enumeration.RoleName;
import hu.oroscom.c1801.service.PatientService;
import hu.oroscom.c1801.util.UserUtil;
import hu.oroscom.c1801.widgetset.NumberField;
import org.vaadin.tokenfield.TokenField;

public class PatientForm extends BaseForm<Patient, PatientService> {
	private Patient patient;

	@PropertyId("status")
	private ComboBox entityStatusComboBox;

	@PropertyId("venue")
	private VenueComboBox venueComboBox;

	@PropertyId("patientStatus")
	private PatientStatusComboBox patientStatusComboBox;

	@PropertyId("gender")
	private GenderComboBox genderComboBox;

	@PropertyId("patientSelection")
	private YesNoComboBox patientSelectionComboBox;

	@PropertyId("patientExclusion")
	private YesNoComboBox patientExclusionComboBox;

	@PropertyId("patientRand")
	private YesNoComboBox patientRandComboBox;

	private String patientRandNer;

	@PropertyId("patientComplied")
	private YesNoComboBox patientCompliedComboBox;

	@PropertyId("patientRandomNr")
	private NumberField patientRandomNrNumberField;

	private PasswordField passwordField;
	private PasswordField confirmPasswordField;

	public PatientForm(Patient entity, Container... container) {
		super(entity, container);
	}

	public PatientForm(Patient entity, boolean isWindowedForm, Container... container) {
		super(entity, isWindowedForm, container);
	}

	@Override
	public void initMemberFields(final BeanFieldGroup<Patient> beanFieldGroup, Container... container) {
		patient = beanFieldGroup.getItemDataSource().getBean();
		User sessionUser = UserUtil.getUser();

//		formLayout.addComponent(beanFieldGroup.buildAndBind(i18n.get("User.firstName"), "firstName"));
//		formLayout.addComponent(beanFieldGroup.buildAndBind(i18n.get("User.lastName"), "lastName"));
//		formLayout.addComponent(beanFieldGroup.buildAndBind(i18n.get("User.userName"), "userName"));
//		formLayout.addComponent(beanFieldGroup.buildAndBind(i18n.get("User.shortName"), "shortName"));
//		formLayout.addComponent(beanFieldGroup.buildAndBind(i18n.get("User.company"), "company"));

		patientStatusComboBox = new PatientStatusComboBox(i18n.get("PatientForm.patientStatus"));
		formLayout.addComponent(patientStatusComboBox);

		formLayout.addComponent(beanFieldGroup.buildAndBind(i18n.get("PatientForm.patientId"), "patientId"));

		patientRandomNrNumberField = new NumberField(i18n.get("PatientForm.patientRandomNr"));
		patientRandomNrNumberField.setConverter(new StringToLongConverter());
		formLayout.addComponent(patientRandomNrNumberField);

		formLayout.addComponent(beanFieldGroup.buildAndBind(i18n.get("PatientForm.patientAge"), "patientAge", NumberField.class));

		genderComboBox = new GenderComboBox(i18n.get("PatientForm.gender"));
		formLayout.addComponent(genderComboBox);

		patientSelectionComboBox = new YesNoComboBox(i18n.get("PatientForm.patientSelection"));
		formLayout.addComponent(patientSelectionComboBox);

		patientExclusionComboBox = new YesNoComboBox(i18n.get("PatientForm.patientExclusion"));
		formLayout.addComponent(patientExclusionComboBox);

		patientRandComboBox = new YesNoComboBox(i18n.get("PatientForm.patientRand"));
		formLayout.addComponent(patientRandComboBox);

		formLayout.addComponent(beanFieldGroup.buildAndBind(i18n.get("PatientForm.patientRandNr"), "patientRandNr"));

		patientCompliedComboBox = new YesNoComboBox(i18n.get("PatientForm.patientComplied"));
		formLayout.addComponent(patientCompliedComboBox);

		venueComboBox = new VenueComboBox(i18n.get("UserForm.venue"));
		formLayout.addComponent(venueComboBox);

		final BeanItemContainer<EntityStatus> entityStatusBeanItemContainer = new BeanItemContainer<>(EntityStatus.class);
		for (EntityStatus entityStatus : EntityStatus.values()) {
			if (entityStatus != EntityStatus.DELETED) {
				entityStatusBeanItemContainer.addBean(entityStatus);
			}
		}
		entityStatusComboBox = new ComboBox(i18n.get("Service.status"), entityStatusBeanItemContainer);
		entityStatusComboBox.setItemCaptionPropertyId("caption");
		entityStatusComboBox.setRequired(true);
		entityStatusComboBox.setNullSelectionAllowed(false);
		formLayout.addComponent(entityStatusComboBox);

		TokenField serviceGroupsTokenField = new TokenField(i18n.get("PatientsForm.serviceGroups"));
		serviceGroupsTokenField.setContainerDataSource(container[0]);
		serviceGroupsTokenField.setTokenCaptionPropertyId("name");
		formLayout.addComponent(serviceGroupsTokenField);
		beanFieldGroup.bind(serviceGroupsTokenField, "serviceGroups");

		formLayout.addComponent(beanFieldGroup.buildAndBind(i18n.get("User.shortDescription"), "description"));
		formLayout.addComponent(beanFieldGroup.buildAndBind(i18n.get("User.description"), "shortDescription", TextArea.class));

		Label changePasswordLabel = new Label(i18n.get("UserForm.changePasswordLabel"));
		formLayout.addComponent(changePasswordLabel);

		passwordField = new PasswordField(i18n.get("User.password"));
		passwordField.setRequired(true);
		passwordField.setNullRepresentation("");
		formLayout.addComponent(passwordField);

		confirmPasswordField = new PasswordField(i18n.get("UserForm.confirmPasswordField"));
		formLayout.addComponent(confirmPasswordField);
		confirmPasswordField.addValidator(new Validator() {
			@Override
			public void validate(Object value) throws InvalidValueException {
				String password = (String) value;
				if (password != null && !password.equals(passwordField.getValue())) {
					throw new InvalidValueException(i18n.get("UserForm.confirmPasswordFieldDoesntMatchErrorMessage"));
				}
			}
		});
		beanFieldGroup.bind(confirmPasswordField, "password");
		passwordField.setValue(confirmPasswordField.getValue());

		beanFieldGroup.bindMemberFields(this);

		beanFieldGroup.addCommitHandler(new FieldGroup.CommitHandler() {
			@Override
			public void preCommit(FieldGroup.CommitEvent commitEvent) throws FieldGroup.CommitException {
				Patient patient = beanFieldGroup.getItemDataSource().getBean();
				patient.setRole(service.findRoleByRoleName(RoleName.ROLE_PATIENT));
				patient.setEntryPoint(EntryPoint.DASHBOARD_VIEW);
			}

			@Override
			public void postCommit(FieldGroup.CommitEvent commitEvent) throws FieldGroup.CommitException {
				Patient patient = beanFieldGroup.getItemDataSource().getBean();
				if (patient.getPassword().length() < 128) {
					patient.setPassword(UserUtil.encryptPassword(patient.getPassword()));
					passwordField.setValue(patient.getPassword());
					confirmPasswordField.setValue(patient.getPassword());
				}
				patient.setCustomer(UserUtil.getCustomer());
			}
		});
	}
}
