package hu.oroscom.c1801.component.form;

import com.vaadin.ui.*;
import com.vaadin.ui.themes.ValoTheme;
import hu.oroscom.c1801.Application;
import hu.oroscom.c1801.domain.Patient;
import hu.oroscom.c1801.domain.ServiceGroup;
import hu.oroscom.c1801.domain.StudyEntryData;
import hu.oroscom.c1801.domain.User;
import hu.oroscom.c1801.enumeration.Action;
import hu.oroscom.c1801.enumeration.Operation;
import hu.oroscom.c1801.service.PatientService;
import hu.oroscom.c1801.service.ServiceGroupService;
import hu.oroscom.c1801.util.EventBusAction;
import org.vaadin.spring.events.EventBus;
import org.vaadin.spring.i18n.I18N;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

/**
 * Created by Norbert on 2015.06.24..
 */
public class ServiceGroupAssignmentPanel extends VerticalLayout implements Button.ClickListener {

	private I18N i18n = (I18N) Application.getApplicationContext().getBean("i18n");
	private EventBus eventBus = (EventBus) Application.getApplicationContext().getBean("uiEventBus");

	private Button applyButton;
	private Button cancelButton;
	private ComboBox operationComboBox;
	private OptionGroup serviceGroupOptionGroup;

	private Long[] selectedPatientIds;
	private ServiceGroupService serviceGroupService;
	private PatientService patientService;

	public ServiceGroupAssignmentPanel(Set<Object> selectedPatientIds, PatientService patientService, ServiceGroupService serviceGroupService) {
		this.selectedPatientIds = selectedPatientIds.toArray(new Long[selectedPatientIds.size()]);
		this.serviceGroupService = serviceGroupService;
		this.patientService = patientService;

		setMargin(true);
		addStyleName("manage-targeting");

		applyButton = new Button(i18n.get("AssignmentPanel.applyButton"), this);
		cancelButton = new Button(i18n.get("AssignmentPanel.cancelButton"), this);

		Label assignmentHeadingLabel = new Label(i18n.get("AssignmentPanel.assignmentHeadingLabel", i18n.get(StudyEntryData.class.getSimpleName())));
		assignmentHeadingLabel.addStyleName("manage-targeting-heading");
		assignmentHeadingLabel.addStyleName(ValoTheme.LABEL_H2);

		Label selectedStudyEntryDataLabel = new Label(i18n.get("PatientAssignmentPanel.selectedPatientLabel", selectedPatientIds.size()));

		serviceGroupOptionGroup = new OptionGroup(i18n.get("AssignmentPanel.valueOptionGroup"));
		serviceGroupOptionGroup.setMultiSelect(true);
		for (ServiceGroup serviceGroup : serviceGroupService.findAllByCustomer()) {
			serviceGroupOptionGroup.addItem(serviceGroup);
			serviceGroupOptionGroup.setItemCaption(serviceGroup, serviceGroup.getName());
		}

		operationComboBox = new ComboBox(i18n.get("AssignmentPanel.operationComboBox"));
		operationComboBox.addStyleName(ValoTheme.COMBOBOX_SMALL);
		operationComboBox.setTextInputAllowed(false);
		operationComboBox.setNullSelectionAllowed(false);
		operationComboBox.addItem(Operation.ADD);
		operationComboBox.setItemCaption(Operation.ADD, Operation.ADD.getCaption());
		operationComboBox.addItem(Operation.DELETE);
		operationComboBox.setItemCaption(Operation.DELETE, Operation.DELETE.getCaption());
		operationComboBox.addItem(Operation.OVERWRITE);
		operationComboBox.setItemCaption(Operation.OVERWRITE, Operation.OVERWRITE.getCaption());
		operationComboBox.setValue(Operation.OVERWRITE);

		HorizontalLayout buttonBarHorizontalLayout = new HorizontalLayout();
		buttonBarHorizontalLayout.addStyleName("margin-top");
		buttonBarHorizontalLayout.setSpacing(true);
		buttonBarHorizontalLayout.setWidth(100, Unit.PERCENTAGE);
		Label spacer = new Label();
		buttonBarHorizontalLayout.addComponent(spacer);
		buttonBarHorizontalLayout.addComponent(applyButton);
		buttonBarHorizontalLayout.addComponent(cancelButton);
		buttonBarHorizontalLayout.setExpandRatio(spacer, 1);

		addComponent(assignmentHeadingLabel);
		addComponent(selectedStudyEntryDataLabel);
		addComponent(new FormLayout(serviceGroupOptionGroup, operationComboBox));
		addComponent(buttonBarHorizontalLayout);
	}

	@Override
	public void buttonClick(Button.ClickEvent event) {
		if (event.getButton().equals(applyButton)) {
			Set<ServiceGroup> selectedServiceGroups = (Set<ServiceGroup>) serviceGroupOptionGroup.getValue();
			if (selectedServiceGroups.isEmpty()) {
				Notification.show(i18n.get("AssignmentPanel.noEntitySelected"), Notification.Type.ERROR_MESSAGE);
				return;
			}
			Set<Patient> selectedPatients = patientService.findAll(selectedPatientIds);

			switch ((Operation)operationComboBox.getValue()) {
				case ADD:
					for (ServiceGroup serviceGroup : selectedServiceGroups) {
						Set<User> users = serviceGroup.getUsers();
						if (users == null) {
							users = new HashSet<>();
							serviceGroup.setUsers(users);
						}
						users.addAll(selectedPatients);
					}
					break;
				case DELETE:
					for (ServiceGroup serviceGroup : selectedServiceGroups) {
						Set<User> users = serviceGroup.getUsers();
						if (users == null) {
							users = new HashSet<>();
							serviceGroup.setUsers(users);
						}
						users.removeAll(selectedPatients);
					}
					break;
				case OVERWRITE:
					for (ServiceGroup serviceGroup : selectedServiceGroups) {
						serviceGroup.setUsers(new HashSet<User>());
						serviceGroup.getUsers().addAll(selectedPatients);
					}
					break;
			}

			serviceGroupService.save(selectedServiceGroups);
			Notification.show(i18n.get("AssignmentPanel.assignmentSaved", i18n.get(ServiceGroup.class.getSimpleName())), Notification.Type.TRAY_NOTIFICATION);
			eventBus.publish(this, new EventBusAction<>(Action.ASSIGNMENT, ServiceGroup.class));
		}
		Collection<Window> windows = UI.getCurrent().getWindows();
		for (Window window : windows) {
			UI.getCurrent().removeWindow(window);
		}
	}
}
