package hu.oroscom.c1801.component.form;

import com.vaadin.data.Container;
import com.vaadin.data.Property;
import com.vaadin.data.fieldgroup.BeanFieldGroup;
import com.vaadin.data.fieldgroup.FieldGroup;
import com.vaadin.data.util.BeanItemContainer;
import com.vaadin.server.FileDownloader;
import com.vaadin.server.Page;
import com.vaadin.server.StreamResource;
import com.vaadin.server.VaadinSession;
import com.vaadin.shared.ui.combobox.FilteringMode;
import com.vaadin.ui.*;
import com.vaadin.ui.themes.ValoTheme;
import hu.oroscom.c1801.component.combobox.ServiceManagementNatureComboBox;
import hu.oroscom.c1801.component.combobox.ServiceManagementStatusComboBox;
import hu.oroscom.c1801.domain.ServiceManagement;
import hu.oroscom.c1801.domain.ServiceManagementComment;
import hu.oroscom.c1801.domain.User;
import hu.oroscom.c1801.enumeration.PermissionName;
import hu.oroscom.c1801.enumeration.StudyEntryType;
import hu.oroscom.c1801.service.ServiceManagementService;
import hu.oroscom.c1801.util.MessagesUtil;
import hu.oroscom.c1801.util.UserUtil;
import hu.oroscom.c1801.util.WindowUtil;
import org.joda.time.format.DateTimeFormat;

import java.io.*;
import java.nio.file.Files;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by Norbert on 2014.10.21..
 */
public class ServiceManagementForm extends BaseForm<ServiceManagement, ServiceManagementService> {
	private ServiceManagement serviceManagement;
	private byte[] fileBytes;
	private String fileName;
	private String mimeType;
	private Button fileButton;
	private FileDownloader fileDownloader;
	private TextArea statusComment;

	public ServiceManagementForm(ServiceManagement entity, boolean isWindowedForm, Container... containers) {
		super(entity, isWindowedForm, containers);
	}

	@Override
	public void initMemberFields(final BeanFieldGroup<ServiceManagement> beanFieldGroup, Container... container) {
		serviceManagement = beanFieldGroup.getItemDataSource().getBean();
		dateTimeFormatter = DateTimeFormat.shortDateTime().withLocale(VaadinSession.getCurrent().getLocale());
		boolean isReadOnly = !service.userHasPermission(PermissionName.PERM_SERVICE_MANAGEMENT_UPDATE_ANY);

		ComboBox serviceManagementNatureComboBox = new ServiceManagementNatureComboBox(i18n.get("ServiceManagementView.serviceManagementNature"));
		serviceManagementNatureComboBox.setReadOnly(isReadOnly);
		serviceManagementNatureComboBox.setFilteringMode(FilteringMode.CONTAINS);
		beanFieldGroup.bind(serviceManagementNatureComboBox, "serviceManagementNature");
		formLayout.addComponent(serviceManagementNatureComboBox);

		BeanItemContainer<User> userBeanItemContainer = (BeanItemContainer<User>) container[0];
		ComboBox userComboBox = new ComboBox(i18n.get("User"), userBeanItemContainer);
		userComboBox.setItemCaptionPropertyId("displayName");
		userComboBox.setNullSelectionAllowed(false);
		userComboBox.setReadOnly(isReadOnly);
		userComboBox.setFilteringMode(FilteringMode.CONTAINS);
		beanFieldGroup.bind(userComboBox, "user");
		formLayout.addComponent(userComboBox);

		ComboBox studyEntryTypeComboBox = new ComboBox(MessagesUtil.get(i18n, "StudyEntry.studyEntryType"));
		studyEntryTypeComboBox.setRequired(true);
		studyEntryTypeComboBox.setNullSelectionAllowed(false);
		studyEntryTypeComboBox.setReadOnly(isReadOnly);
		studyEntryTypeComboBox.setFilteringMode(FilteringMode.CONTAINS);
		for (StudyEntryType studyEntryType : StudyEntryType.values()) {
			studyEntryTypeComboBox.addItem(studyEntryType);
			studyEntryTypeComboBox.setItemCaption(studyEntryType, studyEntryType.getCaption());
		}
		beanFieldGroup.bind(studyEntryTypeComboBox, "studyEntryType");
		formLayout.addComponent(studyEntryTypeComboBox);

		ComboBox serviceManagementStatusComboBox = new ServiceManagementStatusComboBox(i18n.get("ServiceManagementView.serviceManagementStatus"));
		serviceManagementStatusComboBox.setReadOnly(isReadOnly);
		serviceManagementStatusComboBox.setRequired(true);
		serviceManagementStatusComboBox.setNullSelectionAllowed(false);
		serviceManagementStatusComboBox.setFilteringMode(FilteringMode.CONTAINS);
		serviceManagementStatusComboBox.addValueChangeListener(new Property.ValueChangeListener() {
			@Override
			public void valueChange(Property.ValueChangeEvent valueChangeEvent) {
				if (serviceManagement.getId() == null || statusComment == null) {
					return;
				}

				statusComment.setVisible(true);
				WindowUtil.manageWindowHeight();
			}
		});
		beanFieldGroup.bind(serviceManagementStatusComboBox, "serviceManagementStatus");
		formLayout.addComponent(serviceManagementStatusComboBox);

		statusComment = new TextArea(i18n.get("ServiceManagementForm.statusComment"));
		statusComment.setVisible(false);
		formLayout.addComponent(statusComment);

		class FileUploader implements Upload.Receiver, Upload.SucceededListener {
			public File file;

			@Override
			public OutputStream receiveUpload(String filename, String mimeType) {
				FileOutputStream fos;
				try {
					file = new File(System.getProperty("java.io.tmpdir") + File.separator + filename);
					fos = new FileOutputStream(file);
				} catch (FileNotFoundException e) {
					new Notification("Could not open file", e.getMessage(), Notification.Type.ERROR_MESSAGE).show(Page.getCurrent());
					return null;
				}
				return fos;
			}

			@Override
			public void uploadSucceeded(Upload.SucceededEvent event) {
				try {
					fileBytes = Files.readAllBytes(file.toPath());
					fileName = event.getFilename();
					mimeType = event.getMIMEType();
					fileButton.setEnabled(true);
					fileButton.setVisible(true);

					StreamResource.StreamSource streamSource = new StreamResource.StreamSource() {
						@Override
						public InputStream getStream() {
							byte[] decodedBytes = fileBytes;
							return (decodedBytes == null) ? null : new ByteArrayInputStream(decodedBytes);
						}
					};
					fileButton.setCaption(fileName);
					StreamResource resource = new StreamResource(streamSource, fileName);
					resource.setMIMEType(mimeType);
					fileDownloader.setFileDownloadResource(resource);
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}

		FileUploader receiver = new FileUploader();
		Upload upload = new Upload(null, receiver);
		upload.setButtonCaption(i18n.get("ServiceManagementForm.uploadFile"));
		upload.addSucceededListener(receiver);

		fileButton = new Button(serviceManagement.getAttachmentFileName());
		fileButton.addStyleName(ValoTheme.BUTTON_LINK);
		fileButton.setVisible(serviceManagement.getAttachment() != null);
		fileButton.setEnabled(serviceManagement.getAttachment() != null);

		StreamResource.StreamSource streamSource = new StreamResource.StreamSource() {
			@Override
			public InputStream getStream() {
				byte[] decodedBytes = serviceManagement.getAttachment();
				return (decodedBytes == null) ? null : new ByteArrayInputStream(decodedBytes);
			}
		};
		StreamResource resource = new StreamResource(streamSource, serviceManagement.getAttachmentFileName());
		resource.setMIMEType(serviceManagement.getAttachmentMimeType());

		fileDownloader = new FileDownloader(resource);
		fileDownloader.extend(fileButton);
		formLayout.addComponent(fileButton);

		Layout panelContent = new VerticalLayout();
		panelContent.setCaption(i18n.get("ServiceManagementForm.fileUpload"));
		panelContent.addComponents(upload, fileButton);
		formLayout.addComponent(panelContent);

		Component descriptionComponent = beanFieldGroup.buildAndBind(i18n.get("ServiceManagementForm.description"), "description", TextArea.class);
		descriptionComponent.setReadOnly(isReadOnly);
		formLayout.addComponent(descriptionComponent);

		beanFieldGroup.bindMemberFields(this);

		if (serviceManagement.getServiceManagementComments() != null) {
			VerticalLayout statusCommentsVerticalLayout = new VerticalLayout();
			statusCommentsVerticalLayout.setCaption(i18n.get("ServiceManagementForm.statusComments"));
			formLayout.addComponent(statusCommentsVerticalLayout);

			for (ServiceManagementComment serviceManagementComment : serviceManagement.getServiceManagementComments()) {
				Panel panel = new Panel();
				panel.setCaption(serviceManagementComment.getInsertedByUser().getDisplayName() + " - " + dateTimeFormatter.print(serviceManagementComment.getInsertedAt()));
				Label label = new Label(serviceManagementComment.getComment());
				panel.setContent(label);
				statusCommentsVerticalLayout.addComponent(panel);
			}
		}

		formLayout.addComponent(generateHistoryData(serviceManagement.getChangeLog()));

		beanFieldGroup.addCommitHandler(new FieldGroup.CommitHandler() {
			@Override
			public void preCommit(FieldGroup.CommitEvent commitEvent) throws FieldGroup.CommitException {

			}

			@Override
			public void postCommit(FieldGroup.CommitEvent commitEvent) throws FieldGroup.CommitException {
				if (fileBytes != null) {
					serviceManagement.setAttachment(fileBytes);
					serviceManagement.setAttachmentFileName(fileName);
					serviceManagement.setAttachmentMimeType(mimeType);
				}

				if (statusComment.isVisible()) {
					ServiceManagementComment serviceManagementComment = new ServiceManagementComment();
					serviceManagementComment.setComment(statusComment.getValue());
					serviceManagementComment.setInsertedByUser(UserUtil.getUser());
					serviceManagementComment.setServiceManagement(serviceManagement);

					List<ServiceManagementComment> serviceManagementComments = serviceManagement.getServiceManagementComments();
					if (serviceManagementComments == null) {
						serviceManagementComments = new ArrayList<>();
					}

					serviceManagementComments.add(serviceManagementComment);
					serviceManagement.setServiceManagementComments(serviceManagementComments);
				}

				if (serviceManagement.getCustomer() == null) {
					serviceManagement.setCustomer(UserUtil.getCustomer());
				}
			}
		});
	}
}
