package hu.oroscom.c1801.component.form;

import com.vaadin.data.Container;
import com.vaadin.data.Property;
import com.vaadin.data.fieldgroup.BeanFieldGroup;
import com.vaadin.server.FileResource;
import com.vaadin.server.Page;
import com.vaadin.server.StreamResource;
import com.vaadin.ui.*;
import hu.oroscom.c1801.component.label.FormHeadingLabel;
import hu.oroscom.c1801.domain.Setting;
import hu.oroscom.c1801.domain.User;
import hu.oroscom.c1801.enumeration.PermissionName;
import hu.oroscom.c1801.enumeration.SettingName;
import hu.oroscom.c1801.service.SettingService;
import hu.oroscom.c1801.util.UserUtil;
import org.apache.commons.codec.binary.Base64;
import org.vaadin.dialogs.ConfirmDialog;

import java.io.*;
import java.nio.file.Files;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by Norbert on 2014.10.29..
 */
public class SettingsForm extends BaseForm<Setting, SettingService> {
	private static final List<String> TIMES = new ArrayList<>();

	static {
		for (int i = 0; 120 >= i; ) {
			TIMES.add(String.valueOf(i));
			i += 15;
		}
	}
	private OptionGroup optionGroup;

	private Property.ValueChangeListener valueChangeListener;

	public SettingsForm(List<Setting> entities) {
		super(entities);
	}

	@Override
	protected void beforeShow(Setting entity) {
		boolean isEnabled = service.userHasPermission(PermissionName.PERM_SETTINGS_UPDATE);
		saveButton.setEnabled(isEnabled);
		cancelButton.setEnabled(isEnabled);
		if (deleteButton != null) {
			deleteButton.setEnabled(isEnabled);
		}
	}

	@Override
	public void initMemberFields(BeanFieldGroup<Setting> beanFieldGroup, Container... container) {
		final Setting setting = beanFieldGroup.getItemDataSource().getBean();
		boolean isReadOnly = !service.userHasPermission(PermissionName.PERM_SETTINGS_UPDATE);
		SettingName settingName = SettingName.fromString(setting.getName());
		String fieldCaption = settingName.getCaption();

		switch (settingName) {
			case CUSTOMER_LOGO:
				formLayout.addComponent(new FormHeadingLabel(i18n.get("SettingsForm.customerLogo")));
				break;
			case API_KEY:
				formLayout.addComponent(new FormHeadingLabel(i18n.get("SettingsForm.generateApiKey")));
				break;
			case RESEARCH_STATUS:
				formLayout.addComponent(new FormHeadingLabel(i18n.get("SettingsForm.researchStatus")));
				break;
			case PATIENT_DATA_IMPORT:
				formLayout.addComponent(new FormHeadingLabel(i18n.get("SettingsForm.patientDataImport")));
				break;
		}

		switch (settingName) {
			case CUSTOMER_LOGO:
				final Embedded image = new Embedded();
				final Component hiddenComponent = beanFieldGroup.buildAndBind(fieldCaption, "value");
				hiddenComponent.setVisible(false);
				if (setting.getValue() == null || setting.getValue().isEmpty()) {
					image.setVisible(false);
				} else {
					StreamResource.StreamSource streamSource = new StreamResource.StreamSource() {
						@Override
						public InputStream getStream() {
							byte[] decodedBytes = Base64.decodeBase64(setting.getValue());
							return (decodedBytes == null) ? null : new ByteArrayInputStream(decodedBytes);
						}
					};
					StreamResource resource = new StreamResource(streamSource, "logo-" + UserUtil.getCustomer().getId() + ".jpg");
					image.setSource(resource);
				}

				class ImageUploader implements Upload.Receiver, Upload.SucceededListener {
					public File file;

					@Override
					public OutputStream receiveUpload(String filename, String mimeType) {
						FileOutputStream fos;
						try {
							file = new File(System.getProperty("java.io.tmpdir") + File.separator + filename);
							fos = new FileOutputStream(file);
						} catch (FileNotFoundException e) {
							new Notification("Could not open file",
									e.getMessage(),
									Notification.Type.ERROR_MESSAGE)
									.show(Page.getCurrent());
							return null;
						}
						return fos;
					}

					@Override
					public void uploadSucceeded(Upload.SucceededEvent event) {
						image.setVisible(true);
						image.setSource(new FileResource(file));
						try {
							byte[] fileBytes = Files.readAllBytes(file.toPath());
							String encodedFile = new String(Base64.encodeBase64(fileBytes));
							((TextField) hiddenComponent).setValue(encodedFile);
						} catch (IOException e) {
							e.printStackTrace();
						}
					}
				}

				ImageUploader receiver = new ImageUploader();
				Upload upload = new Upload(fieldCaption, receiver);
				upload.setButtonCaption(i18n.get("SettingsForm.uploadLogo"));
				upload.addSucceededListener(receiver);

				Panel panel = new Panel();
				Layout panelContent = new VerticalLayout();
				panelContent.addComponents(upload, image);
				panel.setContent(panelContent);
				formLayout.addComponent(panel);

				formLayout.addComponent(hiddenComponent);
				break;
			case API_KEY:
				final TextArea apiKeyTextArea = new TextArea(i18n.get("SettingsForm.apiKey"));

				final ComboBox apiUsersComboBox = new ComboBox(i18n.get("SettingsForm.apiUser"));
				apiUsersComboBox.setNullSelectionAllowed(false);
				apiUsersComboBox.addValueChangeListener(new Property.ValueChangeListener() {
					@Override
					public void valueChange(Property.ValueChangeEvent valueChangeEvent) {
						User user = (User) valueChangeEvent.getProperty().getValue();
						apiKeyTextArea.setValue(generateApiKey(user));
					}
				});
				List<User> apiUsers = service.findAllApiUsers();
				for (User user : apiUsers) {
					apiUsersComboBox.addItem(user);
					apiUsersComboBox.setItemCaption(user, user.getDisplayName());
				}

				formLayout.addComponent(apiUsersComboBox);
				formLayout.addComponent(apiKeyTextArea);
				break;
			case RESEARCH_STATUS:
				valueChangeListener = new Property.ValueChangeListener() {

					@Override
					public void valueChange(Property.ValueChangeEvent event) {
						ConfirmDialog.show(UI.getCurrent(), "Megerősítés", "Biztos, hogy aktiválni kívánja a vizsgálatot? Ez nagyszámú automatikus üzenet kiküldésével járhat!",
								"Biztos vagyok benne!", "Nem", new ConfirmDialog.Listener() {

									/**
									 * @param dialog
									 */
									public void onClose(ConfirmDialog dialog) {
										optionGroup.removeValueChangeListener(valueChangeListener);
										optionGroup.setValue("inaktiv");
										optionGroup.addValueChangeListener(valueChangeListener);

										if (dialog.isConfirmed()) {
											String msg = "Adatintegritási anomália, a kért művelet jelenleg nem hajtható végre! Kérjük ellenőrizze a C1801-es adatsort!";
											Notification notification = new Notification(null, msg, Notification.Type.ERROR_MESSAGE, true);
											notification.show(Page.getCurrent());
										}
									}
								}
						);
					}
				};

				optionGroup = new OptionGroup(i18n.get("SettingsForm.researchStatus"));
				optionGroup.setReadOnly(isReadOnly);
				optionGroup.addItem("aktiv");
				optionGroup.addItem("inaktiv");
				optionGroup.setItemCaption("aktiv", "Aktív");
				optionGroup.setItemCaption("inaktiv", "Inaktív");
				optionGroup.setValue("inaktiv");
				optionGroup.addValueChangeListener(valueChangeListener);
				formLayout.addComponent(optionGroup);
				break;
			case PATIENT_DATA_IMPORT:
				class PatientDataUploader implements Upload.Receiver, Upload.SucceededListener {
					public File file;

					@Override
					public OutputStream receiveUpload(String filename, String mimeType) {
						FileOutputStream fos;
						try {
							file = new File(System.getProperty("java.io.tmpdir") + File.separator + filename);
							fos = new FileOutputStream(file);
						} catch (FileNotFoundException e) {
							new Notification("Az import nem végrehajtható: a kutatás státusza inaktív!",
									e.getMessage(),
									Notification.Type.ERROR_MESSAGE)
									.show(Page.getCurrent());
							return null;
						}
						return fos;
					}

					@Override
					public void uploadSucceeded(Upload.SucceededEvent event) {
						String msg = "Az import nem végrehajtható: a kutatás státusza inaktív!";
						Notification notification = new Notification(null, msg, Notification.Type.ERROR_MESSAGE, true);
						notification.show(Page.getCurrent());
					}
				}

				PatientDataUploader patientDataUploader = new PatientDataUploader();
				Upload upload2 = new Upload(fieldCaption, patientDataUploader);
				upload2.setButtonCaption("Betegadat importálása");
				upload2.addSucceededListener(patientDataUploader);

				Panel panel2 = new Panel();
				Layout panelContent2 = new VerticalLayout();
				panelContent2.addComponents(upload2);
				panel2.setContent(panelContent2);
				formLayout.addComponent(panel2);
				break;
			default:
				Component component = beanFieldGroup.buildAndBind(fieldCaption, "value");
				component.setReadOnly(isReadOnly);
				formLayout.addComponent(component);
				break;
		}
	}

	private String generateApiKey(User user) {
		return service.generateApiKey(user.getUserName(), user.getPassword(), user.getCustomer().getId());
	}
}
