package hu.oroscom.c1801.component.form;

import com.vaadin.data.Container;
import com.vaadin.data.fieldgroup.BeanFieldGroup;
import com.vaadin.data.fieldgroup.FieldGroup;
import com.vaadin.data.util.converter.Converter;
import com.vaadin.server.Page;
import com.vaadin.shared.ui.MarginInfo;
import com.vaadin.shared.ui.combobox.FilteringMode;
import com.vaadin.ui.*;
import hu.oroscom.c1801.component.combobox.StudyEntryTypeComboBox;
import hu.oroscom.c1801.component.datefield.JodaDateField;
import hu.oroscom.c1801.converter.DateToJodaDateConverter;
import hu.oroscom.c1801.domain.Patient;
import hu.oroscom.c1801.domain.StudyEntry;
import hu.oroscom.c1801.domain.StudyEntryBaseData;
import hu.oroscom.c1801.domain.StudyEntryData;
import hu.oroscom.c1801.domain.relationtable.StudyEntryStudyEntryData;
import hu.oroscom.c1801.enumeration.FormMode;
import hu.oroscom.c1801.enumeration.StudyEntryType;
import hu.oroscom.c1801.service.StudyEntryService;
import hu.oroscom.c1801.util.UserUtil;
import hu.oroscom.c1801.widgetset.NumberField;
import org.joda.time.DateTime;
import org.joda.time.format.DateTimeFormat;
import org.vaadin.dialogs.ConfirmDialog;

import java.io.*;
import java.util.*;

public class StudyEntryForm extends BaseForm<StudyEntry, StudyEntryService> {
	private StudyEntry studyEntry;
	private boolean isReadOnly = false;
	private boolean isDataEditMode = false;
	private Set<Component> studyEntryDataComponents = new HashSet<>();

	public StudyEntryForm(StudyEntry entity, boolean isWindowedForm, Container... container) {
		super(entity, isWindowedForm, container);
	}

	public StudyEntryForm(final StudyEntry entity, boolean isWindowedForm, boolean isDataEditMode, Container... container) {
		super(entity, isWindowedForm, container);
		this.isDataEditMode = isDataEditMode;

		if (formMode.equals(FormMode.EDIT) && UserUtil.isPatient()) {
			formMode = FormMode.SHOW;
		}

		if (isDataEditMode) {
			switch (entity.getStudyEntryType()) {
				case FOOD_INTOLERANCE_1:
				case FOOD_INTOLERANCE_2:
				case MANITOL_LACTULOSE_1:
				case MANITOL_LACTULOSE_2:
					class FileUploader implements Upload.Receiver, Upload.SucceededListener {
						public File file;

						@Override
						public OutputStream receiveUpload(String filename, String mimeType) {
							FileOutputStream fos;
							try {
								file = new File(System.getProperty("java.io.tmpdir") + File.separator + filename);
								fos = new FileOutputStream(file);
							} catch (FileNotFoundException e) {
								new Notification("Could not open file", e.getMessage(), Notification.Type.ERROR_MESSAGE).show(Page.getCurrent());
								return null;
							}
							return fos;
						}

						@Override
						public void uploadSucceeded(Upload.SucceededEvent event) {
							String line;
							final String csvSplitBy = ";";

							try {
								final BufferedReader bufferedReader = new BufferedReader(new FileReader(file));
								line = bufferedReader.readLine();
								Patient patient = entity.getPatient();
								patient.setDisplayName();
								if (!line.split(csvSplitBy)[1].equals(patient.getDisplayName())) {
									ConfirmDialog.show(UI.getCurrent(), i18n.get("RemoveCell.confirmationMessage"), i18n.get("ConfirmationDialog.patientNameNotEqual"),
											i18n.get("BaseForm.yesButton"), i18n.get("BaseForm.noButton"), new ConfirmDialog.Listener() {

												public void onClose(ConfirmDialog dialog) {
													if (dialog.isConfirmed()) {
														String line;
														int i = 0;
														try {
															while ((line = bufferedReader.readLine()) != null) {
																String[] row = line.split(csvSplitBy);
																String key = row[0];
																String value = row[1];

																if (i == 0) {
																	entity.setTestDateTime(DateTime.parse(value, DateTimeFormat.forPattern("yyyyMMdd")));
																} else if (i > 0) {
																	setStudyEntryDataValueByDataKey(key, value);
																}

																i++;
															}
														} catch (IOException e) {
															e.printStackTrace();
														}
													}
												}
											});
								}
							} catch (IOException e) {
								e.printStackTrace();
							}
						}
					}

					FileUploader receiver = new FileUploader();
					Upload upload = new Upload(null, receiver);
					upload.setButtonCaption(i18n.get("EatenFoodForm.uploadFile"));
					upload.setEnabled(entity.getTestDateTime() == null);
					upload.addSucceededListener(receiver);

					Layout panelContent = new VerticalLayout();
					panelContent.setCaption(i18n.get("StudyEntryForm.uploadFoodTest"));
					panelContent.addComponents(upload);
					formLayout.addComponent(panelContent);
					break;
			}


			/**
			 * Data
			 */
			FormLayout dataEditFormLayout = new FormLayout();
			dataEditFormLayout.setStyleName("windowed-form");
			dataEditFormLayout.setMargin(new MarginInfo(false, true, false, false));
			formVerticalLayout.addComponent(dataEditFormLayout, formVerticalLayout.getComponentCount() - 1);

			for (StudyEntryData studyEntryData : entity.getStudyEntryData()) {
				String value = null;
				if (entity.getStudyEntryStudyEntryData() != null) {
					for (StudyEntryStudyEntryData studyEntryStudyEntryData : entity.getStudyEntryStudyEntryData()) {
						if (studyEntryStudyEntryData.getStudyEntryData().equals(studyEntryData)) {
							value = studyEntryStudyEntryData.getDataValue();
							break;
						}
					}
				}
				String dataKey = studyEntryData.getDataKey();
				String name = studyEntryData.getName();
				boolean needShowMode;

				if (FormMode.SHOW.equals(formMode) && UserUtil.isPatient() && (
						StudyEntryType.QUESTIONNAIRE_1.equals(entity.getStudyEntryType()) ||
						StudyEntryType.QUESTIONNAIRE_2.equals(entity.getStudyEntryType()) ||
						StudyEntryType.QUESTIONNAIRE_3.equals(entity.getStudyEntryType())
				)) {
					needShowMode = false;
				} else if (FormMode.EDIT.equals(formMode) && !UserUtil.isPatient() && (
						StudyEntryType.QUESTIONNAIRE_1.equals(entity.getStudyEntryType()) ||
						StudyEntryType.QUESTIONNAIRE_2.equals(entity.getStudyEntryType()) ||
						StudyEntryType.QUESTIONNAIRE_3.equals(entity.getStudyEntryType())
				)) {
					needShowMode = true;
				} else if (FormMode.SHOW.equals(formMode)) {
					needShowMode = true;
				} else {
					needShowMode = false;
				}

				if (needShowMode) {

					String convertedValue = value;
					switch (entity.getStudyEntryType()) {
						case QUESTIONNAIRE_1:
						case QUESTIONNAIRE_2:
						case QUESTIONNAIRE_3:
							if ("q1".equals(dataKey)) {
								if ("0".equals(value)) {
									convertedValue = "Igen";
								} else if ("1".equals(value)) {
									convertedValue = "Nem";
								}
							} else if ("q2".equals(dataKey)) {
								if ("0".equals(value)) {
									convertedValue = "nem dohányzik, és sosem dohányzott";
								} else if ("1".equals(value)) {
									convertedValue = "dohányos";
								} else if ("2".equals(value)) {
									convertedValue = "régen dohányzott";
								}
							} else if ("q3".equals(dataKey)) {
								if ("0".equals(value)) {
									convertedValue = "Igen";
								} else if ("1".equals(value)) {
									convertedValue = "Nem";
								}
							}
							break;
						default:
							if ("foodintolerance".equals(dataKey)) {
								if ("0".equals(value)) {
									convertedValue = "nincs";
								} else if ("1".equals(value)) {
									convertedValue = "enyhe";
								} else if ("2".equals(value)) {
									convertedValue = "közepes";
								} else if ("3".equals(value)) {
									convertedValue = "klinikailag szignifikáns";
								}
							}
							break;
					}

					Label label = new Label(convertedValue);
					label.setCaption(name);
					dataEditFormLayout.addComponent(label);
				} else {
					switch (entity.getStudyEntryType()) {
						case QUESTIONNAIRE_1:
						case QUESTIONNAIRE_2:
						case QUESTIONNAIRE_3:
							if ("q1".equals(dataKey)) {
								OptionGroup optionGroup = new OptionGroup(name);
								optionGroup.setData(studyEntryData);
								optionGroup.setMultiSelect(false);

								optionGroup.addItem(0);
								optionGroup.setItemCaption(0, "Igen");
								optionGroup.addItem(1);
								optionGroup.setItemCaption(1, "Nem");

								if (value != null) {
									optionGroup.setValue(Integer.valueOf(value));
								}

								dataEditFormLayout.addComponent(optionGroup);
								studyEntryDataComponents.add(optionGroup);
							} else if ("q2".equals(dataKey)) {
								OptionGroup optionGroup = new OptionGroup(name);
								optionGroup.setData(studyEntryData);
								optionGroup.setMultiSelect(false);

								optionGroup.addItem(0);
								optionGroup.setItemCaption(0, "nem dohányzik, és sosem dohányzott");
								optionGroup.addItem(1);
								optionGroup.setItemCaption(1, "dohányos");
								optionGroup.addItem(2);
								optionGroup.setItemCaption(2, "régen dohányzott");

								if (value != null) {
									optionGroup.setValue(Integer.valueOf(value));
								}

								dataEditFormLayout.addComponent(optionGroup);
								studyEntryDataComponents.add(optionGroup);
							} else if ("q3".equals(dataKey)) {
								OptionGroup optionGroup = new OptionGroup(name);
								optionGroup.setData(studyEntryData);
								optionGroup.setMultiSelect(false);

								optionGroup.addItem(0);
								optionGroup.setItemCaption(0, "Igen");
								optionGroup.addItem(1);
								optionGroup.setItemCaption(1, "Nem");

								if (value != null) {
									optionGroup.setValue(Integer.valueOf(value));
								}

								dataEditFormLayout.addComponent(optionGroup);
								studyEntryDataComponents.add(optionGroup);
							}
							break;
						default:
							if ("foodintolerance".equals(dataKey)) {
								OptionGroup optionGroup = new OptionGroup(name);
								optionGroup.setData(studyEntryData);
								optionGroup.setMultiSelect(false);

								optionGroup.addItem(0);
								optionGroup.setItemCaption(0, "nincs");
								optionGroup.addItem(1);
								optionGroup.setItemCaption(1, "enyhe");
								optionGroup.addItem(2);
								optionGroup.setItemCaption(2, "közepes");
								optionGroup.addItem(3);
								optionGroup.setItemCaption(3, "klinikailag szignifikáns");

								if (value != null) {
									optionGroup.setValue(Integer.valueOf(value));
								}

								dataEditFormLayout.addComponent(optionGroup);
								studyEntryDataComponents.add(optionGroup);
							} else {
								TextField textField = new TextField(name);
								textField.setNullRepresentation("");
								textField.setData(studyEntryData);
								textField.setValue(value);
								dataEditFormLayout.addComponent(textField);
								studyEntryDataComponents.add(textField);
							}
							break;
					}
				}
			}
		}

	}

	private void setStudyEntryDataValueByDataKey(String dataKey, String value) {
		for (Component component : studyEntryDataComponents) {
			if (component instanceof NumberField) {
				NumberField numberField = (NumberField) component;
				StudyEntryData studyEntryData = (StudyEntryData) numberField.getData();
				if (studyEntryData.getDataKey().equals(dataKey)) {
					numberField.setValue(value);
					break;
				}
			}
		}
	}

	@Override
	public void initMemberFields(final BeanFieldGroup<StudyEntry> beanFieldGroup, Container... container) {
		if (formMode.equals(FormMode.EDIT) && UserUtil.isPatient()) {
			formMode = FormMode.SHOW;
		}

		studyEntry = beanFieldGroup.getItemDataSource().getBean();
		isReadOnly = FormMode.SHOW.equals(formMode);

		formLayout.setEnabled(!isReadOnly);

		if (formMode.equals(FormMode.NEW)) {
			ComboBox studyEntryTypeComboBox = new StudyEntryTypeComboBox(i18n.get("StudyEntry.studyEntryType"));
			studyEntryTypeComboBox.setFilteringMode(FilteringMode.CONTAINS);
			beanFieldGroup.bind(studyEntryTypeComboBox, "studyEntryType");
			formLayout.addComponent(studyEntryTypeComboBox);
		} else {
			Label label = new Label(studyEntry.getStudyEntryType().getCaption());
			label.setCaption(i18n.get("StudyEntry.studyEntryType"));
			formLayout.addComponent(label);
		}
		JodaDateField timeJodaDateField = new JodaDateField(i18n.get("StudyEntry.time")) {
			@Override
			protected Date handleUnparsableDateString(String dateString) throws Converter.ConversionException {
				Notification.show(i18n.get("DateField.invalidDateFormat"), Notification.Type.WARNING_MESSAGE);
				throw new Converter.ConversionException(i18n.get("DateField.invalidDateFormat"));
			}
		};
		timeJodaDateField.setConverter(new DateToJodaDateConverter());
		beanFieldGroup.bind(timeJodaDateField, "time");
		formLayout.addComponent(timeJodaDateField);

		Component textArea = beanFieldGroup.buildAndBind(i18n.get("StudyEntry.description"), "description", TextArea.class);
		formLayout.addComponent(textArea);

		if (studyEntry.getTestDateTime() != null) {
			Label label = new Label(studyEntry.getTestDateTime().toString(DateTimeFormat.shortDate()));
			label.setCaption(i18n.get("StudyEntryForm.testDateTime"));
			formLayout.addComponent(label);
		}

		beanFieldGroup.addCommitHandler(new FieldGroup.CommitHandler() {
			@Override
			public void preCommit(FieldGroup.CommitEvent commitEvent) throws FieldGroup.CommitException {
				studyEntry.setCustomer(UserUtil.getCustomer());
			}

			@Override
			public void postCommit(FieldGroup.CommitEvent commitEvent) throws FieldGroup.CommitException {
				if (studyEntry.getStudyEntryData() == null || studyEntry.getStudyEntryData().isEmpty()) {
					StudyEntryBaseData studyEntryBaseData = service.findOneStudyEntryBaseDataByStudyEntryType(studyEntry.getStudyEntryType());

					if (studyEntryBaseData != null) {
						for (StudyEntryData studyEntryData : studyEntryBaseData.getStudyEntryData()) {
							List<StudyEntry> studyEntries = studyEntryData.getStudyEntries();
							if (studyEntries == null) {
								studyEntries = new ArrayList<>();
							}

							studyEntry = service.save(studyEntry);

							studyEntries.add(studyEntry);

							service.saveStudyEntryData(studyEntryData);
						}
					}
				}

				if (!studyEntryDataComponents.isEmpty()) {
					for (Component component : studyEntryDataComponents) {
						Object value;
						StudyEntryData studyEntryData;
						if (component instanceof OptionGroup) {
							OptionGroup optionGroup = (OptionGroup) component;
							studyEntryData = (StudyEntryData) optionGroup.getData();
							value = optionGroup.getValue();
						} else if (component instanceof TextField) {
							TextField textField = (TextField) component;
							studyEntryData = (StudyEntryData) textField.getData();
							value = textField.getValue();
						} else {
							NumberField numberField = (NumberField) component;
							studyEntryData = (StudyEntryData) numberField.getData();
							value = numberField.getValue();
						}

						Set<StudyEntryStudyEntryData> studyEntryStudyEntryDataSet = studyEntry.getStudyEntryStudyEntryData();
						if (studyEntryStudyEntryDataSet == null) {
							studyEntryStudyEntryDataSet = new HashSet<>();
						}

						if (studyEntryStudyEntryDataSet.isEmpty()) {
							createNew(studyEntryData, value, studyEntryStudyEntryDataSet);
						} else {
							boolean isStudyEntryStudyEntryDataExists = false;
							for (StudyEntryStudyEntryData studyEntryStudyEntryData : studyEntryStudyEntryDataSet) {
								if (studyEntryStudyEntryData.getStudyEntryData().equals(studyEntryData)) {
									isStudyEntryStudyEntryDataExists = true;
									studyEntryStudyEntryData.setDataValue(String.valueOf(value));
									studyEntryData.setStudyEntryStudyEntryData(studyEntryStudyEntryData);
									studyEntryStudyEntryDataSet.add(studyEntryStudyEntryData);
									break;
								}
							}

							if (!isStudyEntryStudyEntryDataExists) {
								createNew(studyEntryData, value, studyEntryStudyEntryDataSet);
							}
						}
					}
				}
			}
		});
	}

	private void createNew(StudyEntryData studyEntryData, Object value, Set<StudyEntryStudyEntryData> studyEntryStudyEntryDataSet) {
		StudyEntryStudyEntryData studyEntryStudyEntryData = new StudyEntryStudyEntryData();
		studyEntryStudyEntryData.setStudyEntry(studyEntry);
		studyEntryStudyEntryData.setStudyEntryData(studyEntryData);
		studyEntryStudyEntryData.setDataValue(String.valueOf(value));
		studyEntryData.setStudyEntryStudyEntryData(studyEntryStudyEntryData);
		studyEntryStudyEntryDataSet.add(studyEntryStudyEntryData);
	}
}
