package hu.oroscom.c1801.component.form;

import com.vaadin.data.Container;
import com.vaadin.data.fieldgroup.BeanFieldGroup;
import com.vaadin.data.fieldgroup.FieldGroup;
import com.vaadin.ui.Label;
import com.vaadin.ui.TextArea;
import hu.oroscom.c1801.domain.UserComment;
import hu.oroscom.c1801.enumeration.UserCommentType;
import hu.oroscom.c1801.service.UserCommentService;
import hu.oroscom.c1801.util.UserUtil;

public class UserCommentForm extends BaseForm<UserComment, UserCommentService> {

	public UserCommentForm(UserComment entity, boolean isWindowedForm, Container... container) {
		super(entity, isWindowedForm, container);
	}

	@Override
	public void initMemberFields(final BeanFieldGroup<UserComment> beanFieldGroup, Container... container) {
		UserComment userComment = beanFieldGroup.getItemDataSource().getBean();
		boolean isNew = userComment.getId() == null;

		switch (UserUtil.getUser().getRole().getRoleNameEnum()) {
			case ROLE_ASSISTANT:
				addDisabledInput(i18n.get("UserComment.question"), userComment.getQuestion());
				if (userComment.getUserCommentType() == null || (isNew && UserCommentType.BROADCAST.equals(userComment.getUserCommentType()))) {
					formLayout.addComponent(beanFieldGroup.buildAndBind(i18n.get("UserComment.answer"), "answer", TextArea.class));
				} else {
					addDisabledInput(i18n.get("UserComment.answer"), userComment.getAnswer());
				}
				break;
			case ROLE_DOCTOR:
				addDisabledInput(i18n.get("UserComment.question"), userComment.getQuestion());
				if (UserCommentType.ASSIGNED_TO_DOCTOR.equals(userComment.getUserCommentType())) {
					formLayout.addComponent(beanFieldGroup.buildAndBind(i18n.get("UserComment.answer"), "answer", TextArea.class));
				} else {
					addDisabledInput(i18n.get("UserComment.answer"), userComment.getAnswer());
				}
				break;
			case ROLE_PATIENT:
				if (isNew) {
					formLayout.addComponent(beanFieldGroup.buildAndBind(i18n.get("UserComment.question"), "question", TextArea.class));
					addDisabledInput(i18n.get("UserComment.answer"), userComment.getAnswer());
				} else {
					addDisabledInput(i18n.get("UserComment.question"), userComment.getQuestion());
					addDisabledInput(i18n.get("UserComment.answer"), userComment.getAnswer());
				}
				break;
		}

		beanFieldGroup.addCommitHandler(new FieldGroup.CommitHandler() {
			@Override
			public void preCommit(FieldGroup.CommitEvent commitEvent) throws FieldGroup.CommitException {
				UserComment userComment = beanFieldGroup.getItemDataSource().getBean();
				boolean isNew = userComment.getId() == null;

				switch (UserUtil.getUser().getRole().getRoleNameEnum()) {
					case ROLE_ASSISTANT:
						if (isNew || UserCommentType.BROADCAST.equals(userComment.getUserCommentType())) {
							userComment.setUserCommentType(UserCommentType.BROADCAST);
						} else {
							userComment.setUserCommentType(UserCommentType.ANSWERED);
						}
					case ROLE_DOCTOR:
						userComment.setUserCommentType(UserCommentType.ANSWERED);
						break;
				}
			}

			@Override
			public void postCommit(FieldGroup.CommitEvent commitEvent) throws FieldGroup.CommitException {

			}
		});
	}

	private void addDisabledInput(String caption, String value) {
		Label disabledInputLabel = new Label(value);
		disabledInputLabel.setCaption(caption);
		formLayout.addComponent(disabledInputLabel);
	}
}
