package hu.oroscom.c1801.component.form;

import com.vaadin.data.Container;
import com.vaadin.data.Validator;
import com.vaadin.data.fieldgroup.BeanFieldGroup;
import com.vaadin.data.fieldgroup.FieldGroup;
import com.vaadin.data.fieldgroup.PropertyId;
import com.vaadin.data.util.BeanItemContainer;
import com.vaadin.ui.*;
import hu.oroscom.c1801.component.combobox.VenueComboBox;
import hu.oroscom.c1801.domain.User;
import hu.oroscom.c1801.enumeration.EntityStatus;
import hu.oroscom.c1801.enumeration.EntryPoint;
import hu.oroscom.c1801.enumeration.PermissionName;
import hu.oroscom.c1801.enumeration.RoleName;
import hu.oroscom.c1801.service.UserService;
import hu.oroscom.c1801.util.UserUtil;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * Created by Norbert on 2014.10.15..
 */
public class UserForm extends BaseForm<User, UserService> {
	private User user;

	@PropertyId("status")
	private ComboBox entityStatusComboBox;

	@PropertyId("entryPoint")
	private OptionGroup entryPointOptionGroup;

	@PropertyId("venue")
	private VenueComboBox venueComboBox;

	private PasswordField passwordField;
	private PasswordField confirmPasswordField;

	public UserForm(User entity, Container... container) {
		super(entity, container);
	}

	public UserForm(User entity, boolean isWindowedForm, Container... container) {
		super(entity, isWindowedForm, container);
	}

	@Override
	public void initMemberFields(final BeanFieldGroup<User> beanFieldGroup, Container... container) {
		user = beanFieldGroup.getItemDataSource().getBean();
		User sessionUser = UserUtil.getUser();

		formLayout.addComponent(beanFieldGroup.buildAndBind(i18n.get("User.firstName"), "firstName"));
		formLayout.addComponent(beanFieldGroup.buildAndBind(i18n.get("User.lastName"), "lastName"));
		formLayout.addComponent(beanFieldGroup.buildAndBind(i18n.get("User.userName"), "userName"));
		formLayout.addComponent(beanFieldGroup.buildAndBind(i18n.get("User.shortName"), "shortName"));
//		formLayout.addComponent(beanFieldGroup.buildAndBind(i18n.get("User.company"), "company"));

		venueComboBox = new VenueComboBox(i18n.get("UserForm.venue"));
		formLayout.addComponent(venueComboBox);

		final BeanItemContainer<EntityStatus> entityStatusBeanItemContainer = new BeanItemContainer<>(EntityStatus.class);
		for (EntityStatus entityStatus : EntityStatus.values()) {
			if (entityStatus != EntityStatus.DELETED) {
				entityStatusBeanItemContainer.addBean(entityStatus);
			}
		}
		entityStatusComboBox = new ComboBox(i18n.get("Service.status"), entityStatusBeanItemContainer);
		entityStatusComboBox.setItemCaptionPropertyId("caption");
		entityStatusComboBox.setRequired(true);
		entityStatusComboBox.setNullSelectionAllowed(false);
		formLayout.addComponent(entityStatusComboBox);

		formLayout.addComponent(beanFieldGroup.buildAndBind(i18n.get("User.shortDescription"), "description"));
		formLayout.addComponent(beanFieldGroup.buildAndBind(i18n.get("User.description"), "shortDescription", TextArea.class));

		Label changePasswordLabel = new Label(i18n.get("UserForm.changePasswordLabel"));
		formLayout.addComponent(changePasswordLabel);

		passwordField = new PasswordField(i18n.get("User.password"));
		passwordField.setRequired(true);
		passwordField.setNullRepresentation("");
		formLayout.addComponent(passwordField);

		confirmPasswordField = new PasswordField(i18n.get("UserForm.confirmPasswordField"));
		formLayout.addComponent(confirmPasswordField);
		confirmPasswordField.addValidator(new Validator() {
			@Override
			public void validate(Object value) throws InvalidValueException {
				String password = (String) value;
				if (password != null && !password.equals(passwordField.getValue())) {
					throw new InvalidValueException(i18n.get("UserForm.confirmPasswordFieldDoesntMatchErrorMessage"));
				}
			}
		});
		beanFieldGroup.bind(confirmPasswordField, "password");
		passwordField.setValue(confirmPasswordField.getValue());

		Label bookingSettingsLabel = new Label(i18n.get("UserForm.bookingSettingsLabel"));
		formLayout.addComponent(bookingSettingsLabel);

		EntryPoint[] entryPoints = EntryPoint.values();
		List<EntryPoint> entryPointList = new ArrayList<>(Arrays.asList(entryPoints));
		entryPointOptionGroup = new OptionGroup(i18n.get("User.entryPoint"), entryPointList);
		entryPointOptionGroup.setVisible(service.userHasPermission(PermissionName.PERM_DASHBOARD_VIEW));
		entryPointOptionGroup.addStyleName("horizontal");
		for (EntryPoint entryPoint : entryPointList) {
			entryPointOptionGroup.setItemCaption(entryPoint, entryPoint.getCaption());
		}
		formLayout.addComponent(entryPointOptionGroup);

		beanFieldGroup.bindMemberFields(this);

		beanFieldGroup.addCommitHandler(new FieldGroup.CommitHandler() {
			@Override
			public void preCommit(FieldGroup.CommitEvent commitEvent) throws FieldGroup.CommitException {
				User user = beanFieldGroup.getItemDataSource().getBean();
				user.setRole(service.findRoleByRoleName(RoleName.ROLE_ADMIN));
			}

			@Override
			public void postCommit(FieldGroup.CommitEvent commitEvent) throws FieldGroup.CommitException {
				User user = beanFieldGroup.getItemDataSource().getBean();
				if (user.getPassword().length() < 128) {
					user.setPassword(UserUtil.encryptPassword(user.getPassword()));
					passwordField.setValue(user.getPassword());
					confirmPasswordField.setValue(user.getPassword());
				}
				user.setCustomer(UserUtil.getCustomer());

				user = service.save(user);
				if (RoleName.ROLE_API.equals(user.getRole().getRoleNameEnum())) {
					service.addUserToApiSettings(user);
				} else {
					service.removeUserToApiSettings(user);
				}
			}
		});
	}
}
