package hu.oroscom.c1801.component.table;

import com.vaadin.data.Container;
import com.vaadin.server.KeyMapper;
import com.vaadin.ui.Table;
import hu.oroscom.c1801.listener.ColumnCollapsedEvent;
import hu.oroscom.c1801.listener.ColumnSortEvent;

import java.lang.reflect.Field;
import java.util.*;

/**
 * Created by Norbert on 2014.10.27..
 */
public class BaseTable extends Table {
	private Set<Object> collapsedColumns;
	private List<ColumnCollapsedEvent.ColumnCollapsedListener> collapseListeners = new ArrayList<>();
	private List<ColumnSortEvent.ColumnSortListener> sortListeners = new ArrayList<>();
	private final HashMap<Object, String> columnHeaderTitles = new HashMap<>();
	private boolean columnHeaderTitleAllowed = false;

	public BaseTable(String caption, Container dataSource) {
		super(caption, dataSource);
		this.collapsedColumns = new HashSet<>();
		setPageLength(50);
		setCacheRate(2);
		setWidth(100, Unit.PERCENTAGE);
		setVisibleColumns();
	}

	@Override
	public String getColumnHeader(Object propertyId) {
		String columnHeader = super.getColumnHeader(propertyId);

		StackTraceElement[] stackTraceElements = Thread.currentThread().getStackTrace();

		if (columnHeader != null && !"export".equals(stackTraceElements[5].getMethodName())) {
			columnHeader = columnHeader.replaceAll("\n", "<br />");
		}

		if (columnHeaderTitleAllowed) {
			return "<span title=\"" + getColumnHeaderTitle(propertyId) + "\">" + columnHeader + "</span>";
		}

		return columnHeader;
	}

	public boolean isColumnHeaderTitleAllowed() {
		return columnHeaderTitleAllowed;
	}

	public void setColumnHeaderTitleAllowed(boolean columnHeaderTitleAllowed) {
		this.columnHeaderTitleAllowed = columnHeaderTitleAllowed;
	}

	public String[] getColumnHeaderTitles() {
		if (columnHeaderTitles == null) {
			return null;
		}
		final String[] headerTitles = new String[getVisibleColumns().length];
		int i = 0;
		for (Object o : getVisibleColumns()) {
			headerTitles[i] = getColumnHeaderTitle(o);
			i++;
		}
		return headerTitles;
	}

	public String getColumnHeaderTitle(Object propertyId) {
		if (getColumnHeaderMode() == ColumnHeaderMode.HIDDEN) {
			return null;
		}

		String title = columnHeaderTitles.get(propertyId);
		if ((title == null && getColumnHeaderMode() == ColumnHeaderMode.EXPLICIT_DEFAULTS_ID) || getColumnHeaderMode() == ColumnHeaderMode.ID) {
			title = propertyId.toString();
		}

		return title;
	}

	public void setColumnHeaderTitle(Object propertyId, String title) {

		if (title == null) {
			columnHeaderTitles.remove(propertyId);
		} else {
			columnHeaderTitles.put(propertyId, title);
		}

		markAsDirty();
	}

	@Override
	public void setColumnCollapsed(Object propertyId, boolean collapsed) throws IllegalStateException {
		super.setColumnCollapsed(propertyId, collapsed);
		if (collapsed) {
			collapsedColumns.add(propertyId);
		} else {
			collapsedColumns.remove(propertyId);
		}
		fireColumnCollapsedEvent(new ColumnCollapsedEvent(this, propertyId, collapsed));
	}

	public void addColumnCollapsedListener(ColumnCollapsedEvent.ColumnCollapsedListener columnCollapsedListener) {
		collapseListeners.add(columnCollapsedListener);
	}

	public void removeColumnCollapsedListener(ColumnCollapsedEvent.ColumnCollapsedListener columnCollapsedListener) {
		collapseListeners.remove(columnCollapsedListener);
	}

	private void fireColumnCollapsedEvent(ColumnCollapsedEvent columnCollapsedEvent) {
		for (ColumnCollapsedEvent.ColumnCollapsedListener columnCollapsedListener : collapseListeners) {
			columnCollapsedListener.columnCollapsed(columnCollapsedEvent);
		}
	}

	public Set<Object> getCollapsedColumns() {
		return collapsedColumns;
	}

	@Override
	public void changeVariables(Object source, Map<String, Object> variables) {
		if (this.isSortEnabled()) {
			if (((Map) variables).containsKey("sortcolumn")) {
				String var14 = (String) ((Map) variables).get("sortcolumn");
				if (var14 != null && !"".equals(var14) && !"null".equals(var14)) {
					try {
						Field columnIdMapField = this.getClass().getSuperclass().getDeclaredField("columnIdMap");
						columnIdMapField.setAccessible(true);
						KeyMapper<Object> columnIdMap = (KeyMapper<Object>) columnIdMapField.get(this);
						Object propertyId = columnIdMap.get(var14);
						fireColumnSortEvent(new ColumnSortEvent(this, propertyId, this.isSortAscending()));
					} catch (NoSuchFieldException | IllegalAccessException e) {
						e.printStackTrace();
					}
				}
			}

			if (((Map) variables).containsKey("sortascending")) {
				boolean var15 = ((Boolean) ((Map) variables).get("sortascending")).booleanValue();
				if (var15 != this.isSortAscending()) {
					fireColumnSortEvent(new ColumnSortEvent(this, this.getSortContainerPropertyId(), var15));
				}
			}

		}
		super.changeVariables(source, variables);
	}

	public void addColumnSortListener(ColumnSortEvent.ColumnSortListener columnSortListener) {
		sortListeners.add(columnSortListener);
	}

	public void removeColumnSortListener(ColumnSortEvent.ColumnSortListener columnSortListener) {
		sortListeners.remove(columnSortListener);
	}

	private void fireColumnSortEvent(ColumnSortEvent columnSortEvent) {
		for (ColumnSortEvent.ColumnSortListener columnSortListener : sortListeners) {
			columnSortListener.columnSort(columnSortEvent);
		}
	}

}
