package hu.oroscom.c1801.component.windowed;

import com.vaadin.data.Property;
import com.vaadin.server.FontAwesome;
import com.vaadin.shared.ui.combobox.FilteringMode;
import com.vaadin.shared.ui.label.ContentMode;
import com.vaadin.ui.*;
import com.vaadin.ui.themes.ValoTheme;
import hu.oroscom.c1801.Application;
import hu.oroscom.c1801.converter.StringToIntegerConverter;
import hu.oroscom.c1801.domain.User;
import hu.oroscom.c1801.domain.filter.Filter;
import hu.oroscom.c1801.enumeration.*;
import hu.oroscom.c1801.service.BaseService;
import hu.oroscom.c1801.service.FilterService;
import hu.oroscom.c1801.service.UserService;
import hu.oroscom.c1801.util.*;
import hu.oroscom.c1801.widgetset.NumberField;
import org.joda.time.DateTime;
import org.joda.time.format.DateTimeFormat;
import org.joda.time.format.DateTimeFormatter;
import org.vaadin.spring.events.EventBus;
import org.vaadin.spring.i18n.I18N;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * Created by Norbert on 2015.07.13..
 */
public class FilterPanel<F extends Filter, S extends BaseService> extends Panel implements Button.ClickListener {
	protected static DateTimeFormatter dateTimeFormatter = DateTimeFormat.forPattern("yyyyMMdd");

	private F filter;
	private S[] services;
	private I18N i18n;
	private EventBus eventBus;
	private Button cancelButton;
	private Button saveButton;
	private Button applyButton;
	private VerticalLayout filterItemVerticalLayout = new VerticalLayout();
	private Label headingLabel;
	private ComboBox addFilterComboBox = new ComboBox();
	private TextField filterNameTextField = new TextField();

	private Property.ValueChangeListener filterValueComponentValueChangeListener = new Property.ValueChangeListener() {
		@Override
		public void valueChange(Property.ValueChangeEvent event) {
			FilterItemBean filterItemBean;
			if (event.getProperty() instanceof ComboBox) {
				ComboBox filterValueComboBox = (ComboBox) event.getProperty();
				filterItemBean = (FilterItemBean) filterValueComboBox.getData();
			} else if (event.getProperty() instanceof NumberField) {
				NumberField filterValueNumberField = (NumberField) event.getProperty();
				filterItemBean = (FilterItemBean) filterValueNumberField.getData();
			} else if (event.getProperty() instanceof DateField) {
				DateField filterValueDateField = (DateField) event.getProperty();
				filterItemBean = (FilterItemBean) filterValueDateField.getData();
			} else {
				TextField filterValueTextField = (TextField) event.getProperty();
				filterItemBean = (FilterItemBean) filterValueTextField.getData();
			}

			try {
				Field filterField = filter.getClass().getDeclaredField(filterItemBean.getFieldName());
				filterField.setAccessible(true);
				List<String> stringList = (List<String>) filterField.get(filter);
				FilterItemBean changedFilterItemBean = FilterUtil.getFilterItem(stringList.get(filterItemBean.getFieldListIndex()));
				stringList.remove(filterItemBean.getFieldListIndex());
				Object value = event.getProperty().getValue();
				if (value == null) {
					value = "";
				}
				String fieldValue = String.valueOf(value);
				stringList.add(filterItemBean.getFieldListIndex(), FilterUtil.getFilterString(changedFilterItemBean.getFilterOperand(), fieldValue));
				filterField.set(filter, stringList);
			} catch (NoSuchFieldException | IllegalAccessException e) {
				e.printStackTrace();
			}
		}
	};

	private ComboBox.ValueChangeListener filterOperandComboBoxValueChangeListener = new Property.ValueChangeListener() {
		@Override
		public void valueChange(Property.ValueChangeEvent event) {
			ComboBox filterOperandComboBox = (ComboBox) event.getProperty();
			String filterValue = null;
			try {
				HorizontalLayout horizontalLayout = (HorizontalLayout) filterOperandComboBox.getParent();
				HorizontalLayout filterValueHorizontalLayout = (HorizontalLayout) horizontalLayout.getComponent(2);
				DateField startDateField = (DateField) filterValueHorizontalLayout.getComponent(0);
				DateField endDateField = (DateField) filterValueHorizontalLayout.getComponent(1);

				if (FilterOperand.INTERVAL.equals(filterOperandComboBox.getValue())) {
					endDateField.setVisible(true);
					filterValue = new DateTime(startDateField.getValue()).toString("yyyyMMdd") + "sep" + new DateTime(endDateField.getValue()).toString("yyyyMMdd");
				} else {
					endDateField.setVisible(false);
					filterValue = new DateTime(startDateField.getValue()).toString("yyyyMMdd");
				}
			} catch (Exception e) {
			}
			FilterItemBean filterItemBean = (FilterItemBean) filterOperandComboBox.getData();
			try {
				Field filterField = filter.getClass().getDeclaredField(filterItemBean.getFieldName());
				filterField.setAccessible(true);
				List<String> stringList = (List<String>) filterField.get(filter);
				FilterItemBean changedFilterItemBean = FilterUtil.getFilterItem(stringList.get(filterItemBean.getFieldListIndex()));
				stringList.remove(filterItemBean.getFieldListIndex());
				if (filterValue == null) {
					filterValue = changedFilterItemBean.getFilterValue();
				}
				stringList.add(filterItemBean.getFieldListIndex(), FilterUtil.getFilterString((FilterOperand) filterOperandComboBox.getValue(), filterValue));
				filterField.set(filter, stringList);
			} catch (NoSuchFieldException | IllegalAccessException e) {
				e.printStackTrace();
			}
		}
	};

	private Button.ClickListener filterRemoveButtonClickListener = new Button.ClickListener() {
		@Override
		public void buttonClick(Button.ClickEvent event) {
			FilterItemBean filterItemBean = (FilterItemBean) event.getButton().getData();
			try {
				Field filterField = filter.getClass().getDeclaredField(filterItemBean.getFieldName());
				filterField.setAccessible(true);
				List<String> stringList = (List<String>) filterField.get(filter);
				stringList.remove(filterItemBean.getFieldListIndex());
				if (stringList.isEmpty()) {
					stringList = null;
				}
				filterField.set(filter, stringList);
				generateFilters();
			} catch (NoSuchFieldException | IllegalAccessException e) {
				e.printStackTrace();
			}
		}
	};

	private ComboBox.ValueChangeListener addFilterComboBoxValueChangeListener = new Property.ValueChangeListener() {
		@Override
		public void valueChange(Property.ValueChangeEvent event) {
			String value = (String) event.getProperty().getValue();
			try {
				Field filterField = filter.getClass().getDeclaredField(value);
				filterField.setAccessible(true);
				List<String> stringList = (List<String>) filterField.get(filter);
				if (stringList == null) {
					stringList = new ArrayList<>();
				}
				stringList.add(FilterUtil.getFilterString(FilterOperand.EQUAL_TO, ""));
				filterField.set(filter, stringList);
				generateFilters();
				addFilterComboBox.removeValueChangeListener(addFilterComboBoxValueChangeListener);
				addFilterComboBox.setValue(null);
				addFilterComboBox.addValueChangeListener(addFilterComboBoxValueChangeListener);
				for (Window window : UI.getCurrent().getWindows()) {
					window.center();
				}
			} catch (NoSuchFieldException | IllegalAccessException e) {
				e.printStackTrace();
			}
		}
	};

	public FilterPanel(F filter, S... services) {
		this.filter = filter;
		this.services = services;
		this.i18n = (I18N) Application.getApplicationContext().getBean("i18n");
		this.eventBus = (EventBus) Application.getApplicationContext().getBean("uiEventBus");

		addStyleName("filter-panel");

		cancelButton = new Button(i18n.get("BaseForm.cancelButton"), this);
		saveButton = new Button(i18n.get("BaseForm.saveButton"), this);
		applyButton = new Button(i18n.get("BaseForm.applyButton"), this);

		headingLabel = new Label(i18n.get("FilterPanel.headingLabel", i18n.get("FilterPanel.custom")));
		headingLabel.addStyleName(ValoTheme.LABEL_H2);

		filterItemVerticalLayout.setWidth(100, Unit.PERCENTAGE);
		filterItemVerticalLayout.setSpacing(true);

		addFilterComboBox.addValueChangeListener(addFilterComboBoxValueChangeListener);
		addFilterComboBox.setNullSelectionAllowed(false);
		addFilterComboBox.addStyleName(ValoTheme.COMBOBOX_TINY);
		addFilterComboBox.setFilteringMode(FilteringMode.CONTAINS);

		if (FilterType.GLOBAL.equals(filter.getFilterType())) {
			saveButton.setEnabled(false);
			filterNameTextField.setEnabled(false);
		}

		generateFilters();

		HorizontalLayout buttonBarHorizontalLayout = new HorizontalLayout();
		buttonBarHorizontalLayout.setSpacing(true);
		buttonBarHorizontalLayout.addComponent(applyButton);
		buttonBarHorizontalLayout.addComponent(new HorizontalLayout(filterNameTextField, saveButton));
		buttonBarHorizontalLayout.addComponent(cancelButton);

		VerticalLayout verticalLayout = new VerticalLayout();
		verticalLayout.addComponent(headingLabel);
		verticalLayout.addComponent(filterItemVerticalLayout);
		verticalLayout.addComponent(addFilterComboBox);
		verticalLayout.addComponent(buttonBarHorizontalLayout);
		verticalLayout.setMargin(true);
		verticalLayout.setSpacing(true);

		setContent(verticalLayout);
	}

	private void generateFilters() {
		filterItemVerticalLayout.removeAllComponents();
		try {
			Field filterNameField = filter.getClass().getSuperclass().getDeclaredField("filterName");
			filterNameField.setAccessible(true);
			Object value = filterNameField.get(filter);
			if (value != null) {
				headingLabel.setValue(i18n.get("FilterPanel.headingLabel", value));
				filterNameTextField.setValue(String.valueOf(value));
			}

			for (Field field : filter.getClass().getDeclaredFields()) {
				field.setAccessible(true);
				value = field.get(filter);

				if (List.class.equals(field.getType()) && !"ids".equals(field.getName())) {
					addFilterComboBox.addItem(field.getName());
					addFilterComboBox.setItemCaption(field.getName(), MessagesUtil.get(i18n, "FilterInfo." + field.getName()));
				}

				if (value != null && List.class.equals(field.getType()) && !"ids".equals(field.getName())) {
					List<String> filterItemStrings = (List<String>) value;
					if (filterItemStrings.isEmpty()) {
						continue;
					}
					int i = 0;
					for (String filterItemString : filterItemStrings) {
						FilterItemBean filterItemBean = FilterUtil.getFilterItem(filterItemString);
						FilterOperand filterOperand = filterItemBean.getFilterOperand();

						Component filterTypeComponent;
						if (i == 0) {
							TextField filterTypeTextField = new TextField();
							filterTypeTextField.addStyleName(ValoTheme.TEXTFIELD_TINY);
							filterTypeTextField.setValue(MessagesUtil.get(i18n, "FilterInfo." + field.getName()));
							filterTypeTextField.setEnabled(false);
							filterTypeComponent = filterTypeTextField;
						} else {
							Label spacer = new Label();
							spacer.setWidth(185, Unit.PIXELS);
							filterTypeComponent = spacer;
						}

						ComboBox filterOperandComboBox = new ComboBox();
						filterOperandComboBox.setData(new FilterItemBean(field.getName(), i));
						filterOperandComboBox.setNullSelectionAllowed(false);
						filterOperandComboBox.setTextInputAllowed(false);
						filterOperandComboBox.addStyleName(ValoTheme.COMBOBOX_TINY);
						filterOperandComboBox.addItem(FilterOperand.EQUAL_TO);
						filterOperandComboBox.setItemCaption(FilterOperand.EQUAL_TO, FilterOperand.EQUAL_TO.getCaption());
						filterOperandComboBox.addItem(FilterOperand.DIFFERENT_FROM);
						filterOperandComboBox.setItemCaption(FilterOperand.DIFFERENT_FROM, FilterOperand.DIFFERENT_FROM.getCaption());

						Component filterValueComponent;
						switch (field.getName()) {
							case "connectedUser":
								UserService userService = (UserService) getService(UserService.class);
								ComboBox filterUserComboBox = new ComboBox();
								filterUserComboBox.setData(new FilterItemBean(field.getName(), i));
								filterUserComboBox.setNullSelectionAllowed(false);
								filterUserComboBox.addStyleName(ValoTheme.COMBOBOX_TINY);
								filterUserComboBox.setFilteringMode(FilteringMode.CONTAINS);
								assert userService != null;
								List<User> users = userService.findAllByCustomer();
								for (User user : users) {
									filterUserComboBox.addItem(user.getId());
									filterUserComboBox.setItemCaption(user.getId(), user.getDisplayName());
								}
								if (!filterItemBean.getFilterValue().equals("")) {
									if (!users.isEmpty()) {
										filterUserComboBox.setValue(Long.valueOf(filterItemBean.getFilterValue()));
									}
								}
								filterUserComboBox.addValueChangeListener(filterValueComponentValueChangeListener);
								if (filterItemBean.getFilterValue().equals("")) {
									if (!users.isEmpty()) {
										filterUserComboBox.setValue(users.get(0).getId());
									}
								}
								filterValueComponent = filterUserComboBox;
								break;
							case "doctor":
								userService = (UserService) getService(UserService.class);
								ComboBox filterDoctorComboBox = new ComboBox();
								filterDoctorComboBox.setData(new FilterItemBean(field.getName(), i));
								filterDoctorComboBox.setNullSelectionAllowed(false);
								filterDoctorComboBox.addStyleName(ValoTheme.COMBOBOX_TINY);
								filterDoctorComboBox.setFilteringMode(FilteringMode.CONTAINS);
								assert userService != null;
								List<User> doctors;

								if (UserUtil.isDoctor()) {
									doctors = new ArrayList<>();
									doctors.add(UserUtil.getUser());
								} else {
									doctors = userService.findAllByRoleName(RoleName.ROLE_DOCTOR);
								}

								for (User user : doctors) {
									filterDoctorComboBox.addItem(user.getId());
									filterDoctorComboBox.setItemCaption(user.getId(), user.getDisplayName());
								}
								if (!filterItemBean.getFilterValue().equals("")) {
									if (!doctors.isEmpty()) {
										filterDoctorComboBox.setValue(Long.valueOf(filterItemBean.getFilterValue()));
									}
								}
								filterDoctorComboBox.addValueChangeListener(filterValueComponentValueChangeListener);
								if (filterItemBean.getFilterValue().equals("")) {
									if (!doctors.isEmpty()) {
										filterDoctorComboBox.setValue(doctors.get(0).getId());
									}
								}
								filterValueComponent = filterDoctorComboBox;
								break;
							case "connectedStudyEntryType":
							case "entryType":
								ComboBox filterStudyEntryTypeComboBox = new ComboBox();
								filterStudyEntryTypeComboBox.setData(new FilterItemBean(field.getName(), i));
								filterStudyEntryTypeComboBox.setNullSelectionAllowed(false);
								filterStudyEntryTypeComboBox.addStyleName(ValoTheme.COMBOBOX_TINY);
								filterStudyEntryTypeComboBox.setFilteringMode(FilteringMode.CONTAINS);

								for (StudyEntryType studyEntryType : StudyEntryType.values()) {
									filterStudyEntryTypeComboBox.addItem(studyEntryType);
									filterStudyEntryTypeComboBox.setItemCaption(studyEntryType, studyEntryType.getCaption());
								}

								filterStudyEntryTypeComboBox.addValueChangeListener(filterValueComponentValueChangeListener);
								filterStudyEntryTypeComboBox.setValue(filterItemBean.getFilterValue().equals("") ? null : StudyEntryType.valueOf(filterItemBean.getFilterValue()));
								filterValueComponent = filterStudyEntryTypeComboBox;
								break;
							case "serviceManagementNature":
								ComboBox filterServiceManagementNatureComboBox = new ComboBox();
								filterServiceManagementNatureComboBox.setData(new FilterItemBean(field.getName(), i));
								filterServiceManagementNatureComboBox.setNullSelectionAllowed(false);
								filterServiceManagementNatureComboBox.addStyleName(ValoTheme.COMBOBOX_TINY);
								filterServiceManagementNatureComboBox.setFilteringMode(FilteringMode.CONTAINS);

								for (ServiceManagementNature serviceManagementNature : ServiceManagementNature.values()) {
									filterServiceManagementNatureComboBox.addItem(serviceManagementNature);
									filterServiceManagementNatureComboBox.setItemCaption(serviceManagementNature, serviceManagementNature.getCaption());
								}

								filterServiceManagementNatureComboBox.addValueChangeListener(filterValueComponentValueChangeListener);
								filterServiceManagementNatureComboBox.setValue(filterItemBean.getFilterValue().equals("") ? null : ServiceManagementNature.valueOf(filterItemBean.getFilterValue()));
								filterValueComponent = filterServiceManagementNatureComboBox;
								break;
							case "serviceManagementStatus":
								ComboBox filterServiceManagementStatusComboBox = new ComboBox();
								filterServiceManagementStatusComboBox.setData(new FilterItemBean(field.getName(), i));
								filterServiceManagementStatusComboBox.setNullSelectionAllowed(false);
								filterServiceManagementStatusComboBox.addStyleName(ValoTheme.COMBOBOX_TINY);
								filterServiceManagementStatusComboBox.setFilteringMode(FilteringMode.CONTAINS);

								for (ServiceManagementStatus serviceManagementStatus : ServiceManagementStatus.values()) {
									filterServiceManagementStatusComboBox.addItem(serviceManagementStatus);
									filterServiceManagementStatusComboBox.setItemCaption(serviceManagementStatus, serviceManagementStatus.getCaption());
								}

								filterServiceManagementStatusComboBox.addValueChangeListener(filterValueComponentValueChangeListener);
								filterServiceManagementStatusComboBox.setValue(filterItemBean.getFilterValue().equals("") ? null : ServiceManagementStatus.valueOf(filterItemBean.getFilterValue()));
								filterValueComponent = filterServiceManagementStatusComboBox;
								break;
							case "patient":
//								PatientService patientService = (PatientService) getService(PatientService.class);
								ComboBox filterPatientComboBox = new ComboBox();
								filterPatientComboBox.setData(new FilterItemBean(field.getName(), i));
								filterPatientComboBox.setNullSelectionAllowed(false);
								filterPatientComboBox.addStyleName(ValoTheme.COMBOBOX_TINY);
								filterPatientComboBox.setFilteringMode(FilteringMode.CONTAINS);
//								assert patientService != null;
//								List<Patient> patients = patientService.findAllByCustomer();
//								for (Patient patient : patients) {
//									filterPatientComboBox.addItem(patient.getId());
//									filterPatientComboBox.setItemCaption(patient.getId(), patient.getDisplayName());
//								}
//								if (!filterItemBean.getFilterValue().equals("")) {
//									if (!patients.isEmpty()) {
//										filterPatientComboBox.setValue(Long.valueOf(filterItemBean.getFilterValue()));
//									}
//								}
//								filterPatientComboBox.addValueChangeListener(filterValueComponentValueChangeListener);
//								if (filterItemBean.getFilterValue().equals("")) {
//									if (!patients.isEmpty()) {
//										filterPatientComboBox.setValue(patients.get(0).getId());
//									}
//								}
								filterValueComponent = filterPatientComboBox;
								break;
							case "approved":
							case "online":
								ComboBox filterBooleanComboBox = new ComboBox();
								filterBooleanComboBox.setData(new FilterItemBean(field.getName(), i));
								filterBooleanComboBox.setNullSelectionAllowed(false);
								filterBooleanComboBox.addStyleName(ValoTheme.COMBOBOX_TINY);
								filterBooleanComboBox.setFilteringMode(FilteringMode.CONTAINS);

								filterBooleanComboBox.addItem(1);
								filterBooleanComboBox.setItemCaption(1, i18n.get("FilterPanel.true"));
								filterBooleanComboBox.addItem(0);
								filterBooleanComboBox.setItemCaption(0, i18n.get("FilterPanel.false"));

								filterBooleanComboBox.addValueChangeListener(filterValueComponentValueChangeListener);
								filterBooleanComboBox.setValue(Integer.valueOf(filterItemBean.getFilterValue().equals("") ? "1" : filterItemBean.getFilterValue()));
								filterValueComponent = filterBooleanComboBox;
								break;
							case "baseStatus":
							case "onlineStatus":
								ComboBox filterEntityStatusComboBox = new ComboBox();
								filterEntityStatusComboBox.setData(new FilterItemBean(field.getName(), i));
								filterEntityStatusComboBox.setNullSelectionAllowed(false);
								filterEntityStatusComboBox.addStyleName(ValoTheme.COMBOBOX_TINY);
								filterEntityStatusComboBox.setFilteringMode(FilteringMode.CONTAINS);

								for (EntityStatus entityStatus : EntityStatus.values()) {
									if (EntityStatus.DELETED.equals(entityStatus)) {
										continue;
									}
									filterEntityStatusComboBox.addItem(entityStatus);
									filterEntityStatusComboBox.setItemCaption(entityStatus, entityStatus.getCaption());
								}

								filterEntityStatusComboBox.addValueChangeListener(filterValueComponentValueChangeListener);
								filterEntityStatusComboBox.setValue(filterItemBean.getFilterValue().equals("") ? null : EntityStatus.valueOf(filterItemBean.getFilterValue()));
								filterValueComponent = filterEntityStatusComboBox;
								break;
							case "exportDate":
							case "entryDateTime":
							case "dateTime":
								DateField filterValueDateField = new DateField();
								filterValueDateField.addStyleName("start");
								filterValueDateField.setData(new FilterItemBean(field.getName(), i));
								filterValueDateField.addStyleName(ValoTheme.DATEFIELD_TINY);

								DateField endDateField = new DateField();
								endDateField.setVisible(false);
								endDateField.addStyleName("end");
								endDateField.setData(new FilterItemBean(field.getName(), i));
								endDateField.addStyleName(ValoTheme.DATEFIELD_TINY);

								if (!filterItemBean.getFilterValue().isEmpty()) {
									if (FilterOperand.INTERVAL.equals(filterOperandComboBox.getValue()) || filterItemBean.getFilterValue().contains("sep")) {
										filterValueDateField.setValue(dateTimeFormatter.parseDateTime(filterItemBean.getFilterValue().split("sep")[0]).toDate());
										endDateField.setValue(dateTimeFormatter.parseDateTime(filterItemBean.getFilterValue().split("sep")[1]).toDate());
										endDateField.setVisible(true);
									} else {
										filterValueDateField.setValue(dateTimeFormatter.parseDateTime(filterItemBean.getFilterValue()).toDate());
									}
								}

								HorizontalLayout horizontalLayout = new HorizontalLayout(filterValueDateField, endDateField);

								filterValueDateField.addValueChangeListener(filterValueComponentValueChangeListener);
								if (filterItemBean.getFilterValue().isEmpty()) {
									filterValueDateField.setValue(new Date());
								}

								endDateField.addValueChangeListener(filterValueComponentValueChangeListener);
								if (filterItemBean.getFilterValue().isEmpty()) {
									endDateField.setValue(new Date());
								}

								filterValueComponent = horizontalLayout;

								filterOperandComboBox.addItem(FilterOperand.GREATER_THAN);
								filterOperandComboBox.setItemCaption(FilterOperand.GREATER_THAN, FilterOperand.GREATER_THAN.getCaption());
								filterOperandComboBox.addItem(FilterOperand.LESS_THAN);
								filterOperandComboBox.setItemCaption(FilterOperand.LESS_THAN, FilterOperand.LESS_THAN.getCaption());
								filterOperandComboBox.addItem(FilterOperand.INTERVAL);
								filterOperandComboBox.setItemCaption(FilterOperand.INTERVAL, FilterOperand.INTERVAL.getCaption());
								break;
							case "quantity":
								NumberField filterValueNumberField = new NumberField();
								filterValueNumberField.setData(new FilterItemBean(field.getName(), i));
								filterValueNumberField.addStyleName(ValoTheme.TEXTFIELD_TINY);
								filterValueNumberField.setValue(filterItemBean.getFilterValue());
								filterValueNumberField.setConverter(new StringToIntegerConverter());
								filterValueNumberField.addValueChangeListener(filterValueComponentValueChangeListener);
								filterValueComponent = filterValueNumberField;

								filterOperandComboBox.addItem(FilterOperand.GREATER_THAN);
								filterOperandComboBox.setItemCaption(FilterOperand.GREATER_THAN, FilterOperand.GREATER_THAN.getCaption());
								filterOperandComboBox.addItem(FilterOperand.LESS_THAN);
								filterOperandComboBox.setItemCaption(FilterOperand.LESS_THAN, FilterOperand.LESS_THAN.getCaption());
								break;
							default:
								TextField filterValueTextField = new TextField();
								filterValueTextField.setData(new FilterItemBean(field.getName(), i));
								filterValueTextField.addStyleName(ValoTheme.TEXTFIELD_TINY);
								filterValueTextField.setValue(filterItemBean.getFilterValue());
								filterValueTextField.addValueChangeListener(filterValueComponentValueChangeListener);
								filterValueComponent = filterValueTextField;

								filterOperandComboBox.addItem(FilterOperand.CONTAINS);
								filterOperandComboBox.setItemCaption(FilterOperand.CONTAINS, FilterOperand.CONTAINS.getCaption());
								filterOperandComboBox.addItem(FilterOperand.DOES_NOT_CONTAIN);
								filterOperandComboBox.setItemCaption(FilterOperand.DOES_NOT_CONTAIN, FilterOperand.DOES_NOT_CONTAIN.getCaption());
								filterOperandComboBox.addItem(FilterOperand.STARTS_WITH);
								filterOperandComboBox.setItemCaption(FilterOperand.STARTS_WITH, FilterOperand.STARTS_WITH.getCaption());
								filterOperandComboBox.addItem(FilterOperand.ENDS_WITH);
								filterOperandComboBox.setItemCaption(FilterOperand.ENDS_WITH, FilterOperand.ENDS_WITH.getCaption());
								break;
						}

						filterOperandComboBox.setValue(filterOperand);

						Button filterRemoveButton = new Button(FontAwesome.TIMES);
						filterRemoveButton.setData(new FilterItemBean(field.getName(), i));
						filterRemoveButton.addClickListener(filterRemoveButtonClickListener);
						filterRemoveButton.addStyleName(ValoTheme.BUTTON_ICON_ONLY);
						filterRemoveButton.addStyleName(ValoTheme.BUTTON_LINK);
						filterRemoveButton.setWidth(13, Unit.PIXELS);
						filterRemoveButton.setHeight(25, Unit.PIXELS);

//						Label spacer = new Label();
//						spacer.setWidth(100, Unit.PERCENTAGE);

						HorizontalLayout filterItemHorizontalLayout = new HorizontalLayout(filterTypeComponent, filterOperandComboBox, filterValueComponent, filterRemoveButton);
						filterItemHorizontalLayout.setSpacing(true);
//						filterItemHorizontalLayout.setWidth(100, Unit.PERCENTAGE);

						filterOperandComboBox.addValueChangeListener(filterOperandComboBoxValueChangeListener);

						filterItemVerticalLayout.addComponent(filterItemHorizontalLayout);
						i++;
					}
					Label logicalOperandLabel = new Label("&nbsp;" + i18n.get("FilterInfo.AND") + "&nbsp;", ContentMode.HTML);
					logicalOperandLabel.addStyleName("fieldset-legend");
					logicalOperandLabel.setWidth(100, Unit.PERCENTAGE);
					filterItemVerticalLayout.addComponent(logicalOperandLabel);
				}
			}

			if (filterItemVerticalLayout.getComponentCount() > 1) {
				filterItemVerticalLayout.removeComponent(filterItemVerticalLayout.getComponent(filterItemVerticalLayout.getComponentCount() - 1));
			}
		} catch (NoSuchFieldException | IllegalAccessException e) {
			e.printStackTrace();
		}
	}

	private S getService(Class aClass) {
		for (S availableService : services) {
			if (availableService.getClass().getName().contains(aClass.getName())) {
				return availableService;
			}
		}

		return null;
	}

	@Override
	public void buttonClick(Button.ClickEvent event) {
		if (event.getSource() == cancelButton) {
			eventBus.publish(this, new EventBusAction<>(Action.CANCEL));
		} else if (event.getSource() == saveButton) {
			if (FilterUtil.isFilterActive(filter) && !filterNameTextField.getValue().isEmpty()) {
				filter.setFilterName(filterNameTextField.getValue());
				filter.setUser(UserUtil.getUser());
				filter.setFilterCategory(FilterCategory.getFromClass(filter.getClass()));
				filter.setFilterType(FilterType.SAVED);
				getService(FilterService.class).save(filter);
				eventBus.publish(this, new EventBusAction<>(Action.SAVE_FILTER, filter));
				Notification.show(i18n.get("BaseForm.savedMessage", filterNameTextField.getValue()), Notification.Type.TRAY_NOTIFICATION);
			} else if (!FilterUtil.isFilterActive(filter)) {
				Notification.show(i18n.get("BaseForm.saveFailedErrorMessage"), i18n.get("FilterPanel.noFilterExpression"), Notification.Type.ERROR_MESSAGE);
			} else if (filterNameTextField.getValue().isEmpty()) {
				Notification.show(i18n.get("BaseForm.saveFailedErrorMessage"), i18n.get("FilterPanel.filterNameMustNotBeEmpty"), Notification.Type.ERROR_MESSAGE);
			}
		} else if (event.getSource() == applyButton) {
			eventBus.publish(this, new EventBusAction<>(Action.FILTER, filter));
		}
	}
}
