package hu.oroscom.c1801.domain;

import hu.oroscom.c1801.changelog.TrackChanges;
import hu.oroscom.c1801.converter.JodaDateTimeConverter;
import hu.oroscom.c1801.enumeration.EntityStatus;
import org.joda.time.DateTime;

import javax.persistence.*;
import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.util.*;

@MappedSuperclass
public abstract class BaseEntity extends EntityBase implements Serializable, Comparable<BaseEntity> {
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	private Long id;

	@Version
	private Integer version;

	@Column(columnDefinition = "TIMESTAMP NULL DEFAULT NULL")
	@org.eclipse.persistence.annotations.Converter(name = "dateTimeConverter", converterClass = JodaDateTimeConverter.class)
	@org.eclipse.persistence.annotations.Convert("dateTimeConverter")
	@NotNull
	private DateTime insertedAt = new DateTime();

	@ManyToOne
	private User insertedByUser;

	@TrackChanges
	@Column(nullable = false)
	private EntityStatus status = EntityStatus.ACTIVE;

	public Long getId() {
		return id;
	}

	public void nullId() {
		id = null;
		version = null;
	}

	public Integer getVersion() {
		return version;
	}

	public DateTime getInsertedAt() {
		return insertedAt;
	}

	public void setInsertedAt(DateTime insertedAt) {
		this.insertedAt = insertedAt;
	}

	public User getInsertedByUser() {
		return insertedByUser;
	}

	public void setInsertedByUser(User insertedByUser) {
		this.insertedByUser = insertedByUser;
	}

	public EntityStatus getStatus() {
		return status;
	}

	public void setStatus(EntityStatus status) {
		this.status = status;
	}

	protected <S extends BaseEntity, C extends Collection<S>> C getActiveEntities(Collection<S> entities) {
        if (entities != null) {
            Collection<S> collection = new HashSet<>();
            if (entities instanceof List) {
                collection = new ArrayList<>();
            }
            for (S entity : entities) {
	            if (entity.getStatus().equals(EntityStatus.ACTIVE)) {
		            collection.add(entity);
                }
            }
            return (C) collection;
        }
        return null;
    }

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}

		BaseEntity other = (BaseEntity) obj;

		return Objects.equals(getId(), (other.getId() == null ? null : other.getId().longValue()));
	}

	@Override
	public int hashCode() {
		return Arrays.hashCode(new Long[]{getId()});
	}

	@Override
	public int compareTo(BaseEntity baseEntity) {
		if ((baseEntity.getId() == null && getId() == null) || (baseEntity.getId().equals(getId()))) {
			return 0;
		} else if ((baseEntity.getId() == null && getId() != null) || (baseEntity.getId() < getId())) {
			return -1;
		} else {
			return 1;
		}
	}
}
