package hu.oroscom.c1801.domain;

import hu.oroscom.c1801.enumeration.RoleName;
import org.springframework.security.core.GrantedAuthority;

import javax.persistence.*;
import java.io.Serializable;
import java.util.HashSet;
import java.util.NoSuchElementException;
import java.util.Set;

@Entity
public class Role extends BaseEntity implements Serializable, GrantedAuthority {
	private String roleName;

	@OneToMany(mappedBy = "role", cascade = CascadeType.ALL)
	private Set<User> users = new HashSet<>();

	@ManyToMany
	private Set<Permission> permissions = new HashSet<>();

	@ManyToOne
	private Customer customer;

	public Role() {
	}

	public Role(String roleName, Customer customer) {
		this.roleName = roleName;
		this.customer = customer;
	}

	public Role(RoleName roleName, Customer customer) {
		this.roleName = roleName.toString();
		this.customer = customer;
	}

	public String getRoleName() {
		return roleName;
	}

	public RoleName getRoleNameEnum() {
		try {
			return RoleName.fromString(roleName);
		} catch (NoSuchElementException e) {
			return null;
		}
	}

	public void setRoleName(String roleName) {
		this.roleName = roleName;
	}

	public void setRoleName(RoleName roleName) {
		this.roleName = roleName.toString();
	}

	@Override
	public String getAuthority() {
		return getRoleName().toString();
	}

	public Set<Permission> getPermissions() {
		return getActiveEntities(permissions);
	}

	public void setPermissions(Set<Permission> permissions) {
		this.permissions = permissions;
	}

	public void addPermission(Permission permission) {
		permissions.add(permission);
	}

	public void removePermission(Permission permission) {
		permissions.remove(permission);
	}

	@Override
	public String toString() {
		return roleName.toString();
	}

	public String getCaption() {
		try {
			return RoleName.fromString(roleName).getCaption();
		} catch (NoSuchElementException e) {
			return roleName;
		}
	}

	public Customer getCustomer() {
		return customer;
	}

	public void setCustomer(Customer customer) {
		this.customer = customer;
	}

	public Set<User> getUsers() {
		return users;
	}

	public void setUsers(Set<User> users) {
		this.users = users;
	}
}
