package hu.oroscom.c1801.domain;

import javax.persistence.Entity;
import javax.persistence.Lob;
import javax.persistence.ManyToOne;
import java.sql.Timestamp;
import java.util.Date;

/**
 * Created by kollerk on 2014.10.07..
 */
@Entity
public class Setting extends BaseEntity {
	private String name;

	@Lob
	private String value;

	@ManyToOne
	private Customer customer;

	public Setting() {
	}

	public Setting(Customer customer, String name, Object value) {
		this.customer = customer;
		this.name = name;
		setTypedValue(value);
	}

	public void setValue(Object value) {
		setTypedValue(value);
	}

	public <T> T getTypedValue(Class<T> targetType) {
		if (String.class.equals(targetType)) {
			return targetType.cast(value);
		}

		if (Boolean.class.equals(targetType) || Boolean.TYPE.equals(targetType)) {
			String stringValue = value.toLowerCase();
			if ("true".equals(stringValue)) {
				return targetType.cast(Boolean.TRUE);
			} else if ("false".equals(stringValue)) {
				return targetType.cast(Boolean.FALSE);
			}
		}

		if (Integer.class.equals(targetType)) {
			return targetType.cast(Integer.valueOf(value));
		}

		if (Float.class.equals(targetType)) {
			return targetType.cast(Float.valueOf(value));
		}

		if (Date.class.equals(targetType)) {
			Date date = new Date(Long.parseLong(value));
			return targetType.cast(date);
		}

		return null;
	}

	public void setTypedValue(Object value) {
		if (value instanceof String) {
			this.value = String.valueOf(value);
		}

		if (value instanceof Boolean) {
			if (value == Boolean.TRUE) {
				this.value = "true";
			} else if (value == Boolean.FALSE) {
				this.value = "false";
			}
		}

		if (value instanceof Integer) {
			this.value = String.valueOf(value);
		}

		if (value instanceof Float) {
			this.value = String.valueOf(value);
		}

		if (value instanceof Date) {
			this.value = String.valueOf(new Timestamp(((Date) value).getTime()).getTime());
		}

	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getValue() {
		return value;
	}

	public void setValue(String value) {
		this.value = value;
	}

	public Customer getCustomer() {
		return customer;
	}

	public void setCustomer(Customer customer) {
		this.customer = customer;
	}
}
