package hu.oroscom.c1801.jparepository;

import com.vaadin.server.VaadinSession;
import hu.oroscom.c1801.domain.BaseEntity;
import hu.oroscom.c1801.domain.EntityBase;
import hu.oroscom.c1801.domain.User;
import hu.oroscom.c1801.util.UserUtil;
import org.springframework.data.jpa.repository.support.CrudMethodMetadata;
import org.springframework.data.jpa.repository.support.JpaEntityInformation;
import org.springframework.data.jpa.repository.support.SimpleJpaRepository;
import org.springframework.data.repository.NoRepositoryBean;
import org.springframework.transaction.annotation.Transactional;

import javax.persistence.EntityManager;
import javax.persistence.Parameter;
import javax.persistence.StoredProcedureQuery;
import javax.persistence.TypedQuery;
import java.io.Serializable;
import java.util.HashMap;
import java.util.List;

/**
 * Created by kollerk on 14. 11. 06..
 */
@NoRepositoryBean
public class GenericRepositoryImpl<T extends EntityBase, ID extends Serializable> extends SimpleJpaRepository<T, ID> implements GenericRepository<T, ID> {

	private final JpaEntityInformation<T, ID> entityInformation;
	private final EntityManager em;

	private CrudMethodMetadata crudMethodMetadata;

	public GenericRepositoryImpl(JpaEntityInformation<T, ID> entityInformation, EntityManager entityManager) {
		super(entityInformation, entityManager);
		this.entityInformation = entityInformation;
		this.em = entityManager;
	}

    @Override
    public T merge(T entity) {
		return em.merge(entity);
	}

	@Override
	@Transactional
	public <S extends T> S save(S entity) {
		User user = null;
		if (VaadinSession.getCurrent() != null) {
			user = UserUtil.getUser();
			if (entity instanceof BaseEntity && ((BaseEntity) entity).getInsertedByUser() == null) {
				((BaseEntity) entity).setInsertedByUser(user);
			}
		}
		S saved = null;
		try {
			saved = super.save(entity);
		} catch (Exception e) {
			e.printStackTrace();
		}
		if (entity instanceof BaseEntity) {
			if (user != null) {
				user = em.find(User.class, user.getId());
				VaadinSession.getCurrent().setAttribute(User.class, user);
			}
		}
		return saved;
	}

	@Override
	public void setRepositoryMethodMetadata(CrudMethodMetadata crudMethodMetadata) {
		super.setRepositoryMethodMetadata(crudMethodMetadata);
		this.crudMethodMetadata = crudMethodMetadata;
	}

	@Transactional
	public List<T> callNamedStoredProcedureQuery(String procedureName, HashMap<String, ?> paramValues) {
		StoredProcedureQuery query = em.createNamedStoredProcedureQuery(procedureName);
		if (paramValues != null) {
			for (Parameter<?> param : query.getParameters()) {
				query.setParameter(param.getName(), paramValues.get(param.getName()));
			}
		}

		return query.getResultList();
	}

	@Transactional
	public void callNamedStoredProcedureUpdate(String procedureName, HashMap<String, ?> paramValues) {
		StoredProcedureQuery query = em.createNamedStoredProcedureQuery(procedureName);
		if (paramValues != null) {
			for (Parameter<?> param : query.getParameters()) {
				query.setParameter(param.getName(), paramValues.get(param.getName()));
			}
		}
		query.executeUpdate();
	}

	public List<T> executeQuery(String queryString, Class<T> aClass, HashMap<String, ?> parameterValues, int maxResults) {
		TypedQuery<T> query = em.createQuery(queryString, aClass);
		for (Parameter<?> parameter : query.getParameters()) {
			query.setParameter(parameter.getName(), parameterValues.get(parameter.getName()));
		}
		query.setMaxResults(maxResults);

		return query.getResultList();
	}

	public EntityManager getEntityManager() {
		return em.getEntityManagerFactory().createEntityManager();
	}
}
