package hu.oroscom.c1801.service;

import hu.oroscom.c1801.domain.Assistant;
import hu.oroscom.c1801.domain.Role;
import hu.oroscom.c1801.domain.filter.AssistantFilter;
import hu.oroscom.c1801.enumeration.PermissionName;
import hu.oroscom.c1801.enumeration.RoleName;
import hu.oroscom.c1801.jparepository.AssistantRepository;
import hu.oroscom.c1801.util.FilterItemBean;
import hu.oroscom.c1801.util.FilterUtil;
import org.springframework.stereotype.Service;

import javax.inject.Inject;
import javax.persistence.EntityManager;
import javax.persistence.TypedQuery;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * Created by Norbert on 2014.10.29..
 */
@Service
public class AssistantService extends BaseService<Assistant> {

	@Inject
	AssistantRepository assistantRepository;

	@Inject
	UserService userService;

	@Inject
	private RoleService roleService;

	@Override
	public Assistant findOne(Long id) {
		return assistantRepository.findOne(id);
	}

	@Override
	public Assistant save(Assistant entity) {
		return assistantRepository.save(entity);
	}

	@Override
	public List<Assistant> save(Iterable<Assistant> entities) {
		return assistantRepository.save(entities);
	}

	@Override
	public List<Assistant> findAll() {
		return assistantRepository.findAll();
	}

	@Override
	public void delete(Assistant entity) {
		assistantRepository.delete(entity);
	}

	@Override
	public void delete(Long id) {
		assistantRepository.delete(id);
	}

	@Override
	public void deleteAll() {
		assistantRepository.deleteAll();
	}

	public boolean userHasPermission(PermissionName... permissionsNames) {
		return userService.userHasPermission(permissionsNames);
	}

	public Role findRoleByRoleName(RoleName roleName) {
		return roleService.findRoleByName(roleName);
	}

	public List<Assistant> findAllWithFilter(AssistantFilter assistantFilter) {
		return queryBuilder(assistantFilter).getResultList();
	}

	private TypedQuery<Assistant> queryBuilder(AssistantFilter assistantFilter) {
		EntityManager em = assistantRepository.getEntityManager();

		String queryString = "SELECT DISTINCT a FROM Assistant a";
		String joinString = "";
		String whereString = " WHERE 1=1";

		if (assistantFilter.getQuickSearch() != null && !assistantFilter.getQuickSearch().isEmpty()) {
			whereString += " AND (LOWER(a.firstName) LIKE :firstName OR LOWER(a.lastName) LIKE :lastName)";
		}

		if (assistantFilter.getIds() != null && !assistantFilter.getIds().isEmpty()) {
			whereString += " AND a.id IN :ids";
		}

		if (assistantFilter.getFirstName() != null && !assistantFilter.getFirstName().isEmpty()) {
			whereString += getFilterItemCondition("a.firstName", assistantFilter.getFirstName());
		}

		if (assistantFilter.getLastName() != null && !assistantFilter.getLastName().isEmpty()) {
			whereString += getFilterItemCondition("a.lastName", assistantFilter.getLastName());
		}

		queryString += joinString + whereString;

		TypedQuery<Assistant> typedQuery = em.createQuery(queryString, Assistant.class);
		typedQuery.setHint("eclipselink.read-only", "true");

		if (assistantFilter.getQuickSearch() != null && !assistantFilter.getQuickSearch().isEmpty()) {
			typedQuery.setParameter("firstName", "%" + assistantFilter.getQuickSearch().toLowerCase() + "%");
			typedQuery.setParameter("lastName", "%" + assistantFilter.getQuickSearch().toLowerCase() + "%");
		}

		if (assistantFilter.getIds() != null && !assistantFilter.getIds().isEmpty()) {
			Set<Long> entityIds = new HashSet<>();
			for (String filterItem : assistantFilter.getIds()) {
				FilterItemBean filterItemBean = FilterUtil.getFilterItem(filterItem);
				entityIds.add(Long.valueOf(filterItemBean.getFilterValue()));
			}

			typedQuery.setParameter("ids", entityIds);
		}

		return typedQuery;
	}
}
