package hu.oroscom.c1801.service;

import hu.oroscom.c1801.domain.EntityBase;
import hu.oroscom.c1801.domain.User;
import hu.oroscom.c1801.enumeration.EntityStatus;
import hu.oroscom.c1801.enumeration.ServiceManagementNature;
import hu.oroscom.c1801.enumeration.ServiceManagementStatus;
import hu.oroscom.c1801.enumeration.StudyEntryType;
import hu.oroscom.c1801.util.FilterItemBean;
import hu.oroscom.c1801.util.FilterUtil;
import org.joda.time.DateTime;
import org.joda.time.format.DateTimeFormat;
import org.joda.time.format.DateTimeFormatter;
import org.springframework.stereotype.Service;

import javax.persistence.TypedQuery;
import java.util.List;
import java.util.Set;

/**
 * Created by Norbert on 2014.10.22..
 */
@Service
public abstract class BaseService<T extends EntityBase> {
	private static DateTimeFormatter dateTimeFormatter = DateTimeFormat.forPattern("yyyyMMdd");

	public abstract T save(T entity);

	public abstract List<T> save(Iterable<T> entities);

	public abstract T findOne(Long id);

	public abstract List<T> findAll();

	public abstract void delete(T entity);

	public abstract void delete(Long id);

	public abstract void deleteAll();

	protected String getFilterItemCondition(String columnName, List<String> filterItems) {
		return getFilterItemCondition(columnName, filterItems, String.class);
	}

	protected String getFilterItemCondition(String columnName, List<String> filterItems, Class aClass) {
		return getFilterItemCondition(columnName, filterItems, aClass, null);
	}

	protected String getFilterItemCondition(String columnName, List<String> filterItems, Class aClass, String parameterName) {
		String whereString = " AND (";
		String condition = "";
		int index = 0;
		for (String filterItem : filterItems) {
			FilterItemBean filterItemBean = FilterUtil.getFilterItem(filterItem);
			if (String.class.equals(aClass)) {
				String filterValue = filterItemBean.getFilterValue().toLowerCase();
				switch (filterItemBean.getFilterOperand()) {
					case CONTAINS:
						condition += " OR LOWER(" + columnName + ") LIKE '%" + filterValue + "%'";
						break;
					case DOES_NOT_CONTAIN:
						condition += " OR LOWER(" + columnName + ") NOT LIKE '%" + filterValue + "%'";
						break;
					case ENDS_WITH:
						condition += " OR LOWER(" + columnName + ") LIKE '%" + filterValue + "'";
						break;
					case STARTS_WITH:
						condition += " OR LOWER(" + columnName + ") LIKE '" + filterValue + "%'";
						break;
					case DIFFERENT_FROM:
						condition += " OR LOWER(" + columnName + ") <> '" + filterValue + "'";
						break;
					case EQUAL_TO:
						condition += " OR LOWER(" + columnName + ") = '" + filterValue + "'";
						break;
				}
			} else if (Integer.class.equals(aClass) || Float.class.equals(aClass)) {
				String filterValue = filterItemBean.getFilterValue();
				switch (filterItemBean.getFilterOperand()) {
					case EQUAL_TO:
						if (filterValue.isEmpty()) {
							condition += " OR " + columnName + " IS NULL";
						} else {
							condition += " OR " + columnName + " = " + filterValue;
						}
						break;
					case DIFFERENT_FROM:
						if (filterValue.isEmpty()) {
							condition += " OR " + columnName + " IS NOT NULL";
						} else {
							condition += " OR " + columnName + " <> " + filterValue;
						}
						break;
					case GREATER_THAN:
						if (!filterValue.isEmpty()) {
							condition += " OR " + columnName + " > " + filterValue;
						}
						break;
					case LESS_THAN:
						if (!filterValue.isEmpty()) {
							condition += " OR " + columnName + " < " + filterValue;
						}
						break;
				}
			} else if (DateTime.class.equals(aClass)) {
				parameterName = parameterName == null ? "" : parameterName;

				switch (filterItemBean.getFilterOperand()) {
					case EQUAL_TO:
						condition += " OR " + columnName + " BETWEEN :" + parameterName + "start" + index + " AND :" + parameterName + "end" + index;
						break;
					case DIFFERENT_FROM:
						condition += " OR " + columnName + " NOT BETWEEN :" + parameterName + "start" + index + " AND :" + parameterName + "end" + index;
						break;
					case GREATER_THAN:
						condition += " OR " + columnName + " > :" + parameterName + index;
						break;
					case LESS_THAN:
						condition += " OR " + columnName + " < :" + parameterName + index;
						break;
					case INTERVAL:
						condition += " OR " + columnName + " BETWEEN :" + parameterName + "start" + index + " AND :" + parameterName + "end" + index;
						break;
				}
			} else if (Boolean.class.equals(aClass)) {
				String filterValue = filterItemBean.getFilterValue().toLowerCase();
				switch (filterItemBean.getFilterOperand()) {
					case DIFFERENT_FROM:
						condition += " OR " + columnName + " <> " + filterValue;
						break;
					case EQUAL_TO:
						condition += " OR " + columnName + " = " + filterValue;
						break;
				}
			} else if (StudyEntryType.class.equals(aClass)) {
				switch (filterItemBean.getFilterOperand()) {
					case EQUAL_TO:
						condition += " OR " + columnName + " = :studyEntryType" + index;
						break;
					case DIFFERENT_FROM:
						condition += " OR " + columnName + " <> :studyEntryType" + index;
						break;
				}
			} else if (User.class.equals(aClass)) {
				switch (filterItemBean.getFilterOperand()) {
					case EQUAL_TO:
						condition += " OR " + columnName + " = :user" + index;
						break;
					case DIFFERENT_FROM:
						condition += " OR " + columnName + " <> :user" + index;
						break;
				}
			}  else if (EntityStatus.class.equals(aClass)) {
				parameterName = parameterName == null ? "" : parameterName;
				switch (filterItemBean.getFilterOperand()) {
					case EQUAL_TO:
						condition += " OR " + columnName + " = :" + parameterName + index;
						break;
					case DIFFERENT_FROM:
						condition += " OR " + columnName + " <> :" + parameterName + index;
						break;
				}
			} else if (ServiceManagementStatus.class.equals(aClass)) {
				switch (filterItemBean.getFilterOperand()) {
					case EQUAL_TO:
						condition += " OR " + columnName + " = :serviceManagementStatus" + index;
						break;
					case DIFFERENT_FROM:
						condition += " OR " + columnName + " <> :serviceManagementStatus" + index;
						break;
				}
			} else if (ServiceManagementNature.class.equals(aClass)) {
				switch (filterItemBean.getFilterOperand()) {
					case EQUAL_TO:
						condition += " OR " + columnName + " = :serviceManagementNature" + index;
						break;
					case DIFFERENT_FROM:
						condition += " OR " + columnName + " <> :serviceManagementNature" + index;
						break;
				}
			}
			index++;
		}
		condition = condition.replaceFirst(" OR ", "");
		whereString += condition;
		whereString += ")";
		return whereString;
	}

	protected void setupDateTimeParameters(TypedQuery<T> deletedBookingTypedQuery, List<String> filterField, String parameterName) {
		int index = 0;
		for (String filterItem : filterField) {
			FilterItemBean filterItemBean = FilterUtil.getFilterItem(filterItem);
			switch (filterItemBean.getFilterOperand()) {
				case EQUAL_TO:
				case DIFFERENT_FROM:
					DateTime startDateTime = DateTime.parse(filterItemBean.getFilterValue(), dateTimeFormatter).withTime(0, 0, 0, 0);
					DateTime endDateTime = startDateTime.withTime(23, 59, 59, 999);
					deletedBookingTypedQuery.setParameter(parameterName + "start" + index, startDateTime);
					deletedBookingTypedQuery.setParameter(parameterName + "end" + index, endDateTime);
					break;
				case GREATER_THAN:
					deletedBookingTypedQuery.setParameter(parameterName + index, DateTime.parse(filterItemBean.getFilterValue(), dateTimeFormatter).withTime(23, 59, 59, 999));
					break;
				case LESS_THAN:
					deletedBookingTypedQuery.setParameter(parameterName + index, DateTime.parse(filterItemBean.getFilterValue(), dateTimeFormatter).withTime(0, 0, 0, 0));
					break;
				case INTERVAL:
					String[] dateStringArray = filterItemBean.getFilterValue().split("sep");
					startDateTime = DateTime.parse(dateStringArray[0], dateTimeFormatter).withTimeAtStartOfDay();
					endDateTime = DateTime.parse(dateStringArray[1], dateTimeFormatter).withTime(23, 59, 59, 999);
					deletedBookingTypedQuery.setParameter(parameterName + "start" + index, startDateTime);
					deletedBookingTypedQuery.setParameter(parameterName + "end" + index, endDateTime);
					break;
			}

			index++;
		}
	}

	protected void waitForThreads(String excludedThreadName, String... threadNames) {
		for (String threadName : threadNames) {
			boolean isRunning = false;
			Set<Thread> threadSet = Thread.getAllStackTraces().keySet();
			for (Thread thread : threadSet) {
				if (!excludedThreadName.equals(threadName) && threadName.contains(thread.getName())) {
					isRunning = true;
				}
			}

			while (isRunning) {
				threadSet = Thread.getAllStackTraces().keySet();
				boolean hasRunningThread = false;
				for (Thread thread : threadSet) {
					if (!excludedThreadName.equals(threadName) && threadName.contains(thread.getName())) {
						hasRunningThread = true;
					}
				}
				isRunning = hasRunningThread;
			}
		}
	}
}
