package hu.oroscom.c1801.service;

import hu.oroscom.c1801.domain.Doctor;
import hu.oroscom.c1801.domain.Role;
import hu.oroscom.c1801.domain.filter.DoctorFilter;
import hu.oroscom.c1801.enumeration.PermissionName;
import hu.oroscom.c1801.enumeration.RoleName;
import hu.oroscom.c1801.jparepository.DoctorRepository;
import hu.oroscom.c1801.util.FilterItemBean;
import hu.oroscom.c1801.util.FilterUtil;
import hu.oroscom.c1801.util.UserUtil;
import org.springframework.stereotype.Service;

import javax.inject.Inject;
import javax.persistence.EntityManager;
import javax.persistence.TypedQuery;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * Created by Norbert on 2014.10.29..
 */
@Service
public class DoctorService extends BaseService<Doctor> {

	@Inject
	DoctorRepository doctorRepository;

	@Inject
	UserService userService;

	@Inject
	private RoleService roleService;

	@Override
	public Doctor findOne(Long id) {
		return doctorRepository.findOne(id);
	}

	@Override
	public Doctor save(Doctor entity) {
		return doctorRepository.save(entity);
	}

	@Override
	public List<Doctor> save(Iterable<Doctor> entities) {
		return doctorRepository.save(entities);
	}

	@Override
	public List<Doctor> findAll() {
		return doctorRepository.findAll();
	}

	@Override
	public void delete(Doctor entity) {
		doctorRepository.delete(entity);
	}

	@Override
	public void delete(Long id) {
		doctorRepository.delete(id);
	}

	@Override
	public void deleteAll() {
		doctorRepository.deleteAll();
	}

	public boolean userHasPermission(PermissionName... permissionsNames) {
		return userService.userHasPermission(permissionsNames);
	}

	public Role findRoleByRoleName(RoleName roleName) {
		return roleService.findRoleByName(roleName);
	}

	public Set<Doctor> findAllByCustomer() {
		return doctorRepository.findAllByCustomer(UserUtil.getCustomer());
	}

	public List<Doctor> findAllWithFilter(DoctorFilter doctorFilter) {
		return queryBuilder(doctorFilter).getResultList();
	}

	private TypedQuery<Doctor> queryBuilder(DoctorFilter doctorFilter) {
		EntityManager em = doctorRepository.getEntityManager();

		String queryString = "SELECT DISTINCT d FROM Doctor d";
		String joinString = "";
		String whereString = " WHERE 1=1";

		if (doctorFilter.getQuickSearch() != null && !doctorFilter.getQuickSearch().isEmpty()) {
			whereString += " AND (LOWER(p.firstName) LIKE :firstName OR LOWER(p.lastName) LIKE :lastName)";
		}

		if (doctorFilter.getIds() != null && !doctorFilter.getIds().isEmpty()) {
			whereString += " AND d.id IN :ids";
		}

		if (doctorFilter.getFirstName() != null && !doctorFilter.getFirstName().isEmpty()) {
			whereString += getFilterItemCondition("d.firstName", doctorFilter.getFirstName());
		}

		if (doctorFilter.getLastName() != null && !doctorFilter.getLastName().isEmpty()) {
			whereString += getFilterItemCondition("d.lastName", doctorFilter.getLastName());
		}

		queryString += joinString + whereString;

		TypedQuery<Doctor> typedQuery = em.createQuery(queryString, Doctor.class);
		typedQuery.setHint("eclipselink.read-only", "true");

		if (doctorFilter.getQuickSearch() != null && !doctorFilter.getQuickSearch().isEmpty()) {
			typedQuery.setParameter("firstName", "%" + doctorFilter.getQuickSearch().toLowerCase() + "%");
			typedQuery.setParameter("lastName", "%" + doctorFilter.getQuickSearch().toLowerCase() + "%");
		}

		if (doctorFilter.getIds() != null && !doctorFilter.getIds().isEmpty()) {
			Set<Long> entityIds = new HashSet<>();
			for (String filterItem : doctorFilter.getIds()) {
				FilterItemBean filterItemBean = FilterUtil.getFilterItem(filterItem);
				entityIds.add(Long.valueOf(filterItemBean.getFilterValue()));
			}

			typedQuery.setParameter("ids", entityIds);
		}

		return typedQuery;
	}
}
