package hu.oroscom.c1801.service;

import hu.oroscom.c1801.domain.EatenFood;
import hu.oroscom.c1801.domain.Patient;
import hu.oroscom.c1801.domain.filter.EatenFoodFilter;
import hu.oroscom.c1801.enumeration.PermissionName;
import hu.oroscom.c1801.jparepository.EatenFoodRepository;
import hu.oroscom.c1801.util.FilterItemBean;
import hu.oroscom.c1801.util.FilterUtil;
import org.joda.time.DateTime;
import org.springframework.stereotype.Service;

import javax.inject.Inject;
import javax.persistence.EntityManager;
import javax.persistence.TypedQuery;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * Created by Norbert on 2014.10.29..
 */
@Service
public class EatenFoodService extends BaseService<EatenFood> {

	@Inject
	EatenFoodRepository eatenFoodRepository;

	@Inject
	UserService userService;

	@Override
	public EatenFood findOne(Long id) {
		return eatenFoodRepository.findOne(id);
	}

	@Override
	public EatenFood save(EatenFood entity) {
		return eatenFoodRepository.save(entity);
	}

	@Override
	public List<EatenFood> save(Iterable<EatenFood> entities) {
		return eatenFoodRepository.save(entities);
	}

	@Override
	public List<EatenFood> findAll() {
		return eatenFoodRepository.findAll();
	}

	@Override
	public void delete(EatenFood entity) {
		eatenFoodRepository.delete(entity);
	}

	@Override
	public void delete(Long id) {
		eatenFoodRepository.delete(id);
	}

	@Override
	public void deleteAll() {
		eatenFoodRepository.deleteAll();
	}

	public boolean userHasPermission(PermissionName... permissionsNames) {
		return userService.userHasPermission(permissionsNames);
	}

	public List<EatenFood> findAllByPatient(Patient patient) {
		return eatenFoodRepository.findAllByPatient(patient);
	}

	public List<EatenFood> findAllWithFilter(EatenFoodFilter eatenFoodFilter) {
		return queryBuilder(eatenFoodFilter).getResultList();
	}

	private TypedQuery<EatenFood> queryBuilder(EatenFoodFilter eatenFoodFilter) {
		EntityManager em = eatenFoodRepository.getEntityManager();

		String queryString = "SELECT DISTINCT ef FROM EatenFood ef";
		String joinString = "";
		String whereString = " WHERE ef.patient = :patient";

		if (eatenFoodFilter.getQuickSearch() != null && !eatenFoodFilter.getQuickSearch().isEmpty()) {
			whereString += " AND (LOWER(ef.description) LIKE :description)";
		}

		if (eatenFoodFilter.getIds() != null && !eatenFoodFilter.getIds().isEmpty()) {
			whereString += " AND ef.id IN :ids";
		}

		if (eatenFoodFilter.getDescription() != null && !eatenFoodFilter.getDescription().isEmpty()) {
			whereString += getFilterItemCondition("ef.description", eatenFoodFilter.getDescription());
		}

		if (eatenFoodFilter.getDateTime() != null && !eatenFoodFilter.getDateTime().isEmpty()) {
			whereString += getFilterItemCondition("ef.dateTime", eatenFoodFilter.getDateTime(), DateTime.class, "time");
		}

		queryString += joinString + whereString;

		TypedQuery<EatenFood> typedQuery = em.createQuery(queryString, EatenFood.class);
		typedQuery.setHint("eclipselink.read-only", "true");
		typedQuery.setParameter("patient", eatenFoodFilter.getPatient());

		if (eatenFoodFilter.getQuickSearch() != null && !eatenFoodFilter.getQuickSearch().isEmpty()) {
			typedQuery.setParameter("description", "%" + eatenFoodFilter.getQuickSearch().toLowerCase() + "%");
		}

		if (eatenFoodFilter.getIds() != null && !eatenFoodFilter.getIds().isEmpty()) {
			Set<Long> entityIds = new HashSet<>();
			for (String filterItem : eatenFoodFilter.getIds()) {
				FilterItemBean filterItemBean = FilterUtil.getFilterItem(filterItem);
				entityIds.add(Long.valueOf(filterItemBean.getFilterValue()));
			}

			typedQuery.setParameter("ids", entityIds);
		}

		if (eatenFoodFilter.getDateTime() != null && !eatenFoodFilter.getDateTime().isEmpty()) {
			setupDateTimeParameters(typedQuery, eatenFoodFilter.getDateTime(), "time");
		}

		return typedQuery;
	}
}
