package hu.oroscom.c1801.service;

import hu.oroscom.c1801.domain.ExcelExportLog;
import hu.oroscom.c1801.domain.Setting;
import hu.oroscom.c1801.domain.filter.ExcelExportLogFilter;
import hu.oroscom.c1801.enumeration.EntityStatus;
import hu.oroscom.c1801.enumeration.PermissionName;
import hu.oroscom.c1801.enumeration.SettingName;
import hu.oroscom.c1801.jparepository.ExcelExportLogRepository;
import hu.oroscom.c1801.util.FilterItemBean;
import hu.oroscom.c1801.util.FilterUtil;
import hu.oroscom.c1801.util.UserUtil;
import org.joda.time.DateTime;
import org.springframework.stereotype.Service;

import javax.inject.Inject;
import javax.persistence.EntityManager;
import javax.persistence.TypedQuery;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * Created by Norbert on 2016. 08. 01..
 */
@Service
public class ExcelExportLogService extends BaseService<ExcelExportLog> {

	@Inject
	ExcelExportLogRepository excelExportLogRepository;

	@Inject
	UserService userService;

	@Inject
	SettingService settingService;

	@Override
	public ExcelExportLog save(ExcelExportLog entity) {
		return excelExportLogRepository.save(entity);
	}

	@Override
	public List<ExcelExportLog> save(Iterable<ExcelExportLog> entities) {
		return excelExportLogRepository.save(entities);
	}

	@Override
	public ExcelExportLog findOne(Long id) {
		return excelExportLogRepository.findOne(id);
	}

	@Override
	public List<ExcelExportLog> findAll() {
		return excelExportLogRepository.findAll();
	}

	@Override
	public void delete(ExcelExportLog entity) {
		excelExportLogRepository.delete(entity);
	}

	@Override
	public void delete(Long id) {
		excelExportLogRepository.delete(id);
	}

	@Override
	public void deleteAll() {
		excelExportLogRepository.deleteAll();
	}

	public List<ExcelExportLog> findAllByCustomer() {
		return excelExportLogRepository.findAllByCustomer(UserUtil.getCustomer());
	}

	public boolean userHasPermission(PermissionName... permissionsNames) {
		return userService.userHasPermission(permissionsNames);
	}

	public Setting getSettingByName(SettingName settingName) {
		return settingService.findSetting(settingName);
	}

	public List<ExcelExportLog> findAllWithFilter(ExcelExportLogFilter excelExportLogFilter) {
		return queryBuilder(excelExportLogFilter).getResultList();
	}

	private TypedQuery<ExcelExportLog> queryBuilder(ExcelExportLogFilter excelExportLogFilter) {
		EntityManager em = excelExportLogRepository.getEntityManager();

		String queryString = "SELECT DISTINCT eel FROM ExcelExportLog eel";
		String joinString = "";
		String whereString = " WHERE eel.status = :status AND eel.customer = :customer";

		if (excelExportLogFilter.getQuickSearch() != null && !excelExportLogFilter.getQuickSearch().isEmpty()) {
			whereString += " AND LOWER(eel.viewName) LIKE :viewName OR LOWER(eel.exportedUserName) LIKE :exportedUserName";
		}

		if (excelExportLogFilter.getIds() != null && !excelExportLogFilter.getIds().isEmpty()) {
			whereString += " AND eel.id IN :ids";
		}

		if (excelExportLogFilter.getExportedUserName() != null && !excelExportLogFilter.getExportedUserName().isEmpty()) {
			whereString += getFilterItemCondition("eel.exportedUserName", excelExportLogFilter.getExportedUserName());
		}

		if (excelExportLogFilter.getExportDate() != null && !excelExportLogFilter.getExportDate().isEmpty()) {
			whereString += getFilterItemCondition("eel.exportDate", excelExportLogFilter.getExportDate(), DateTime.class, "exportDate");
		}

		if (excelExportLogFilter.getViewName() != null && !excelExportLogFilter.getViewName().isEmpty()) {
			whereString += getFilterItemCondition("eel.viewName", excelExportLogFilter.getViewName());
		}

		if (excelExportLogFilter.getExportedRowCount() != null && !excelExportLogFilter.getExportedRowCount().isEmpty()) {
			whereString += getFilterItemCondition("eel.exportedRowCount", excelExportLogFilter.getExportedRowCount(), Integer.class);
		}

		queryString += joinString + whereString + " ORDER BY eel.exportDate DESC";

		TypedQuery<ExcelExportLog> excelExportLogTypedQuery = em.createQuery(queryString, ExcelExportLog.class);
		excelExportLogTypedQuery.setHint("eclipselink.read-only", "true");
		excelExportLogTypedQuery.setParameter("status", EntityStatus.ACTIVE);
		excelExportLogTypedQuery.setParameter("customer", UserUtil.getCustomer());

		if (excelExportLogFilter.getQuickSearch() != null && !excelExportLogFilter.getQuickSearch().isEmpty()) {
			excelExportLogTypedQuery.setParameter("viewName", "%" + excelExportLogFilter.getQuickSearch().toLowerCase() + "%");
			excelExportLogTypedQuery.setParameter("exportedUserName", "%" + excelExportLogFilter.getQuickSearch().toLowerCase() + "%");
		}

		if (excelExportLogFilter.getIds() != null && !excelExportLogFilter.getIds().isEmpty()) {
			Set<Long> deletedBookingIds = new HashSet<>();
			for (String filterItem : excelExportLogFilter.getIds()) {
				FilterItemBean filterItemBean = FilterUtil.getFilterItem(filterItem);
				deletedBookingIds.add(Long.valueOf(filterItemBean.getFilterValue()));
			}

			excelExportLogTypedQuery.setParameter("ids", deletedBookingIds);
		}

		if (excelExportLogFilter.getExportDate() != null && !excelExportLogFilter.getExportDate().isEmpty()) {
			setupDateTimeParameters(excelExportLogTypedQuery, excelExportLogFilter.getExportDate(), "exportDate");
		}

		return excelExportLogTypedQuery;
	}
}
