package hu.oroscom.c1801.service;

import hu.oroscom.c1801.domain.IntolerantMaterial;
import hu.oroscom.c1801.domain.Patient;
import hu.oroscom.c1801.domain.filter.IntolerantMaterialFilter;
import hu.oroscom.c1801.enumeration.PermissionName;
import hu.oroscom.c1801.jparepository.IntolerantMaterialRepository;
import hu.oroscom.c1801.util.FilterItemBean;
import hu.oroscom.c1801.util.FilterUtil;
import org.joda.time.DateTime;
import org.springframework.stereotype.Service;

import javax.inject.Inject;
import javax.persistence.EntityManager;
import javax.persistence.TypedQuery;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * Created by Norbert on 2014.10.29..
 */
@Service
public class IntolerantMaterialService extends BaseService<IntolerantMaterial> {

	@Inject
	IntolerantMaterialRepository intolerantMaterialRepository;

	@Inject
	UserService userService;

	@Override
	public IntolerantMaterial findOne(Long id) {
		return intolerantMaterialRepository.findOne(id);
	}

	@Override
	public IntolerantMaterial save(IntolerantMaterial entity) {
		return intolerantMaterialRepository.save(entity);
	}

	@Override
	public List<IntolerantMaterial> save(Iterable<IntolerantMaterial> entities) {
		return intolerantMaterialRepository.save(entities);
	}

	@Override
	public List<IntolerantMaterial> findAll() {
		return intolerantMaterialRepository.findAll();
	}

	@Override
	public void delete(IntolerantMaterial entity) {
		intolerantMaterialRepository.delete(entity);
	}

	@Override
	public void delete(Long id) {
		intolerantMaterialRepository.delete(id);
	}

	@Override
	public void deleteAll() {
		intolerantMaterialRepository.deleteAll();
	}

	public boolean userHasPermission(PermissionName... permissionsNames) {
		return userService.userHasPermission(permissionsNames);
	}

	public List<IntolerantMaterial> findAllByPatient(Patient patient) {
		return intolerantMaterialRepository.findAllByPatient(patient);
	}
	public List<IntolerantMaterial> findAllWithFilter(IntolerantMaterialFilter intolerantMaterialFilter) {
		return queryBuilder(intolerantMaterialFilter).getResultList();
	}

	private TypedQuery<IntolerantMaterial> queryBuilder(IntolerantMaterialFilter intolerantMaterialFilter) {
		EntityManager em = intolerantMaterialRepository.getEntityManager();

		String queryString = "SELECT DISTINCT im FROM IntolerantMaterial im";
		String joinString = "";
		String whereString = " WHERE im.patient = :patient";

		if (intolerantMaterialFilter.getQuickSearch() != null && !intolerantMaterialFilter.getQuickSearch().isEmpty()) {
			whereString += " AND (LOWER(im.name) LIKE :name OR LOWER(im.description) LIKE :description)";
		}

		if (intolerantMaterialFilter.getIds() != null && !intolerantMaterialFilter.getIds().isEmpty()) {
			whereString += " AND im.id IN :ids";
		}

		if (intolerantMaterialFilter.getName() != null && !intolerantMaterialFilter.getName().isEmpty()) {
			whereString += getFilterItemCondition("im.name", intolerantMaterialFilter.getName());
		}

		if (intolerantMaterialFilter.getQuantity() != null && !intolerantMaterialFilter.getQuantity().isEmpty()) {
			whereString += getFilterItemCondition("im.quantity", intolerantMaterialFilter.getQuantity(), Integer.class);
		}

		if (intolerantMaterialFilter.getDescription() != null && !intolerantMaterialFilter.getDescription().isEmpty()) {
			whereString += getFilterItemCondition("im.description", intolerantMaterialFilter.getDescription());
		}

		if (intolerantMaterialFilter.getDateTime() != null && !intolerantMaterialFilter.getDateTime().isEmpty()) {
			whereString += getFilterItemCondition("im.dateTime", intolerantMaterialFilter.getDateTime(), DateTime.class, "time");
		}

		queryString += joinString + whereString;

		TypedQuery<IntolerantMaterial> typedQuery = em.createQuery(queryString, IntolerantMaterial.class);
		typedQuery.setHint("eclipselink.read-only", "true");
		typedQuery.setParameter("patient", intolerantMaterialFilter.getPatient());

		if (intolerantMaterialFilter.getQuickSearch() != null && !intolerantMaterialFilter.getQuickSearch().isEmpty()) {
			typedQuery.setParameter("name", "%" + intolerantMaterialFilter.getQuickSearch().toLowerCase() + "%");
			typedQuery.setParameter("description", "%" + intolerantMaterialFilter.getQuickSearch().toLowerCase() + "%");
		}

		if (intolerantMaterialFilter.getIds() != null && !intolerantMaterialFilter.getIds().isEmpty()) {
			Set<Long> entityIds = new HashSet<>();
			for (String filterItem : intolerantMaterialFilter.getIds()) {
				FilterItemBean filterItemBean = FilterUtil.getFilterItem(filterItem);
				entityIds.add(Long.valueOf(filterItemBean.getFilterValue()));
			}

			typedQuery.setParameter("ids", entityIds);
		}

		if (intolerantMaterialFilter.getDateTime() != null && !intolerantMaterialFilter.getDateTime().isEmpty()) {
			setupDateTimeParameters(typedQuery, intolerantMaterialFilter.getDateTime(), "time");
		}

		return typedQuery;
	}
}
