package hu.oroscom.c1801.service;

import hu.oroscom.c1801.domain.Doctor;
import hu.oroscom.c1801.domain.Patient;
import hu.oroscom.c1801.domain.Role;
import hu.oroscom.c1801.domain.User;
import hu.oroscom.c1801.domain.filter.PatientFilter;
import hu.oroscom.c1801.enumeration.PermissionName;
import hu.oroscom.c1801.enumeration.RoleName;
import hu.oroscom.c1801.jparepository.PatientRepository;
import hu.oroscom.c1801.util.FilterItemBean;
import hu.oroscom.c1801.util.FilterUtil;
import hu.oroscom.c1801.util.UserUtil;
import org.springframework.stereotype.Service;

import javax.inject.Inject;
import javax.persistence.EntityManager;
import javax.persistence.TypedQuery;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * Created by Norbert on 2014.10.29..
 */
@Service
public class PatientService extends BaseService<Patient> {

	@Inject
	private PatientRepository patientRepository;

	@Inject
	private UserService userService;

	@Inject
	private RoleService roleService;

	@Inject
	private DoctorService doctorService;

	@Override
	public Patient findOne(Long id) {
		return patientRepository.findOne(id);
	}

	@Override
	public Patient save(Patient entity) {
		return patientRepository.save(entity);
	}

	@Override
	public List<Patient> save(Iterable<Patient> entities) {
		return patientRepository.save(entities);
	}

	@Override
	public List<Patient> findAll() {
		return patientRepository.findAll();
	}

	@Override
	public void delete(Patient entity) {
		patientRepository.delete(entity);
	}

	@Override
	public void delete(Long id) {
		patientRepository.delete(id);
	}

	@Override
	public void deleteAll() {
		patientRepository.deleteAll();
	}

	public boolean userHasPermission(PermissionName... permissionsNames) {
		return userService.userHasPermission(permissionsNames);
	}

	public Role findRoleByRoleName(RoleName roleName) {
		return roleService.findRoleByName(roleName);
	}

	public Set<Patient> findAll(Long... ids) {
		return patientRepository.findAll(ids);
	}

	public List<Patient> findAllWithFilter(PatientFilter patientFilter) {
		return queryBuilder(patientFilter).getResultList();
	}

	private TypedQuery<Patient> queryBuilder(PatientFilter patientFilter) {
		EntityManager em = patientRepository.getEntityManager();

		String queryString = "SELECT DISTINCT p FROM Patient p";
		String joinString = "";
		String whereString = " WHERE p.customer = :customer";

		if (patientFilter.getQuickSearch() != null && !patientFilter.getQuickSearch().isEmpty()) {
			whereString += " AND (p.patientId LIKE :patientId)";
		}

		if (patientFilter.getIds() != null && !patientFilter.getIds().isEmpty()) {
			whereString += " AND p.id IN :ids";
		}

		if (patientFilter.getPatientId() != null && !patientFilter.getPatientId().isEmpty()) {
			whereString += getFilterItemCondition("p.patientId", patientFilter.getPatientId());
		}

		if (patientFilter.getDoctor() != null && !patientFilter.getDoctor().isEmpty()) {
			whereString += getFilterItemCondition("p.doctor", patientFilter.getDoctor(), User.class);
		}

		queryString += joinString + whereString;

		TypedQuery<Patient> typedQuery = em.createQuery(queryString, Patient.class);
		typedQuery.setHint("eclipselink.read-only", "true");
		typedQuery.setParameter("customer", UserUtil.getCustomer());

		if (patientFilter.getQuickSearch() != null && !patientFilter.getQuickSearch().isEmpty()) {
			typedQuery.setParameter("patientId", "%" + patientFilter.getQuickSearch().toLowerCase() + "%");
		}

		if (patientFilter.getIds() != null && !patientFilter.getIds().isEmpty()) {
			Set<Long> entityIds = new HashSet<>();
			for (String filterItem : patientFilter.getIds()) {
				FilterItemBean filterItemBean = FilterUtil.getFilterItem(filterItem);
				entityIds.add(Long.valueOf(filterItemBean.getFilterValue()));
			}

			typedQuery.setParameter("ids", entityIds);
		}

		if (patientFilter.getDoctor() != null && !patientFilter.getDoctor().isEmpty()) {
			int index = 0;
			for (String filterItem : patientFilter.getDoctor()) {
				FilterItemBean filterItemBean = FilterUtil.getFilterItem(filterItem);
				Doctor filterValue = doctorService.findOne(Long.valueOf(filterItemBean.getFilterValue()));
				typedQuery.setParameter("user" + index, filterValue);
				index++;
			}
		}

		return typedQuery;
	}
}
