package hu.oroscom.c1801.service;

import hu.oroscom.c1801.domain.ServiceGroup;
import hu.oroscom.c1801.domain.filter.ServiceGroupFilter;
import hu.oroscom.c1801.enumeration.EntityStatus;
import hu.oroscom.c1801.enumeration.PermissionName;
import hu.oroscom.c1801.jparepository.ServiceGroupRepository;
import hu.oroscom.c1801.util.FilterItemBean;
import hu.oroscom.c1801.util.FilterUtil;
import hu.oroscom.c1801.util.UserUtil;
import org.springframework.stereotype.Service;

import javax.inject.Inject;
import javax.persistence.EntityManager;
import javax.persistence.TypedQuery;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * Created by Norbert on 2014.10.29..
 */
@Service
public class ServiceGroupService extends BaseService<ServiceGroup> {

	@Inject
	ServiceGroupRepository serviceGroupRepository;

	@Inject
	UserService userService;

	@Override
	public ServiceGroup findOne(Long id) {
		return serviceGroupRepository.findOne(id);
	}

	@Override
	public ServiceGroup save(ServiceGroup entity) {
		return serviceGroupRepository.save(entity);
	}

	@Override
	public List<ServiceGroup> save(Iterable<ServiceGroup> entities) {
		return serviceGroupRepository.save(entities);
	}

	@Override
	public List<ServiceGroup> findAll() {
		return serviceGroupRepository.findAll();
	}

	@Override
	public void delete(ServiceGroup entity) {
		serviceGroupRepository.delete(entity);
	}

	@Override
	public void delete(Long id) {
		serviceGroupRepository.delete(id);
	}

	@Override
	public void deleteAll() {
		serviceGroupRepository.deleteAll();
	}

	public boolean userHasPermission(PermissionName... permissionsNames) {
		return userService.userHasPermission(permissionsNames);
	}

	public List<ServiceGroup> findAllByCustomer() {
		return serviceGroupRepository.findAllByCustomer(UserUtil.getCustomer(), EntityStatus.ACTIVE);
	}

	public List<ServiceGroup> findAllWithFilter(ServiceGroupFilter serviceGroupFilter) {
		return queryBuilder(serviceGroupFilter).getResultList();
	}

	private TypedQuery<ServiceGroup> queryBuilder(ServiceGroupFilter serviceGroupFilter) {
		EntityManager em = serviceGroupRepository.getEntityManager();

		String queryString = "SELECT DISTINCT sg FROM ServiceGroup sg";
		String joinString = "";
		String whereString = " WHERE sg.customer = :customer";

		if (serviceGroupFilter.getQuickSearch() != null && !serviceGroupFilter.getQuickSearch().isEmpty()) {
			whereString += " AND (LOWER(sg.name) LIKE :name)";
		}

		if (serviceGroupFilter.getIds() != null && !serviceGroupFilter.getIds().isEmpty()) {
			whereString += " AND sg.id IN :ids";
		}

		if (serviceGroupFilter.getName() != null && !serviceGroupFilter.getName().isEmpty()) {
			whereString += getFilterItemCondition("sg.name", serviceGroupFilter.getName());
		}

		queryString += joinString + whereString;

		TypedQuery<ServiceGroup> typedQuery = em.createQuery(queryString, ServiceGroup.class);
		typedQuery.setHint("eclipselink.read-only", "true");
		typedQuery.setParameter("customer", UserUtil.getCustomer());

		if (serviceGroupFilter.getQuickSearch() != null && !serviceGroupFilter.getQuickSearch().isEmpty()) {
			typedQuery.setParameter("name", "%" + serviceGroupFilter.getQuickSearch().toLowerCase() + "%");
		}

		if (serviceGroupFilter.getIds() != null && !serviceGroupFilter.getIds().isEmpty()) {
			Set<Long> entityIds = new HashSet<>();
			for (String filterItem : serviceGroupFilter.getIds()) {
				FilterItemBean filterItemBean = FilterUtil.getFilterItem(filterItem);
				entityIds.add(Long.valueOf(filterItemBean.getFilterValue()));
			}

			typedQuery.setParameter("ids", entityIds);
		}

		return typedQuery;
	}
}
