package hu.oroscom.c1801.service;

import hu.oroscom.c1801.changelog.Editor;
import hu.oroscom.c1801.domain.ServiceManagement;
import hu.oroscom.c1801.domain.User;
import hu.oroscom.c1801.domain.filter.ServiceManagementFilter;
import hu.oroscom.c1801.enumeration.PermissionName;
import hu.oroscom.c1801.enumeration.ServiceManagementNature;
import hu.oroscom.c1801.enumeration.ServiceManagementStatus;
import hu.oroscom.c1801.enumeration.StudyEntryType;
import hu.oroscom.c1801.jparepository.ServiceManagementRepository;
import hu.oroscom.c1801.util.FilterItemBean;
import hu.oroscom.c1801.util.FilterUtil;
import hu.oroscom.c1801.util.UserUtil;
import org.springframework.stereotype.Service;

import javax.inject.Inject;
import javax.persistence.EntityManager;
import javax.persistence.TypedQuery;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * Created by Norbert on 2014.10.29..
 */
@Service
public class ServiceManagementService extends BaseService<ServiceManagement> {

	@Inject
	private ServiceManagementRepository serviceManagementRepository;

	@Inject
	private UserService userService;

	@Override
	public ServiceManagement findOne(Long id) {
		return serviceManagementRepository.findOne(id);
	}

	@Override
	public ServiceManagement save(ServiceManagement entity) {
		Editor.markEditable(UserUtil.getUser(), ServiceManagement.class);
		ServiceManagement serviceManagement = serviceManagementRepository.save(entity);
		Editor.cleanupThread();
		return serviceManagement;
	}

	@Override
	public List<ServiceManagement> save(Iterable<ServiceManagement> entities) {
		return serviceManagementRepository.save(entities);
	}

	@Override
	public List<ServiceManagement> findAll() {
		return serviceManagementRepository.findAll();
	}

	@Override
	public void delete(ServiceManagement entity) {
		serviceManagementRepository.delete(entity);
	}

	@Override
	public void delete(Long id) {
		serviceManagementRepository.delete(id);
	}

	@Override
	public void deleteAll() {
		serviceManagementRepository.deleteAll();
	}

	public List<ServiceManagement> findAllByStudyEntryType(StudyEntryType studyEntryType) {
		return serviceManagementRepository.findAllByStudyEntryType(studyEntryType);
	}

	public boolean userHasPermission(PermissionName... permissionsNames) {
		return userService.userHasPermission(permissionsNames);
	}

	public List<ServiceManagement> findAllWithFilter(ServiceManagementFilter serviceManagementFilter) {
		return queryBuilder(serviceManagementFilter).getResultList();
	}

	private TypedQuery<ServiceManagement> queryBuilder(ServiceManagementFilter serviceManagementFilter) {
		EntityManager em = serviceManagementRepository.getEntityManager();

		String queryString = "SELECT DISTINCT sm FROM ServiceManagement sm";
		String joinString = "";
		String whereString = " WHERE sm.customer = :customer";

		if (serviceManagementFilter.getQuickSearch() != null && !serviceManagementFilter.getQuickSearch().isEmpty()) {
			whereString += " AND (LOWER(s.id) LIKE :id)";
		}

		if (serviceManagementFilter.getIds() != null && !serviceManagementFilter.getIds().isEmpty()) {
			whereString += " AND s.id IN :ids";
		}

		if (serviceManagementFilter.getConnectedUser() != null && !serviceManagementFilter.getConnectedUser().isEmpty()) {
			whereString += getFilterItemCondition("sm.user", serviceManagementFilter.getConnectedUser(), User.class);
		}

		if (serviceManagementFilter.getConnectedStudyEntryType() != null && !serviceManagementFilter.getConnectedStudyEntryType().isEmpty()) {
			whereString += getFilterItemCondition("sm.studyEntryType", serviceManagementFilter.getConnectedStudyEntryType(), StudyEntryType.class);
		}

		if (serviceManagementFilter.getServiceManagementNature() != null && !serviceManagementFilter.getServiceManagementNature().isEmpty()) {
			whereString += getFilterItemCondition("sm.serviceManagementNature", serviceManagementFilter.getServiceManagementNature(), ServiceManagementNature.class);
		}

		if (serviceManagementFilter.getServiceManagementStatus() != null && !serviceManagementFilter.getServiceManagementStatus().isEmpty()) {
			whereString += getFilterItemCondition("sm.serviceManagementStatus", serviceManagementFilter.getServiceManagementStatus(), ServiceManagementStatus.class);
		}

		queryString += joinString + whereString;

		TypedQuery<ServiceManagement> serviceManagementTypedQuery = em.createQuery(queryString, ServiceManagement.class);
		serviceManagementTypedQuery.setHint("eclipselink.read-only", "true");
		serviceManagementTypedQuery.setParameter("customer", UserUtil.getCustomer());

		if (serviceManagementFilter.getQuickSearch() != null && !serviceManagementFilter.getQuickSearch().isEmpty()) {
			serviceManagementTypedQuery.setParameter("id", "%" + serviceManagementFilter.getQuickSearch().toLowerCase() + "%");
		}

		if (serviceManagementFilter.getIds() != null && !serviceManagementFilter.getIds().isEmpty()) {
			Set<Long> serviceManagementIds = new HashSet<>();
			for (String filterItem : serviceManagementFilter.getIds()) {
				FilterItemBean filterItemBean = FilterUtil.getFilterItem(filterItem);
				serviceManagementIds.add(Long.valueOf(filterItemBean.getFilterValue()));
			}

			serviceManagementTypedQuery.setParameter("ids", serviceManagementIds);
		}

		if (serviceManagementFilter.getConnectedStudyEntryType() != null && !serviceManagementFilter.getConnectedStudyEntryType().isEmpty()) {
			int index = 0;
			for (String filterItem : serviceManagementFilter.getConnectedStudyEntryType()) {
				FilterItemBean filterItemBean = FilterUtil.getFilterItem(filterItem);
				StudyEntryType filterValue = StudyEntryType.valueOf(filterItemBean.getFilterValue());
				serviceManagementTypedQuery.setParameter("studyEntryType" + index, filterValue);
				index++;
			}
		}

		if (serviceManagementFilter.getConnectedUser() != null && !serviceManagementFilter.getConnectedUser().isEmpty()) {
			int index = 0;
			for (String filterItem : serviceManagementFilter.getConnectedUser()) {
				FilterItemBean filterItemBean = FilterUtil.getFilterItem(filterItem);
				User filterValue = userService.findOne(Long.valueOf(filterItemBean.getFilterValue()));
				serviceManagementTypedQuery.setParameter("user" + index, filterValue);
				index++;
			}
		}

		if (serviceManagementFilter.getServiceManagementNature() != null && !serviceManagementFilter.getServiceManagementNature().isEmpty()) {
			int index = 0;
			for (String filterItem : serviceManagementFilter.getServiceManagementNature()) {
				FilterItemBean filterItemBean = FilterUtil.getFilterItem(filterItem);
				ServiceManagementNature filterValue = ServiceManagementNature.valueOf(filterItemBean.getFilterValue());
				serviceManagementTypedQuery.setParameter("serviceManagementNature" + index, filterValue);
				index++;
			}
		}

		if (serviceManagementFilter.getServiceManagementStatus() != null && !serviceManagementFilter.getServiceManagementStatus().isEmpty()) {
			int index = 0;
			for (String filterItem : serviceManagementFilter.getServiceManagementStatus()) {
				FilterItemBean filterItemBean = FilterUtil.getFilterItem(filterItem);
				ServiceManagementStatus filterValue = ServiceManagementStatus.valueOf(filterItemBean.getFilterValue());
				serviceManagementTypedQuery.setParameter("serviceManagementStatus" + index, filterValue);
				index++;
			}
		}

		return serviceManagementTypedQuery;
	}

}
