package hu.oroscom.c1801.service;

import hu.oroscom.c1801.ProjectProperties;
import hu.oroscom.c1801.domain.Customer;
import hu.oroscom.c1801.domain.Role;
import hu.oroscom.c1801.domain.Setting;
import hu.oroscom.c1801.domain.User;
import hu.oroscom.c1801.enumeration.PermissionName;
import hu.oroscom.c1801.enumeration.RoleName;
import hu.oroscom.c1801.enumeration.SettingName;
import hu.oroscom.c1801.jparepository.SettingRepository;
import hu.oroscom.c1801.util.UserUtil;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import javax.inject.Inject;
import java.util.List;

/**
 * Created by Norbert on 2014.10.29..
 */
@Service
public class SettingService extends BaseService<Setting> {

	@Inject
	SettingRepository settingRepository;

	@Inject
	UserService userService;

	@Inject
	RoleService roleService;

	private ProjectProperties projectProperties;

	@PostConstruct
	private void init() {
		load();
	}

	private void load() {
		projectProperties = new ProjectProperties();
		projectProperties.load(findAll());
	}

	public Setting findSetting(SettingName key) {
		return projectProperties.getProperty(UserUtil.getCustomer(), key);
	}

	public List<Setting> findSetting(SettingName... key) {
		return projectProperties.getProperty(UserUtil.getCustomer(), key);
	}

	public Setting findSetting(Customer customer, SettingName key) {
		return projectProperties.getProperty(customer, key);
	}

	public List<Setting> findSetting(Customer customer, SettingName... key) {
		return projectProperties.getProperty(customer, key);
	}

	@Override
	public List<Setting> save(Iterable entities) {
		List<Setting> settingList = settingRepository.save(entities);
		projectProperties.setProperty(settingList);
		return settingList;
	}

	@Override
	public Setting findOne(Long id) {
		return settingRepository.findOne(id);
	}

	@Override
	public Setting save(Setting entity) {
		Setting setting = settingRepository.save(entity);
		projectProperties.setProperty(setting);
		return setting;
	}

	@Override
	public List<Setting> findAll() {
		return settingRepository.findAll();
	}

	@Override
	public void delete(Setting entity) {
		settingRepository.delete(entity);
	}

	@Override
	public void delete(Long id) {
		settingRepository.delete(id);
	}

	@Override
	public void deleteAll() {
		settingRepository.deleteAll();
	}

	public boolean userHasPermission(PermissionName... permissionsNames) {
		return userService.userHasPermission(permissionsNames);
	}

	public List<Role> findAllRoles() {
		return roleService.findAllWithoutRole(RoleName.ROLE_SUPER_ADMIN);
	}

	public List<User> findAllApiUsers() {
		return userService.findAllApiUsers(RoleName.ROLE_API);
	}

	public String generateApiKey(String userName, String password, Long customerId) {
		return TokenService.createToken(userName, password, customerId);
	}
}
