package hu.oroscom.c1801.service;

import hu.oroscom.c1801.domain.StudyEntryData;
import hu.oroscom.c1801.domain.filter.StudyEntryDataFilter;
import hu.oroscom.c1801.enumeration.PermissionName;
import hu.oroscom.c1801.jparepository.StudyEntryDataRepository;
import hu.oroscom.c1801.util.FilterItemBean;
import hu.oroscom.c1801.util.FilterUtil;
import hu.oroscom.c1801.util.UserUtil;
import org.springframework.stereotype.Service;

import javax.inject.Inject;
import javax.persistence.EntityManager;
import javax.persistence.TypedQuery;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * Created by Norbert on 2014.10.29..
 */
@Service
public class StudyEntryDataService extends BaseService<StudyEntryData> {

	@Inject
	StudyEntryDataRepository studyEntryDataRepository;

	@Inject
	UserService userService;

	@Override
	public StudyEntryData findOne(Long id) {
		return studyEntryDataRepository.findOne(id);
	}

	@Override
	public StudyEntryData save(StudyEntryData entity) {
		return studyEntryDataRepository.save(entity);
	}

	@Override
	public List<StudyEntryData> save(Iterable<StudyEntryData> entities) {
		return studyEntryDataRepository.save(entities);
	}

	@Override
	public List<StudyEntryData> findAll() {
		return studyEntryDataRepository.findAll();
	}

	@Override
	public void delete(StudyEntryData entity) {
		studyEntryDataRepository.delete(entity);
	}

	@Override
	public void delete(Long id) {
		studyEntryDataRepository.delete(id);
	}

	@Override
	public void deleteAll() {
		studyEntryDataRepository.deleteAll();
	}

	public boolean userHasPermission(PermissionName... permissionsNames) {
		return userService.userHasPermission(permissionsNames);
	}

	public List<StudyEntryData> findAllByCustomer() {
		return studyEntryDataRepository.findAllByCustomer(UserUtil.getCustomer());
	}

	public Set<StudyEntryData> findAll(Long... ids) {
		return studyEntryDataRepository.findAll(ids);
	}

	public List<StudyEntryData> findAllWithFilter(StudyEntryDataFilter studyEntryDataFilter) {
		return queryBuilder(studyEntryDataFilter).getResultList();
	}

	private TypedQuery<StudyEntryData> queryBuilder(StudyEntryDataFilter studyEntryDataFilter) {
		EntityManager em = studyEntryDataRepository.getEntityManager();

		String queryString = "SELECT DISTINCT sed FROM StudyEntryData sed";
		String joinString = "";
		String whereString = " WHERE sed.customer = :customer";

		if (studyEntryDataFilter.getQuickSearch() != null && !studyEntryDataFilter.getQuickSearch().isEmpty()) {
			whereString += " AND (LOWER(sed.name) LIKE :name OR LOWER(sed.dataKey) LIKE :dataKey)";
		}

		if (studyEntryDataFilter.getIds() != null && !studyEntryDataFilter.getIds().isEmpty()) {
			whereString += " AND sed.id IN :ids";
		}

		if (studyEntryDataFilter.getEntryName() != null && !studyEntryDataFilter.getEntryName().isEmpty()) {
			whereString += getFilterItemCondition("sed.name", studyEntryDataFilter.getEntryName());
		}

		if (studyEntryDataFilter.getEntryKey() != null && !studyEntryDataFilter.getEntryKey().isEmpty()) {
			whereString += getFilterItemCondition("sed.dataKey", studyEntryDataFilter.getEntryKey());
		}

		queryString += joinString + whereString;

		TypedQuery<StudyEntryData> typedQuery = em.createQuery(queryString, StudyEntryData.class);
		typedQuery.setHint("eclipselink.read-only", "true");
		typedQuery.setParameter("customer", UserUtil.getCustomer());

		if (studyEntryDataFilter.getQuickSearch() != null && !studyEntryDataFilter.getQuickSearch().isEmpty()) {
			typedQuery.setParameter("name", "%" + studyEntryDataFilter.getQuickSearch().toLowerCase() + "%");
			typedQuery.setParameter("dataKey", "%" + studyEntryDataFilter.getQuickSearch().toLowerCase() + "%");
		}

		if (studyEntryDataFilter.getIds() != null && !studyEntryDataFilter.getIds().isEmpty()) {
			Set<Long> entityIds = new HashSet<>();
			for (String filterItem : studyEntryDataFilter.getIds()) {
				FilterItemBean filterItemBean = FilterUtil.getFilterItem(filterItem);
				entityIds.add(Long.valueOf(filterItemBean.getFilterValue()));
			}

			typedQuery.setParameter("ids", entityIds);
		}

		return typedQuery;
	}
}
