package hu.oroscom.c1801.service;

import hu.oroscom.c1801.domain.*;
import hu.oroscom.c1801.domain.filter.StudyEntryFilter;
import hu.oroscom.c1801.enumeration.PermissionName;
import hu.oroscom.c1801.enumeration.StudyEntryType;
import hu.oroscom.c1801.jparepository.StudyEntryRepository;
import hu.oroscom.c1801.util.FilterItemBean;
import hu.oroscom.c1801.util.FilterUtil;
import hu.oroscom.c1801.util.UserUtil;
import org.joda.time.DateTime;
import org.springframework.stereotype.Service;

import javax.inject.Inject;
import javax.persistence.EntityManager;
import javax.persistence.TypedQuery;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * Created by Norbert on 2014.10.29..
 */
@Service
public class StudyEntryService extends BaseService<StudyEntry> {

	@Inject
	private StudyEntryRepository studyEntryRepository;

	@Inject
	private UserService userService;

	@Inject
	private StudyEntryBaseDataService studyEntryBaseDataService;

	@Inject
	private StudyEntryDataService studyEntryDataService;

//	@PostConstruct
//	public void generateMissingStudyEntryData() {
//		for (StudyEntry studyEntry : findAll()) {
//			if (studyEntry.getStudyEntryData() == null || studyEntry.getStudyEntryData().isEmpty()) {
//				StudyEntryBaseData studyEntryBaseData = findOneStudyEntryBaseDataByStudyEntryType(studyEntry.getCustomer(), studyEntry.getStudyEntryType());
//
//				if (studyEntryBaseData != null) {
//					for (StudyEntryData studyEntryData : studyEntryBaseData.getStudyEntryData()) {
//						List<StudyEntry> studyEntries = studyEntryData.getStudyEntries();
//						if (studyEntries == null) {
//							studyEntries = new ArrayList<>();
//						}
//
//						studyEntry = save(studyEntry);
//
//						studyEntries.add(studyEntry);
//
//						saveStudyEntryData(studyEntryData);
//					}
//				}
//			}
//		}
//	}

	@Override
	public StudyEntry findOne(Long id) {
		return studyEntryRepository.findOne(id);
	}

	@Override
	public StudyEntry save(StudyEntry entity) {
		return studyEntryRepository.save(entity);
	}

	@Override
	public List<StudyEntry> save(Iterable<StudyEntry> entities) {
		return studyEntryRepository.save(entities);
	}

	@Override
	public List<StudyEntry> findAll() {
		return studyEntryRepository.findAll();
	}

	@Override
	public void delete(StudyEntry entity) {
		studyEntryRepository.delete(entity);
	}

	@Override
	public void delete(Long id) {
		studyEntryRepository.delete(id);
	}

	@Override
	public void deleteAll() {
		studyEntryRepository.deleteAll();
	}

	public boolean userHasPermission(PermissionName... permissionsNames) {
		return userService.userHasPermission(permissionsNames);
	}

	public List<StudyEntry> findAllByPatient(Patient patient) {
		return studyEntryRepository.findAllByPatient(patient);
	}

	public Set<StudyEntry> findAllByStudyEntryType(StudyEntryType... studyEntryTypes) {
		return studyEntryRepository.findAllByStudyEntryType(UserUtil.getCustomer(), studyEntryTypes);
	}

	public StudyEntryBaseData findOneStudyEntryBaseDataByStudyEntryType(StudyEntryType studyEntryType) {
		return studyEntryBaseDataService.findOneByStudyEntryType(studyEntryType);
	}

	public StudyEntryBaseData findOneStudyEntryBaseDataByStudyEntryType(Customer customer, StudyEntryType studyEntryType) {
		return studyEntryBaseDataService.findOneByStudyEntryType(customer, studyEntryType);
	}

	public StudyEntryData saveStudyEntryData(StudyEntryData studyEntryData) {
		return studyEntryDataService.save(studyEntryData);
	}

	public StudyEntry findOneWithRelationTable(Long id) {
		return studyEntryRepository.findOneWithRelationTable(id);
	}

	public List<StudyEntry> findAllWithFilter(StudyEntryFilter studyEntryFilter) {
		return queryBuilder(studyEntryFilter).getResultList();
	}

	private TypedQuery<StudyEntry> queryBuilder(StudyEntryFilter studyEntryFilter) {
		EntityManager em = studyEntryRepository.getEntityManager();

		String queryString = "SELECT DISTINCT se FROM StudyEntry se";
		String joinString = "";
		String whereString = " WHERE se.customer = :customer AND se.patient = :patient";

		if (studyEntryFilter.getQuickSearch() != null && !studyEntryFilter.getQuickSearch().isEmpty()) {
			whereString += " AND (LOWER(se.description) LIKE :description)";
		}

		if (studyEntryFilter.getIds() != null && !studyEntryFilter.getIds().isEmpty()) {
			whereString += " AND se.id IN :ids";
		}

		if (studyEntryFilter.getEntryType() != null && !studyEntryFilter.getEntryType().isEmpty()) {
			whereString += getFilterItemCondition("se.studyEntryType", studyEntryFilter.getEntryType(), StudyEntryType.class);
		}

		if (studyEntryFilter.getEntryDateTime() != null && !studyEntryFilter.getEntryDateTime().isEmpty()) {
			whereString += getFilterItemCondition("se.time", studyEntryFilter.getEntryDateTime(), DateTime.class, "time");
		}

		queryString += joinString + whereString;

		TypedQuery<StudyEntry> typedQuery = em.createQuery(queryString, StudyEntry.class);
		typedQuery.setHint("eclipselink.read-only", "true");
		typedQuery.setParameter("customer", UserUtil.getCustomer());
		typedQuery.setParameter("patient", studyEntryFilter.getPatient());

		if (studyEntryFilter.getQuickSearch() != null && !studyEntryFilter.getQuickSearch().isEmpty()) {
			typedQuery.setParameter("description", "%" + studyEntryFilter.getQuickSearch().toLowerCase() + "%");
		}

		if (studyEntryFilter.getIds() != null && !studyEntryFilter.getIds().isEmpty()) {
			Set<Long> entityIds = new HashSet<>();
			for (String filterItem : studyEntryFilter.getIds()) {
				FilterItemBean filterItemBean = FilterUtil.getFilterItem(filterItem);
				entityIds.add(Long.valueOf(filterItemBean.getFilterValue()));
			}

			typedQuery.setParameter("ids", entityIds);
		}

		if (studyEntryFilter.getEntryDateTime() != null && !studyEntryFilter.getEntryDateTime().isEmpty()) {
			setupDateTimeParameters(typedQuery, studyEntryFilter.getEntryDateTime(), "time");
		}

		if (studyEntryFilter.getEntryType() != null && !studyEntryFilter.getEntryType().isEmpty()) {
			int index = 0;
			for (String filterItem : studyEntryFilter.getEntryType()) {
				FilterItemBean filterItemBean = FilterUtil.getFilterItem(filterItem);
				StudyEntryType filterValue = StudyEntryType.valueOf(filterItemBean.getFilterValue());
				typedQuery.setParameter("studyEntryType" + index, filterValue);
				index++;
			}
		}

		return typedQuery;
	}
}
