package hu.oroscom.c1801.service;

import hu.oroscom.c1801.webservice.TokenProxy;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import javax.crypto.Cipher;
import javax.inject.Inject;
import javax.servlet.ServletContext;
import java.io.*;
import java.math.BigInteger;
import java.security.Key;
import java.security.KeyFactory;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.spec.KeySpec;
import java.security.spec.RSAPrivateKeySpec;
import java.security.spec.RSAPublicKeySpec;

/**
 * Created by kollerk on 15. 01. 30..
 */
@Service
public class TokenService {

	public static final String tokenSeparator = "<-!->";
	private static final String privateKeyFile = "private.key";
	private static final String publicKeyFile = "public.key";
	private static String keyPath;
	@Inject
	UserService userService;

	@Inject
	ServletContext servletContext;

	private static final byte[] hexStringToByteArray(String str) {
		int i = 0;
		byte[] results = new byte[str.length() / 2];
		for (int k = 0; k < str.length(); ) {
			results[i] = (byte) (Character.digit(str.charAt(k++), 16) << 4);
			results[i] += (byte) (Character.digit(str.charAt(k++), 16));
			i++;
		}
		return results;
	}

	private static byte[] rsaCrypt(byte[] data, int mode) throws Exception {
		Key key = readKeyFromFile(mode);
		Cipher cipher = Cipher.getInstance("RSA");
		cipher.init(mode, key);
		byte[] cipherData = cipher.doFinal(data);
		return cipherData;
	}

	private static Key readKeyFromFile(int mode) throws Exception {
		String keyFileName = keyPath;
		if (mode == Cipher.ENCRYPT_MODE) {
			keyFileName += publicKeyFile;
		} else {
			keyFileName += privateKeyFile;
		}
		InputStream in = new FileInputStream(keyFileName);
		ObjectInputStream oin = new ObjectInputStream(new BufferedInputStream(in));
		try {
			BigInteger m = (BigInteger) oin.readObject();
			BigInteger e = (BigInteger) oin.readObject();
			KeyFactory fact = KeyFactory.getInstance("RSA");
			KeySpec keySpec;
			Key key;
			if (mode == Cipher.ENCRYPT_MODE) {
				keySpec = new RSAPublicKeySpec(m, e);
				key = fact.generatePublic(keySpec);
			} else {
				keySpec = new RSAPrivateKeySpec(m, e);
				key = fact.generatePrivate(keySpec);
			}
			return key;
		} finally {
			oin.close();
		}
	}

	public static TokenProxy getTokenProxyByToken(String token) throws Exception {

		byte[] data = rsaCrypt(hexStringToByteArray(token), Cipher.DECRYPT_MODE);
		String tokenString = new String(data);
		String[] tokens = tokenString.split(tokenSeparator);

		return new TokenProxy(tokens[0], tokens[1], Long.valueOf(tokens[2]));

	}

	private static String byteArrayToHexString(byte[] bytes) {
		StringBuffer buffer = new StringBuffer();
		for (int i = 0; i < bytes.length; i++) {
			if (((int) bytes[i] & 0xff) < 0x10)
				buffer.append("0");
			buffer.append(Long.toString((int) bytes[i] & 0xff, 16));
		}
		return buffer.toString();
	}

	public static String createToken(String username, String password, Long customerId) {
		try {
			return byteArrayToHexString(rsaCrypt((username + tokenSeparator + password + tokenSeparator + customerId).getBytes(), Cipher.ENCRYPT_MODE));
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	@PostConstruct
	private void init() {
		keyPath = servletContext.getRealPath("/WEB-INF/");
		if (!keyPath.endsWith("/")) {
			keyPath += "/";
		}
	}

	private void saveToFile(String fileName, BigInteger mod, BigInteger exp) throws IOException {
		fileName = keyPath + "/" + fileName;
		ObjectOutputStream oout = new ObjectOutputStream(new BufferedOutputStream(new FileOutputStream(fileName)));
		try {
			oout.writeObject(mod);
			oout.writeObject(exp);
		} catch (Exception e) {
			throw new IOException("Unexpected error", e);
		} finally {
			oout.close();
		}
	}

	public void generateRSAKeyPair() {
		try {
			File file = new File(keyPath + publicKeyFile);
			if (!file.exists()) {
				KeyPairGenerator kpg = KeyPairGenerator.getInstance("RSA");
				kpg.initialize(2048);
				KeyPair kp = kpg.genKeyPair();
				Key publicKey = kp.getPublic();
				Key privateKey = kp.getPrivate();

				KeyFactory fact = KeyFactory.getInstance("RSA");
				RSAPublicKeySpec pub = fact.getKeySpec(publicKey, RSAPublicKeySpec.class);
				RSAPrivateKeySpec priv = fact.getKeySpec(privateKey, RSAPrivateKeySpec.class);

				saveToFile(publicKeyFile, pub.getModulus(), pub.getPublicExponent());
				saveToFile(privateKeyFile, priv.getModulus(), priv.getPrivateExponent());

			}

		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}
