package hu.oroscom.c1801.ui;

import com.vaadin.annotations.Theme;
import com.vaadin.data.Property;
import com.vaadin.event.ShortcutAction;
import com.vaadin.event.ShortcutListener;
import com.vaadin.server.Page;
import com.vaadin.server.Responsive;
import com.vaadin.server.VaadinRequest;
import com.vaadin.server.VaadinSession;
import com.vaadin.ui.*;
import hu.oroscom.c1801.Application;
import hu.oroscom.c1801.util.UserUtil;
import org.vaadin.spring.annotation.VaadinUI;
import org.vaadin.spring.i18n.I18N;
import org.vaadin.spring.security.VaadinSecurity;

import javax.inject.Inject;
import java.util.Locale;

@VaadinUI(path = "/login")
@Theme("starter")
//@PreserveOnRefresh
public class LoginUI extends UI {

	@Inject
	VaadinSecurity security;

	@Inject
	I18N i18n;

	@Override
	protected void init(final VaadinRequest request) {
//		Locale locale = new Locale("hu", "HU");
//		this.setLocale(locale);
//		this.getSession().setLocale(locale);
		Responsive.makeResponsive(this);

		getReconnectDialogConfiguration().setDialogText(i18n.get("SystemMessage.ServerConnectionLostTryingToReconnect"));
		getReconnectDialogConfiguration().setDialogTextGaveUp(i18n.get("SystemMessage.ServerConnectionLost"));

		getPage().setTitle(i18n.get("LoginUI.loginLabel"));

		VerticalLayout root = new VerticalLayout();
		setContent(root);

		root.removeAllComponents();
		root.setSizeFull();
		root.setPrimaryStyleName("login");

		VerticalLayout content = new VerticalLayout();
		content.setSpacing(true);
		content.setMargin(true);
		content.setWidth("320px");

		final ComboBox languageChanger = new ComboBox(i18n.get("LoginUI.localization"));
		languageChanger.addItem("hu_HU");
		languageChanger.setItemCaption("hu_HU", "Magyar");
		languageChanger.addItem("en_GB");
		languageChanger.setItemCaption("en_GB", "English");
		languageChanger.addItem("de_DE");
		languageChanger.setItemCaption("de_DE", "Deutsch");

		languageChanger.setNullSelectionAllowed(false);
		languageChanger.setTextInputAllowed(false);
		languageChanger.setImmediate(true);
		languageChanger.setWidth(100, Unit.PERCENTAGE);
		languageChanger.setValue(VaadinSession.getCurrent().getLocale().toString());
		languageChanger.addValueChangeListener(new Property.ValueChangeListener() {
			@Override
			public void valueChange(Property.ValueChangeEvent event) {
				if (languageChanger.getValue() != null) {
					Locale locale = new Locale((String) languageChanger.getValue());
					UI.getCurrent().setLocale(locale);
					UI.getCurrent().getSession().setLocale(locale);
//					VaadinSession.getCurrent().setLocale(locale);
					Locale.setDefault(locale);
					init(request);
				}
			}
		});
		content.addComponent(languageChanger);

		final TextField name = new TextField(i18n.get("LoginUI.usernameLabel"));
		name.setId("username");
		name.setWidth("100%");
		content.addComponent(name);

		final PasswordField password = new PasswordField(i18n.get("LoginUI.passwordLabel"));
		password.setId("password");
		password.setWidth("100%");
		content.addComponent(password);

		final Button button = new Button(i18n.get("LoginUI.loginButton")); //Login
		button.addClickListener(new Button.ClickListener() {

			@Override
			public void buttonClick(Button.ClickEvent event) {
				try {
					security.login(name.getValue(), UserUtil.encryptPassword(password.getValue()));
					Page.getCurrent().setLocation(Application.contextPath + "/");

				} catch (Exception e) {
					Notification.show(i18n.get("LoginUI.loginFailureMessage")); //Hiba a bejelentkezéskor.
				}
			}
		});
		content.addComponent(button);
		content.setComponentAlignment(button, Alignment.MIDDLE_RIGHT);

		TabSheet tabSheet = new TabSheet();
		tabSheet.setWidthUndefined();
		tabSheet.addTab(content, i18n.get("LoginUI.loginLabel"));
		root.addComponent(tabSheet);

		setSizeFull();
		root.setComponentAlignment(tabSheet, Alignment.MIDDLE_CENTER);

		name.focus();
		password.addShortcutListener(new ShortcutListener("EnterOnPasswordShortcut", ShortcutAction.KeyCode.ENTER, null) {
			@Override
			public void handleAction(Object sender, Object target) {
				button.click();
			}
		});
	}

}
