package hu.oroscom.c1801.ui;

import com.vaadin.annotations.PreserveOnRefresh;
import com.vaadin.annotations.Push;
import com.vaadin.annotations.Theme;
import com.vaadin.navigator.Navigator;
import com.vaadin.navigator.ViewChangeListener;
import com.vaadin.server.*;
import com.vaadin.shared.communication.PushMode;
import com.vaadin.shared.ui.label.ContentMode;
import com.vaadin.shared.ui.ui.Transport;
import com.vaadin.ui.*;
import com.vaadin.ui.themes.ValoTheme;
import hu.oroscom.c1801.Application;
import hu.oroscom.c1801.bean.BroadcastBean;
import hu.oroscom.c1801.broadcaster.Broadcaster;
import hu.oroscom.c1801.component.MainLayout;
import hu.oroscom.c1801.domain.Patient;
import hu.oroscom.c1801.domain.Setting;
import hu.oroscom.c1801.domain.User;
import hu.oroscom.c1801.domain.UserComment;
import hu.oroscom.c1801.enumeration.EntryPoint;
import hu.oroscom.c1801.enumeration.PermissionName;
import hu.oroscom.c1801.enumeration.RoleName;
import hu.oroscom.c1801.enumeration.SettingName;
import hu.oroscom.c1801.service.SettingService;
import hu.oroscom.c1801.service.UserCommentService;
import hu.oroscom.c1801.service.UserService;
import hu.oroscom.c1801.util.MessagesUtil;
import hu.oroscom.c1801.util.UserUtil;
import hu.oroscom.c1801.view.*;
import org.apache.commons.codec.binary.Base64;
import org.springframework.security.core.context.SecurityContextHolder;
import org.vaadin.spring.annotation.VaadinUI;
import org.vaadin.spring.events.EventBus;
import org.vaadin.spring.i18n.I18N;
import org.vaadin.spring.navigator.SpringViewProvider;
import org.vaadin.spring.security.VaadinSecurity;

import javax.inject.Inject;
import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.LinkedHashMap;
import java.util.Map;

@VaadinUI
@Theme("starter")
@PreserveOnRefresh
@Push(value = PushMode.MANUAL, transport = Transport.LONG_POLLING)
public class MainUI extends UI implements Broadcaster.BroadcastListener {
	private static final long serialVersionUID = 1L;
	private final MainLayout root = new MainLayout();
	private final ComponentContainer viewDisplay = root.getContentContainer();
	private final Navigator navigator = new Navigator(this, viewDisplay);
	private final CssLayout menu = new CssLayout();
	private final CssLayout menuItemsLayout = new CssLayout();
	private final LinkedHashMap<String, String> menuItems = new LinkedHashMap<>();
	private final LinkedHashMap<String, FontAwesome> menuIcons = new LinkedHashMap<>();
	//	private final TestIcon testIcon = new TestIcon(100);
	private boolean reloadPrices = true;

	@Inject
	private VaadinSecurity security;

	@Inject
	private I18N i18n;

	@Inject
	private SpringViewProvider viewProvider;

	@Inject
	private UserService userService;

	@Inject
	private SettingService settingService;

	@Inject
	private EventBus eventBus;

	@Inject
	private UserCommentService userCommentService;

	@Override
	protected void init(final VaadinRequest vaadinRequest) {
		final User user = UserUtil.getUser();
		navigator.addProvider(viewProvider);
		setNavigator(navigator);

		Responsive.makeResponsive(this);
		this.addStyleName(ValoTheme.UI_WITH_MENU);
		menu.setId("mainMenu");
		menu.setHeight(getPage().getBrowserWindowHeight(), Unit.PIXELS);
		Page.getCurrent().addBrowserWindowResizeListener(new Page.BrowserWindowResizeListener() {
			@Override
			public void browserWindowResized(Page.BrowserWindowResizeEvent event) {
				menu.setHeight(getPage().getBrowserWindowHeight(), Unit.PIXELS);
			}
		});

		getReconnectDialogConfiguration().setDialogText(i18n.get("SystemMessage.ServerConnectionLostTryingToReconnect"));
		getReconnectDialogConfiguration().setDialogTextGaveUp(i18n.get("SystemMessage.ServerConnectionLost"));

		getPage().setTitle(i18n.get("MainUI.title"));

		setContent(root);
		root.setWidth("100%");

		root.addMenu(buildMenu(user));

		final String f = Page.getCurrent().getUriFragment();
		if (f == null || f.equals("")) {
			if (userService.userHasPermission(PermissionName.PERM_DASHBOARD_VIEW) && EntryPoint.DASHBOARD_VIEW.equals(UserUtil.getUser().getEntryPoint())) {
				navigator.navigateTo(DashboardView.NAME);
			} else {
				logout();
				return;
			}
		}

		navigator.addViewChangeListener(new ViewChangeListener() {
			@Override
			public boolean beforeViewChange(ViewChangeEvent event) {
				BaseView view = (BaseView) event.getNewView();
				if (UserUtil.isAdmin() || view != null) {
					if (view instanceof SettingsView) {
						return true;
					}
					return userService.userHasAnyPermission(user, BaseView.getPermissions(view.getClass()));
				} else {
					return false;
				}
			}

			@Override
			public void afterViewChange(ViewChangeEvent event) {

			}
		});

		this.addDetachListener(new DetachListener() {
			@Override
			public void detach(DetachEvent detachEvent) {
			}
		});

		Broadcaster.register(this);
	}

	private CssLayout buildMenu(User user) {
		if (userService.userHasAnyPermission(user, BaseView.getPermissions(DashboardView.class))) {
			menuItems.put(DashboardView.NAME, i18n.get("MainUI.dashboardMenuItem"));
			menuIcons.put(DashboardView.NAME, FontAwesome.DASHBOARD);
		}

		if (userService.userHasAnyPermission(user, BaseView.getPermissions(UserCommentsView.class))) {
			menuItems.put(UserCommentsView.NAME, i18n.get("MainUI.userCommentsMenuItem"));
			menuIcons.put(UserCommentsView.NAME, FontAwesome.COMMENTS);
		}

		if (userService.userHasAnyPermission(user, BaseView.getPermissions(StudyEntryDataView.class))) {
			menuItems.put(StudyEntryDataView.NAME, i18n.get("MainUI.studyEntryDataMenuItem"));
			menuIcons.put(StudyEntryDataView.NAME, FontAwesome.STUMBLEUPON_CIRCLE);
		}

		if (UserUtil.getUser() instanceof Patient) {
			if (userService.userHasAnyPermission(user, BaseView.getPermissions(StudyEntriesView.class))) {
				menuItems.put(StudyEntriesView.NAME, i18n.get("MainUI.studyEntriesMenuItem"));
				menuIcons.put(StudyEntriesView.NAME, FontAwesome.STUMBLEUPON_CIRCLE);
			}

			if (userService.userHasAnyPermission(user, BaseView.getPermissions(EatenFoodsView.class))) {
				menuItems.put(EatenFoodsView.NAME, i18n.get("MainUI.eatenFoodsMenuItem"));
				menuIcons.put(EatenFoodsView.NAME, FontAwesome.CUTLERY);
			}

			if (userService.userHasAnyPermission(user, BaseView.getPermissions(IntolerantMaterialsView.class))) {
				menuItems.put(IntolerantMaterialsView.NAME, i18n.get("MainUI.intolerantMaterialsMenuItem"));
				menuIcons.put(IntolerantMaterialsView.NAME, FontAwesome.MEH_O);
			}

			if (userService.userHasAnyPermission(user, BaseView.getPermissions(ComplaintsView.class))) {
				menuItems.put(ComplaintsView.NAME, i18n.get("MainUI.complaintsMenuItem"));
				menuIcons.put(ComplaintsView.NAME, FontAwesome.MEDKIT);
			}
		}

		if (userService.userHasAnyPermission(user, BaseView.getPermissions(ServiceGroupsView.class))) {
			menuItems.put(ServiceGroupsView.NAME, i18n.get("MainUI.serviceGroupsMenuItem"));
			menuIcons.put(ServiceGroupsView.NAME, FontAwesome.TH);
		}

		if (userService.userHasAnyPermission(user, BaseView.getPermissions(ServiceManagementView.class))) {
			menuItems.put(ServiceManagementView.NAME, MessagesUtil.get(i18n, "MainUI.serviceManagementMenuItem"));
			menuIcons.put(ServiceManagementView.NAME, FontAwesome.BUG);
		}

		if (userService.userHasAnyPermission(user, BaseView.getPermissions(UsersView.class))
				|| userService.userHasAnyPermission(user, BaseView.getPermissions(AssistantsView.class))
				|| userService.userHasAnyPermission(user, BaseView.getPermissions(DoctorsView.class))
				|| userService.userHasAnyPermission(user, BaseView.getPermissions(PatientsView.class))
				) {
			menuItems.put("separator2", "");
		}

		if (userService.userHasAnyPermission(user, BaseView.getPermissions(AssistantsView.class))) {
			menuItems.put(AssistantsView.NAME, i18n.get("MainUI.assistantsMenuItem"));
			menuIcons.put(AssistantsView.NAME, FontAwesome.USERS);
		}

		if (userService.userHasAnyPermission(user, BaseView.getPermissions(DoctorsView.class))) {
			menuItems.put(DoctorsView.NAME, i18n.get("MainUI.doctorsMenuItem"));
			menuIcons.put(DoctorsView.NAME, FontAwesome.USERS);
		}

		if (userService.userHasAnyPermission(user, BaseView.getPermissions(PatientsView.class))) {
			menuItems.put(PatientsView.NAME, i18n.get("MainUI.patientsMenuItem"));
			menuIcons.put(PatientsView.NAME, FontAwesome.USERS);
		}

		if (userService.userHasAnyPermission(user, BaseView.getPermissions(UsersView.class))) {
			menuItems.put(UsersView.NAME, i18n.get("MainUI.usersMenuItem"));
			menuIcons.put(UsersView.NAME, FontAwesome.USER_PLUS);
		}

		if (userService.userHasAnyPermission(user, BaseView.getPermissions(ReportsView.class))) {
			menuItems.put("separator3", "");
		}

		if (userService.userHasAnyPermission(user, BaseView.getPermissions(ReportsView.class))) {
			menuItems.put(ReportsView.NAME, i18n.get("MainUI.reportsMenuItem"));
			menuIcons.put(ReportsView.NAME, FontAwesome.BAR_CHART_O);
		}

		Resource logoResource;
		if (UserUtil.getCustomer() == null) {
			logoResource = new ThemeResource("../starter/img/corporate-logo.png");
		} else {
			final Setting customerLogoSetting = settingService.findSetting(SettingName.CUSTOMER_LOGO);
			if (customerLogoSetting != null && customerLogoSetting.getValue() != null && !customerLogoSetting.getValue().isEmpty()) {
				StreamResource.StreamSource streamSource = new StreamResource.StreamSource() {
					@Override
					public InputStream getStream() {
						byte[] decodedBytes = Base64.decodeBase64(customerLogoSetting.getValue());
						return (decodedBytes == null) ? null : new ByteArrayInputStream(decodedBytes);
					}
				};
				logoResource = new StreamResource(streamSource, "logo-" + UserUtil.getCustomer().getId() + ".jpg");

			} else {
				logoResource = new ThemeResource("../starter/img/corporate-logo.png");
			}
		}
		Image corporateLogo = new Image(null, logoResource);
		corporateLogo.addStyleName("corporate-logo");

		final HorizontalLayout top = new HorizontalLayout();
		top.setWidth(100, Unit.PERCENTAGE);
		top.setDefaultComponentAlignment(Alignment.MIDDLE_LEFT);
		top.addStyleName("valo-menu-title");
		top.addComponent(corporateLogo);
		menu.addComponent(top);

		final Button showMenu = new Button("Menu", new Button.ClickListener() {
			@Override
			public void buttonClick(final Button.ClickEvent event) {
				if (menu.getStyleName().contains("valo-menu-visible")) {
					menu.removeStyleName("valo-menu-visible");
				} else {
					menu.addStyleName("valo-menu-visible");
				}
			}
		});
		showMenu.addStyleName(ValoTheme.BUTTON_PRIMARY);
		showMenu.addStyleName(ValoTheme.BUTTON_SMALL);
		showMenu.addStyleName("valo-menu-toggle");
		showMenu.setIcon(FontAwesome.LIST);
		menu.addComponent(showMenu);

		final MenuBar settings = new MenuBar();
		settings.addStyleName("user-menu");

		final MenuBar.MenuItem settingsItem = settings.addItem(user.getDisplayName(), null);
		settingsItem.setStyleName("logged-in-user");
		settingsItem.addItem(i18n.get("MainUI.editProfileMenuItem"), FontAwesome.USER, new MenuBar.Command() {
			@Override
			public void menuSelected(MenuBar.MenuItem menuItem) {
				navigator.navigateTo(UserView.NAME);
			}
		});

		if (userService.userHasAnyPermission(user, BaseView.getPermissions(RolesView.class))) {
			settingsItem.addItem(i18n.get("MainUI.rolesMenuItem"), FontAwesome.ROCKET, new MenuBar.Command() {
				@Override
				public void menuSelected(MenuBar.MenuItem menuItem) {
					navigator.navigateTo(RolesView.NAME);
				}
			});
		}

		if (userService.userHasAnyPermission(user, BaseView.getPermissions(PermissionsView.class))) {
			settingsItem.addItem(i18n.get("MainUI.permissionsMenuItem"), FontAwesome.EYEDROPPER, new MenuBar.Command() {
				@Override
				public void menuSelected(MenuBar.MenuItem menuItem) {
					navigator.navigateTo(PermissionsView.NAME);
				}
			});
		}

		if (userService.userHasAnyPermission(user, BaseView.getPermissions(SettingsView.class))) {
			settingsItem.addItem(i18n.get("MainUI.preferencesMenuItem"), FontAwesome.GEARS, new MenuBar.Command() {
				@Override
				public void menuSelected(MenuBar.MenuItem menuItem) {
					navigator.navigateTo(SettingsView.NAME);
				}
			});
		}
		settingsItem.addItem(i18n.get("MainUI.filtersMenuItem"), FontAwesome.FILTER, new MenuBar.Command() {
			@Override
			public void menuSelected(MenuBar.MenuItem menuItem) {
				navigator.navigateTo(FiltersView.NAME);
			}
		});

		if (user.hasRole(RoleName.ROLE_ADMIN)) {
			settingsItem.addItem(i18n.get("MainUI.excelExportLogsMenuItem"), FontAwesome.DOWNLOAD, new MenuBar.Command() {
				@Override
				public void menuSelected(MenuBar.MenuItem menuItem) {
					navigator.navigateTo(ExcelExportLogsView.NAME);
				}
			});
		}

		settingsItem.addSeparator();
		settingsItem.addItem(i18n.get("MainUI.signOutMenuItem"), FontAwesome.POWER_OFF, new MenuBar.Command() {
			@Override
			public void menuSelected(MenuBar.MenuItem menuItem) {
				logout();
			}
		});
		menu.addComponent(settings);

		menuItemsLayout.setPrimaryStyleName("valo-menuitems");
		menu.addComponent(menuItemsLayout);

		Label label;
		for (final Map.Entry<String, String> item : menuItems.entrySet()) {
			if (item.getKey().startsWith("separator")) {
				label = new Label("", ContentMode.HTML);
				label.setPrimaryStyleName("valo-menu-subtitle");
				label.addStyleName("h4");
				label.setSizeUndefined();
				menuItemsLayout.addComponent(label);
			} else {
				final Button b = new Button(item.getValue(), new Button.ClickListener() {
					@Override
					public void buttonClick(final Button.ClickEvent event) {
						navigator.navigateTo(item.getKey());
					}
				});
				if (UserCommentsView.NAME.equals(item.getKey())) {
					b.setCaption(b.getCaption() + generateValoMenuBadge(userCommentService.findUnmanagedCommentCount()));
				}
				b.setHtmlContentAllowed(true);
				b.setData(item.getKey());
				b.setPrimaryStyleName("valo-menu-item");
				if (menuIcons.get(item.getKey()) != null) {
					b.setIcon(menuIcons.get(item.getKey()));
				}
				menuItemsLayout.addComponent(b);
			}
		}

		return menu;
	}

	private void logout() {
		UI.getCurrent().getPushConfiguration().setPushMode(PushMode.DISABLED);
		if (SecurityContextHolder.getContext().getAuthentication() != null) {
			security.getAuthentication().setAuthenticated(false);
			Page.getCurrent().setLocation(Application.contextPath);
		} else {
			VaadinService.getCurrentRequest().getWrappedSession().invalidate();
			Page.getCurrent().setLocation(Application.contextPath);
		}
		UserUtil.deleteSession();
	}

	private Thread getThreadByName(String threadName) {
		for (Thread t : Thread.getAllStackTraces().keySet()) {
			if (t.getName().equals(threadName)) return t;
		}
		return null;
	}

	public SpringViewProvider getViewProvider() {
		return viewProvider;
	}

	@Override
	public void detach() {
		Broadcaster.unregister(this);
		super.detach();
	}

	@Override
	public void receiveBroadcast(final BroadcastBean broadcastBean) {
		access(new Runnable() {
			@Override
			public void run() {

				switch (broadcastBean.getBroadcastType()) {
					case MESSAGE:
						UserComment userComment = (UserComment) broadcastBean.getBroadcastObject();
						if (userComment != null) {
							int componentCount = menuItemsLayout.getComponentCount();
							int i = 1;
							for (Component component : menuItemsLayout) {
								if (component instanceof Button) {
									Button button = (Button) component;
									if (UserCommentsView.NAME.equals(button.getData())) {
										button.setCaption(menuItems.get(UserCommentsView.NAME) + generateValoMenuBadge(userCommentService.findUnmanagedCommentCount()));
										break;
									}
								}

								if (i >= componentCount) {
									break;
								}

								i++;
							}

							if (UserUtil.getUser().equals(userComment.getInsertedByUser()) && !UserUtil.getCustomer().equals(userComment.getCustomer())) {
								return;
							}

							if (UserCommentsView.class.equals(navigator.getCurrentView().getClass())) {
								UserCommentsView userCommentsView = (UserCommentsView) navigator.getCurrentView();
								userCommentsView.refreshView();
							}
						}
						break;
				}
				push();
			}
		});
	}

	private String generateValoMenuBadge(int count) {
		if (count <= 0) {
			return "";
		}
		return "&nbsp;<span class=\"valo-menu-badge\">" + count + "</span>";
	}
}
