package hu.oroscom.c1801.util;

import com.google.common.base.Joiner;
import com.vaadin.addon.tableexport.ExcelExport;
import com.vaadin.data.Property;
import com.vaadin.data.util.ObjectProperty;
import com.vaadin.ui.Table;
import hu.oroscom.c1801.domain.ExcelExportLog;
import hu.oroscom.c1801.service.ExcelExportLogService;
import org.joda.time.DateTime;

import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

/**
 * Created by ifino on 2017.06.07..
 */
public class LoggingExcelExport extends ExcelExport {
	private ExcelExportLogService excelExportLogService;
	private String viewName;

	public LoggingExcelExport(Table table, String sheetName, String reportTitle, String exportFileName, boolean hasTotalsRow, ExcelExportLogService excelExportLogService, String viewName) {
		super(table, sheetName, reportTitle, exportFileName, hasTotalsRow);
		this.excelExportLogService = excelExportLogService;
		this.viewName = viewName;
	}

	@Override
	public void export() {
		super.export();
		log();
	}

	private void log() {
		ExcelExportLog excelExportLog = new ExcelExportLog();
		excelExportLog.setCustomer(UserUtil.getCustomer());
		excelExportLog.setExportDate(new DateTime());
		excelExportLog.setExportedUserName(UserUtil.getUser().getDisplayName());
		excelExportLog.setViewName(viewName);

		StringBuilder csvStringBuilder = new StringBuilder();
		Collection<?> itemIds = this.getTableHolder().getContainerDataSource().getItemIds();
		excelExportLog.setExportedRowCount(itemIds.size());

		List<String> columnHeaderNames = new ArrayList<>();
		for (int col = 0; col < this.getPropIds().size(); ++col) {
			Object propId = this.getPropIds().get(col);
			String columnName = this.getTableHolder().getColumnHeader(propId).toString();
			if ("ID".equals(columnName)) {
				columnName = "Id";
			}
			columnHeaderNames.add(columnName);
		}

		String csvLine = Joiner.on(";").join(columnHeaderNames) + "\r\n";
		csvStringBuilder.append(csvLine);

		for (Iterator i$ = itemIds.iterator(); i$.hasNext(); ) {
			Object itemId = i$.next();
			List<String> values = new ArrayList<>();

			for (int col = 0; col < this.getPropIds().size(); ++col) {
				Object propId = this.getPropIds().get(col);
				Property prop = this.getProperty(itemId, propId);
				Object value;
				if (null == prop) {
					value = null;
				} else {
					value = prop.getValue();
				}

				if (null != value) {

					if (!this.isNumeric(prop.getType())) {
						values.add("\"" + value.toString() + "\"");
//						if(Date.class.isAssignableFrom(prop.getType())) {
//							Date date = (Date)value;
//						} else {
//							String s = value.toString();
//						}
					} else {
						try {
							Float f = Float.valueOf(Float.parseFloat(value.toString()));
							DecimalFormat df = new DecimalFormat("#.#");
							values.add(df.format(f));
						} catch (NumberFormatException var13) {
						}
					}
				}
			}

			csvLine = Joiner.on(";").join(values) + "\r\n";
			csvStringBuilder.append(csvLine);
		}

		excelExportLog.setExportedCsv(csvStringBuilder.toString());

		excelExportLogService.save(excelExportLog);
	}


	private Property getProperty(Object rootItemId, Object propId) {
		Object prop;
		if (this.getTableHolder().isGeneratedColumn(propId)) {
			prop = this.getTableHolder().getPropertyForGeneratedColumn(propId, rootItemId);
		} else {
			prop = this.getTableHolder().getContainerDataSource().getContainerProperty(rootItemId, propId);
			if (this.useTableFormatPropertyValue) {
				if (this.getTableHolder().isExportableFormattedProperty()) {
					String formattedProp = this.getTableHolder().getFormattedPropertyValue(rootItemId, propId, (Property) prop);
					if (null == prop) {
						prop = new ObjectProperty(formattedProp, String.class);
					} else {
						Object val = ((Property) prop).getValue();
						if (null == val) {
							prop = new ObjectProperty(formattedProp, String.class);
						} else if (!val.toString().equals(formattedProp)) {
							prop = new ObjectProperty(formattedProp, String.class);
						}
					}
				}
			}
		}

		return (Property) prop;
	}

	private boolean isNumeric(Class<?> type) {
		return this.isIntegerLongShortOrBigDecimal(type) ? true : this.isDoubleOrFloat(type);
	}

	private boolean isIntegerLongShortOrBigDecimal(Class<?> type) {
		return !Integer.class.equals(type) && !Integer.TYPE.equals(type) ? (!Long.class.equals(type) && !Long.TYPE.equals(type) ? (!Short.class.equals(type) && !Short.TYPE.equals(type) ? BigDecimal.class.equals(type) || BigDecimal.class.equals(type) : true) : true) : true;
	}

	private boolean isDoubleOrFloat(Class<?> type) {
		return !Double.class.equals(type) && !Double.TYPE.equals(type) ? Float.class.equals(type) || Float.TYPE.equals(type) : true;
	}

}
