package hu.oroscom.c1801.util;

import com.vaadin.server.VaadinSession;
import hu.oroscom.c1801.domain.Customer;
import hu.oroscom.c1801.domain.User;
import hu.oroscom.c1801.enumeration.RoleName;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;

/**
 * Created by Norbert on 2015.08.10..
 */
public class UserUtil {
	private static ThreadLocal<User> userThreadLocal = new ThreadLocal<>();

	public static boolean isAdmin() {
		User currentUser = getUser();
		return (RoleName.ROLE_SUPER_ADMIN.toString().equals(currentUser.getRole().toString()) || RoleName.ROLE_ADMIN.toString().equals(currentUser.getRole().toString()));
	}

	public static boolean isSuperAdmin() {
		User currentUser = getUser();
		return (RoleName.ROLE_SUPER_ADMIN.toString().equals(currentUser.getRole().toString()));
	}

	public static boolean isCustomerAdmin() {
		User currentUser = getUser();
		return (RoleName.ROLE_ADMIN.toString().equals(currentUser.getRole().toString()));
	}

	public static boolean isAssistant() {
		User currentUser = getUser();
		return (RoleName.ROLE_ASSISTANT.toString().equals(currentUser.getRole().toString()));
	}

	public static boolean isDoctor() {
		User currentUser = getUser();
		return (RoleName.ROLE_DOCTOR.toString().equals(currentUser.getRole().toString()));
	}

	public static boolean isPatient() {
		User currentUser = getUser();
		return (RoleName.ROLE_PATIENT.toString().equals(currentUser.getRole().toString()));
	}

	public static String encryptPassword(String password) {
		return get_SHA_512_SecurePassword(password);
	}

	private static String get_SHA_512_SecurePassword(String passwordToHash) {
		String generatedPassword = null;
		try {
			String salt = getSalt();
			MessageDigest md = MessageDigest.getInstance("SHA-512");
//			md.update(salt.getBytes());
			byte[] bytes = md.digest(passwordToHash.getBytes());
			StringBuilder sb = new StringBuilder();
			for (int i = 0; i < bytes.length; i++) {
				sb.append(Integer.toString((bytes[i] & 0xff) + 0x100, 16).substring(1));
			}
			generatedPassword = sb.toString();
		} catch (NoSuchAlgorithmException e) {
			e.printStackTrace();
		}
		return generatedPassword;
	}

	private static String getSalt() throws NoSuchAlgorithmException {
		SecureRandom sr = SecureRandom.getInstance("SHA1PRNG");
		byte[] salt = new byte[16];
		sr.nextBytes(salt);
		return salt.toString();
	}

	public static User getUser() {
		try {
			setUser(VaadinSession.getCurrent().getAttribute(User.class));
			return userThreadLocal.get();
		} catch (Exception e) {
			return userThreadLocal.get();
		}
	}

	public static void setUser(User user) {
		userThreadLocal.set(user);
	}

	public static Customer getCustomer() {
		return getUser().getCustomer();
	}

	public static void deleteSession() {
		userThreadLocal.remove();
	}
}
