package hu.oroscom.c1801.view;

import com.vaadin.navigator.View;
import com.vaadin.navigator.ViewChangeListener;
import com.vaadin.server.Page;
import com.vaadin.server.Page.Styles;
import com.vaadin.server.VaadinSession;
import com.vaadin.shared.ui.MarginInfo;
import com.vaadin.ui.*;
import com.vaadin.ui.themes.ValoTheme;
import hu.oroscom.c1801.NeededAnyPermission;
import hu.oroscom.c1801.component.form.BaseForm;
import hu.oroscom.c1801.domain.EntityBase;
import hu.oroscom.c1801.enumeration.Action;
import hu.oroscom.c1801.enumeration.PermissionName;
import hu.oroscom.c1801.util.EventBusAction;
import org.joda.time.format.DateTimeFormat;
import org.joda.time.format.DateTimeFormatter;
import org.vaadin.spring.events.EventBus;
import org.vaadin.spring.events.annotation.EventBusListenerMethod;
import org.vaadin.spring.i18n.I18N;

import javax.annotation.PostConstruct;
import javax.inject.Inject;
import java.lang.annotation.Annotation;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.*;

/**
 * Created by kollerk on 2014.10.07..
 */
public abstract class BaseView extends VerticalLayout implements View, Window.CloseListener {
	protected static DateTimeFormatter dateTimeFormatter;
	protected static DateTimeFormatter dateFormatter;
	protected static DateTimeFormatter dateNameFormatter;
	protected static DecimalFormat decimalFormat;

	public Window window = new Window();
	protected Label h1;
	protected HorizontalLayout headerTitleHorizontalLayout = new HorizontalLayout();

	@Inject
	I18N i18n;

	@Inject
	EventBus eventBus;

	protected BaseView() {
		setMargin(true);
		setMargin(new MarginInfo(false, true, true, true));
		setSpacing(true);
		setSizeFull();
		dateTimeFormatter = DateTimeFormat.shortDateTime().withLocale(VaadinSession.getCurrent().getLocale());
		dateFormatter = DateTimeFormat.shortDate().withLocale(VaadinSession.getCurrent().getLocale());
		dateNameFormatter = DateTimeFormat.forPattern("E").withLocale(VaadinSession.getCurrent().getLocale());
		decimalFormat = (DecimalFormat) NumberFormat.getInstance(VaadinSession.getCurrent().getLocale());
	}

	public static Set<PermissionName> getPermissions(Class aClass) {
		Annotation annotation = aClass.getAnnotation(NeededAnyPermission.class);
		if (annotation instanceof NeededAnyPermission) {
			NeededAnyPermission neededAnyPermission = (NeededAnyPermission) annotation;
			return new HashSet<>(Arrays.asList(neededAnyPermission.values()));
		}

		return new HashSet<>(Arrays.asList(PermissionName.PERM_NO_PERMISSION_NEEDED));
	}

	@PostConstruct
	protected abstract void init();

	protected void defaultComponents(String caption) {
		eventBus.unsubscribe(this); //többszörös init() hívás miatt!
		eventBus.subscribe(this);
		removeAllComponents();
		headerTitleHorizontalLayout.removeAllComponents();

		configureWindow();

		h1 = new Label(caption);
		h1.addStyleName(ValoTheme.LABEL_H1);
		h1.addStyleName("view-heading");

		headerTitleHorizontalLayout.setWidth(100, Unit.PERCENTAGE);
		headerTitleHorizontalLayout.addComponent(h1);
		headerTitleHorizontalLayout.setExpandRatio(h1, 1);
		addComponent(headerTitleHorizontalLayout);
	}

	protected void addFormToWindow(BaseForm baseForm) {
		addContentToWindow(baseForm);
	}

	protected void addContentToWindow(Component component) {
		window.setContent(component);
		if (!window.isAttached()) {
			UI.getCurrent().addWindow(window);
		}
		window.center();
	}

	protected <E extends EntityBase> void removeWindow() {
		Collection<Window> windows = UI.getCurrent().getWindows();
		List<Window> windowList = new ArrayList<>(windows);
		if (!windows.isEmpty()) {
			UI.getCurrent().removeWindow(windowList.get(windowList.size() - 1));
		}
	}

	private void configureWindow() {
		int browserWindowWidth = UI.getCurrent().getPage().getBrowserWindowWidth();
		int browserWindowHeight = UI.getCurrent().getPage().getBrowserWindowHeight();

		window.addCloseListener(this);
		window.setModal(true);
		window.center();
		window.setClosable(true);
		window.setResizable(false);
		window.setWidth(browserWindowWidth < 1000 ? browserWindowWidth : 1000, Unit.PIXELS);

		Styles styles = Page.getCurrent().getStyles();
		styles.add(".v-app .v-window .v-panel {max-height:" + browserWindowHeight + "px;border:none;box-shadow:none;}");
	}

	@Override
	public void enter(ViewChangeListener.ViewChangeEvent event) {
		//TODO Csak fejlesztési célokra!
//		removeAllComponents();
//		init();
	}

	@EventBusListenerMethod
	public void buttonClicked(EventBusAction eventBusAction) {
		if (eventBusAction.getAction().equals(Action.SAVE_FILTER) || eventBusAction.getAction().equals(Action.EDIT)) {
			return;
		}
		if (!eventBusAction.isAlreadyCalled()) {
			removeWindow();
		}
		eventBusAction.setAlreadyCalled(true);
//		if (eventBusAction.getEntity() != null) {
//			Class formClass = eventBusAction.getEntity().getClass();
//			if (eventBusAction.getAction().equals(Action.SAVE) || eventBusAction.getAction().equals(Action.DELETE)) {
//				for (Class aClass : BASE_FORMS) {
//					if (aClass.getSimpleName().contains(formClass.getSimpleName())) {
////						conflictService.recalculateConflicts();
//						return;
//					}
//				}
//			}
//		} else if (eventBusAction.getAction().equals(Action.DELETE)) {
//			if (this.getClass().equals(CalendarEntryView.class)) {
////				conflictService.recalculateConflicts();
//			}
////			for (Class aClass : BASE_FORMS) {
////				if (aClass.getSimpleName().contains(formClass.getSimpleName())) {
////					conflictService.recalculateConflicts();
////					return;
////				}
////			}
//		}
	}
}
