package hu.oroscom.c1801.view;

import com.vaadin.data.Item;
import com.vaadin.ui.Window;
import hu.oroscom.c1801.NeededAnyPermission;
import hu.oroscom.c1801.component.cell.RemoveCell;
import hu.oroscom.c1801.component.form.FilterForm;
import hu.oroscom.c1801.domain.filter.Filter;
import hu.oroscom.c1801.enumeration.Action;
import hu.oroscom.c1801.enumeration.FilterCategory;
import hu.oroscom.c1801.enumeration.PermissionName;
import hu.oroscom.c1801.service.FilterService;
import hu.oroscom.c1801.util.EventBusAction;
import org.vaadin.spring.annotation.VaadinUIScope;
import org.vaadin.spring.events.annotation.EventBusListenerMethod;
import org.vaadin.spring.navigator.annotation.VaadinView;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * Created by kollerk on 2014.10.07..
 */
@VaadinUIScope
@VaadinView(name = FiltersView.NAME)
@NeededAnyPermission(values = {
		PermissionName.PERM_NO_PERMISSION_NEEDED
})
public class FiltersView extends TreeGridView<Filter, FilterService, FilterForm> {
	public static final String NAME = "filters";

	@Inject
	FilterService filterService;

	@Override
	protected void init() {
		super.defaultComponents(i18n.get("FiltersView.filtersLabel"));
		initTable();
	}

	@Override
	protected void customizeView() {
	}

	@Override
	protected void registerColumns() {
		List<Object> visibleColumns = new ArrayList<>(Arrays.asList(new Object[]{"displayName", "remove"}));

		addGeneratedColumn("remove", new RemoveCell<>(FilterService.class));

		addContainerProperty("displayName", String.class, null);
		addContainerProperty("isEditCellEnabled", Boolean.class, false);
		addContainerProperty("isRemoveCellEnabled", Boolean.class, false);
		addContainerProperty("isVisible", Boolean.class, false);

		baseTreeTable.setColumnHeader("displayName", i18n.get("FiltersView.displayName"));
		baseTreeTable.setColumnHeader("remove", "");

		setVisibleColumns(visibleColumns.toArray());
	}

	@Override
	protected Filter getFormEntity() {
		return new Filter();
	}

	@Override
	protected void generateForm(Filter tEntity) {
		form = new FilterForm(tEntity, true);
	}

	@Override
	protected void setGridDataSource() {
		for (FilterCategory filterCategory : FilterCategory.values()) {
			Item rootItem = hierarchicalContainer.addItem(filterCategory);
			rootItem.getItemProperty("displayName").setValue(filterCategory.getCaption());
			List<Filter> filters = filterService.findAllByCategory(filterCategory);
			for (Filter filter : filters) {
				Item parentItem = hierarchicalContainer.addItem(filter.getId());
				parentItem.getItemProperty("displayName").setValue(filter.getFilterName());
				parentItem.getItemProperty("isVisible").setValue(Boolean.TRUE);
				parentItem.getItemProperty("isEditCellEnabled").setValue(Boolean.FALSE);
				parentItem.getItemProperty("isRemoveCellEnabled").setValue(Boolean.TRUE);
				hierarchicalContainer.setParent(filter.getId(), filterCategory);
				baseTreeTable.setCollapsed(filter.getId(), false);
				baseTreeTable.setChildrenAllowed(filter.getId(), false);
			}
			baseTreeTable.setCollapsed(filterCategory, false);
		}
	}

	@Override
	public void windowClose(Window.CloseEvent closeEvent) {

	}

	@EventBusListenerMethod
	public void buttonClicked(EventBusAction eventBusAction) {
		if (eventBusAction.getAction().equals(Action.SAVE) || eventBusAction.getAction().equals(Action.DELETE)) {
			refreshTable();
		}
	}
}
