package hu.oroscom.c1801.view;

import com.vaadin.data.Item;
import com.vaadin.navigator.ViewChangeListener;
import com.vaadin.server.FontAwesome;
import com.vaadin.ui.Button;
import com.vaadin.ui.Table;
import com.vaadin.ui.VerticalLayout;
import com.vaadin.ui.Window;
import com.vaadin.ui.themes.ValoTheme;
import hu.oroscom.c1801.NeededAnyPermission;
import hu.oroscom.c1801.component.cell.PermissionCell;
import hu.oroscom.c1801.component.form.PermissionForm;
import hu.oroscom.c1801.component.table.BaseTable;
import hu.oroscom.c1801.domain.Permission;
import hu.oroscom.c1801.domain.Role;
import hu.oroscom.c1801.enumeration.PermissionName;
import hu.oroscom.c1801.service.PermissionService;
import hu.oroscom.c1801.service.RoleService;
import org.vaadin.spring.annotation.VaadinUIScope;
import org.vaadin.spring.navigator.annotation.VaadinView;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * Created by kollerk on 2014.10.07..
 */
@VaadinUIScope
@VaadinView(name = PermissionsView.NAME)
@NeededAnyPermission(values = {
		PermissionName.PERM_ROLE_SELECT_ANY
})
public class PermissionsView extends GridView<Permission, PermissionService, PermissionForm> {
	public static final String NAME = "permissions";
	private final List<Object> visibleColumns = new ArrayList<>();

	@Inject
	RoleService roleService;

	@Inject
	PermissionService permissionService;

	private Button.ClickListener permissionCellClickListener = new Button.ClickListener() {
		@Override
		public void buttonClick(Button.ClickEvent event) {
			Object[] objects = (Object[]) event.getButton().getData();
			Long permissionId = (Long) objects[0];
			Long roleId = (Long) objects[1];
			Permission permission = permissionService.findOne(permissionId);
			Role role = roleService.findOne(roleId);

			if (event.getButton().getIcon() == null) {
				role.addPermission(permission);
				event.getButton().setIcon(FontAwesome.REMOVE);
			} else {
				role.removePermission(permission);
				event.getButton().setIcon(null);
			}

			roleService.save(role);
//			refreshTable();
		}
	};

	@Override
	protected void init() {
		super.defaultComponents(i18n.get("PermissionsView.permissionsLabel"));
		initTable();
	}

	@Override
	protected void customizeView() {
	}

	@Override
	protected void registerColumns() {
		baseTable.addStyleName("permissionsView-view");
		baseTable.setHeight(100, Unit.PERCENTAGE);
		baseTable.addStyleName(ValoTheme.TABLE_SMALL);
		baseTable.setColumnCollapsingAllowed(true);
		baseTable.setColumnReorderingAllowed(true);

		visibleColumns.clear();
		visibleColumns.addAll(Arrays.asList(new Object[]{"permissionName"}));

		addContainerProperty("permissionName", String.class, "");

		for (Role role : roleService.findAllByCustomer()) {
			addGeneratedColumn(role.getId(), new PermissionCell(permissionCellClickListener));
			addContainerProperty(role.getId(), Boolean.class, Boolean.FALSE);
			baseTable.setColumnHeader(role.getId(), role.getCaption());
			visibleColumns.add(role.getId());
			baseTable.setColumnAlignment(role.getId(), Table.Align.CENTER);
		}

//		addContainerProperty("isRemoveCellEnabled", Boolean.class, false);
		addContainerProperty("isPermissionCellEnabled", Boolean.class, false);

		baseTable.setColumnHeader("permissionName", i18n.get("Permission.name"));
	}

	@Override
	protected Permission getFormEntity() {
		return new Permission();
	}

	@Override
	protected void generateForm(Permission tEntity) {
		form = new PermissionForm(tEntity, true);
	}

	@Override
	protected void setGridDataSource() {
		baseTable.setContainerDataSource(null);
		for (Permission permission : permissionService.findAll()) {
			Item item = indexedContainer.addItem(permission.getId());
			item.getItemProperty("permissionName").setValue(permission.getPermissionName().getCaption());

			for (Role role : roleService.findAllByCustomer()) {
				item.getItemProperty(role.getId()).setValue(role.getPermissions().contains(permission));
			}

			if (roleService.userHasPermission(PermissionName.PERM_ROLE_UPDATE_ANY)) {
				item.getItemProperty("isPermissionCellEnabled").setValue(Boolean.TRUE);
			}
		}
		baseTable.setContainerDataSource(indexedContainer);
		setVisibleColumns(visibleColumns.toArray());
	}

	@Override
	protected void setBaseTable() {
		baseTable = new BaseTable(null, indexedContainer);
		baseTable.setCacheRate(2);

		VerticalLayout filterInfoVerticalLayout = new VerticalLayout();
		filterInfoVerticalLayout.setImmediate(true);
		filterInfoVerticalLayout.setSizeFull();
		filterInfoVerticalLayout.addComponent(baseTable);
		filterInfoVerticalLayout.setExpandRatio(baseTable, 1);
		addComponent(filterInfoVerticalLayout);
		setExpandRatio(filterInfoVerticalLayout, 1);
	}

	@Override
	public void windowClose(Window.CloseEvent closeEvent) {

	}

	@Override
	public void enter(ViewChangeListener.ViewChangeEvent viewChangeEvent) {
		removeAllComponents();
		init();
		super.enter(viewChangeEvent);
	}
}
