package hu.oroscom.c1801.view;

import com.vaadin.data.Item;
import com.vaadin.server.FontAwesome;
import com.vaadin.ui.Button;
import com.vaadin.ui.Table;
import com.vaadin.ui.VerticalLayout;
import com.vaadin.ui.Window;
import com.vaadin.ui.themes.ValoTheme;
import hu.oroscom.c1801.NeededAnyPermission;
import hu.oroscom.c1801.component.cell.ButtonCell;
import hu.oroscom.c1801.component.cell.EditCell;
import hu.oroscom.c1801.component.cell.RemoveCell;
import hu.oroscom.c1801.component.form.RoleForm;
import hu.oroscom.c1801.component.table.BaseTable;
import hu.oroscom.c1801.domain.Role;
import hu.oroscom.c1801.enumeration.PermissionName;
import hu.oroscom.c1801.service.RoleService;
import org.vaadin.spring.annotation.VaadinUIScope;
import org.vaadin.spring.navigator.annotation.VaadinView;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@VaadinUIScope
@VaadinView(name = RolesView.NAME)
@NeededAnyPermission(values = {
		PermissionName.PERM_ROLE_SELECT_ANY
})
public class RolesView extends GridView<Role, RoleService, RoleForm> {
	public static final String NAME = "roles";
	private final List<Object> visibleColumns = new ArrayList<>(Arrays.asList(new Object[]{"roleName", "duplicate", "remove"}));

	@Inject
	RoleService roleService;

	private Button.ClickListener duplicateButtonCellClickListener = new Button.ClickListener() {
		@Override
		public void buttonClick(Button.ClickEvent event) {
			Long roleId = (Long) event.getButton().getData();
			roleService.duplicateRole(roleId);
			refreshTable();
		}
	};

	@Override
	protected void init() {
		super.defaultComponents(i18n.get("RolesView.rolesLabel"));
		initTable();
	}

	@Override
	protected void customizeView() {
		if (roleService.userHasPermission(PermissionName.PERM_ROLE_CREATE_ANY)) {
			setAddButton(i18n.get("RolesView.addRoleButton"));
		}
	}

	@Override
	protected void registerColumns() {
		baseTable.addStyleName("rolesView-view");
		baseTable.setHeight(100, Unit.PERCENTAGE);
		baseTable.addStyleName(ValoTheme.TABLE_SMALL);
		baseTable.setColumnCollapsingAllowed(true);
		baseTable.setColumnReorderingAllowed(true);

		addGeneratedColumn("roleName", new EditCell(editClickListener));
		addGeneratedColumn("duplicate", new ButtonCell(FontAwesome.COPY, duplicateButtonCellClickListener));
		addGeneratedColumn("remove", new RemoveCell<>(RoleService.class));

		addContainerProperty("roleName", String.class, "");
		addContainerProperty("isRemoveCellEnabled", Boolean.class, false);
		addContainerProperty("isEditCellEnabled", Boolean.class, false);
		addContainerProperty("isButtonCellEnabled", Boolean.class, false);

		baseTable.setColumnHeader("roleName", i18n.get("Role.name"));
		baseTable.setColumnHeader("duplicate", "");
		baseTable.setColumnHeader("remove", "");

		baseTable.setColumnWidth("duplicate", 40);
		baseTable.setColumnAlignment("duplicate", Table.Align.CENTER);
	}

	@Override
	protected Role getFormEntity() {
		return new Role();
	}

	@Override
	protected void generateForm(Role tEntity) {
		form = new RoleForm(tEntity, true);
	}

	@Override
	protected void setGridDataSource() {
		baseTable.setContainerDataSource(null);
		for (Role role : roleService.findAllByCustomer()) {
			Item item = indexedContainer.addItem(role.getId());
			item.getItemProperty("roleName").setValue(role.getCaption());
			if (roleService.userHasPermission(PermissionName.PERM_ROLE_DELETE_ANY) && role.getRoleNameEnum() == null) {
				item.getItemProperty("isRemoveCellEnabled").setValue(Boolean.TRUE);
			}
			if (roleService.userHasPermission(PermissionName.PERM_ROLE_UPDATE_ANY) && role.getRoleNameEnum() == null) {
				item.getItemProperty("isEditCellEnabled").setValue(Boolean.TRUE);
			}
			if (roleService.userHasPermission(PermissionName.PERM_ROLE_CREATE_ANY)) {
				item.getItemProperty("isButtonCellEnabled").setValue(Boolean.TRUE);
			}
		}
		baseTable.setContainerDataSource(indexedContainer);
		setVisibleColumns(visibleColumns.toArray());
	}

	@Override
	protected void setBaseTable() {
		baseTable = new BaseTable(null, indexedContainer);
		baseTable.setCacheRate(2);

		VerticalLayout filterInfoVerticalLayout = new VerticalLayout();
		filterInfoVerticalLayout.setImmediate(true);
		filterInfoVerticalLayout.setSizeFull();
		filterInfoVerticalLayout.addComponent(baseTable);
		filterInfoVerticalLayout.setExpandRatio(baseTable, 1);
		addComponent(filterInfoVerticalLayout);
		setExpandRatio(filterInfoVerticalLayout, 1);
	}

	@Override
	public void windowClose(Window.CloseEvent closeEvent) {

	}
}
