package hu.oroscom.c1801.view;

import com.vaadin.data.util.HierarchicalContainer;
import com.vaadin.ui.Table;
import hu.oroscom.c1801.component.form.BaseForm;
import hu.oroscom.c1801.component.table.BaseTreeTable;
import hu.oroscom.c1801.domain.BaseEntity;
import hu.oroscom.c1801.service.BaseService;

/**
 * Created by Norbert on 2014.11.24..
 */
public abstract class TreeGridView<T extends BaseEntity, S extends BaseService, F extends BaseForm<T, S>> extends GridView<T, S, F> {

	protected HierarchicalContainer hierarchicalContainer;
	protected BaseTreeTable baseTreeTable;

	protected abstract void setGridDataSource();

	protected void addContainerProperty(Object id, Class<?> type, Object defaultValue) {
		hierarchicalContainer.addContainerProperty(id, type, defaultValue);
	}

	@Override
	protected void setBeanContainer() {
		hierarchicalContainer = new HierarchicalContainer();
	}

	@Override
	protected void setBaseTable() {
		baseTreeTable = new BaseTreeTable(null, hierarchicalContainer);
//		baseTreeTable.setColumnReorderingAllowed(true);
		addComponent(baseTreeTable);
		setExpandRatio(baseTreeTable, 1);
	}

	@Override
	protected void refreshTable() {
		hierarchicalContainer.removeAllItems();
		setGridDataSource();
		baseTreeTable.setPageLength(hierarchicalContainer.getItemIds().size() + 1);
		baseTreeTable.refreshRowCache();
	}

	@Override
	protected void setVisibleColumns(Object... columns) {
		baseTreeTable.setVisibleColumns(columns);
	}

	@Override
	protected void addGeneratedColumn(Object id, Table.ColumnGenerator columnGenerator) {
		if ("remove".equals(id)) {
			baseTreeTable.setColumnWidth("remove", 40);
			baseTreeTable.setColumnAlignment("remove", Table.Align.CENTER);
		}
		baseTreeTable.addGeneratedColumn(id, columnGenerator);
	}
}
