package hu.oroscom.c1801.view;

import com.vaadin.data.util.BeanItemContainer;
import com.vaadin.ui.UI;
import com.vaadin.ui.Window;
import hu.oroscom.c1801.component.form.AssistantForm;
import hu.oroscom.c1801.component.form.DoctorForm;
import hu.oroscom.c1801.component.form.PatientForm;
import hu.oroscom.c1801.component.form.UserForm;
import hu.oroscom.c1801.domain.*;
import hu.oroscom.c1801.enumeration.Action;
import hu.oroscom.c1801.enumeration.RoleName;
import hu.oroscom.c1801.service.CustomerService;
import hu.oroscom.c1801.service.RoleService;
import hu.oroscom.c1801.util.EventBusAction;
import hu.oroscom.c1801.util.UserUtil;
import org.vaadin.spring.annotation.VaadinUIScope;
import org.vaadin.spring.events.annotation.EventBusListenerMethod;
import org.vaadin.spring.navigator.annotation.VaadinView;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by Norbert on 2014.10.13..
 */
@VaadinUIScope
@VaadinView(name = UserView.NAME)
public class UserView extends BaseView {
	public static final String NAME = "user";

	@Inject
	RoleService roleService;

	@Inject
	CustomerService customerService;

	@Override
	protected void init() {
		super.defaultComponents(i18n.get("UserView.userLabel"));

		User user = UserUtil.getUser();
		List<Role> roleList = new ArrayList<>();
		if (user.hasRole(RoleName.ROLE_SUPER_ADMIN)) {
			roleList.add(roleService.findSuperAdmin());
		} else {
			roleList = roleService.findAllByCustomer();
		}
		BeanItemContainer<Role> roleBeanItemContainer = new BeanItemContainer<>(Role.class, roleList);

		List<Customer> customers = customerService.findAll();
		BeanItemContainer<Customer> customerBeanItemContainer = new BeanItemContainer<>(Customer.class, customers);

		if (user instanceof Assistant) {
			AssistantForm assistantForm = new AssistantForm((Assistant) user, roleBeanItemContainer, customerBeanItemContainer);
			addComponent(assistantForm);
			setExpandRatio(assistantForm, 1);
		} else if (user instanceof Doctor) {
			DoctorForm doctorForm = new DoctorForm((Doctor) user, roleBeanItemContainer, customerBeanItemContainer);
			addComponent(doctorForm);
			setExpandRatio(doctorForm, 1);
		} else if (user instanceof Patient) {
			PatientForm patientForm = new PatientForm((Patient) user, roleBeanItemContainer, customerBeanItemContainer);
			addComponent(patientForm);
			setExpandRatio(patientForm, 1);
		} else {
			UserForm userForm = new UserForm(user, roleBeanItemContainer, customerBeanItemContainer);
			addComponent(userForm);
			setExpandRatio(userForm, 1);
		}

		addStyleName("v-scrollable");
		setHeightUndefined();
	}

	@Override
	public void windowClose(Window.CloseEvent closeEvent) {

	}

	@EventBusListenerMethod
	public void buttonClicked(EventBusAction eventBusAction) {
		if (eventBusAction.getEntity() != null && User.class.equals(eventBusAction.getEntity().getClass()) && Action.SAVE.equals(eventBusAction.getAction()) && UserView.this.equals(UI.getCurrent().getNavigator().getCurrentView())) {
			init();
		}
	}
}
