package hu.oroscom.c1801.view;

import com.vaadin.data.Item;
import com.vaadin.data.util.BeanItemContainer;
import com.vaadin.ui.Window;
import hu.oroscom.c1801.NeededAnyPermission;
import hu.oroscom.c1801.component.cell.EditCell;
import hu.oroscom.c1801.component.cell.RemoveCell;
import hu.oroscom.c1801.component.cell.StatusCell;
import hu.oroscom.c1801.component.form.UserForm;
import hu.oroscom.c1801.domain.*;
import hu.oroscom.c1801.enumeration.Action;
import hu.oroscom.c1801.enumeration.EntityStatus;
import hu.oroscom.c1801.enumeration.PermissionName;
import hu.oroscom.c1801.enumeration.RoleName;
import hu.oroscom.c1801.service.*;
import hu.oroscom.c1801.service.UserService;
import hu.oroscom.c1801.util.EventBusAction;
import hu.oroscom.c1801.util.UserUtil;
import org.vaadin.spring.annotation.VaadinUIScope;
import org.vaadin.spring.events.annotation.EventBusListenerMethod;
import org.vaadin.spring.navigator.annotation.VaadinView;

import javax.inject.Inject;
import java.util.*;

/**
 * Created by kollerk on 2014.10.07..
 */
@VaadinUIScope
@VaadinView(name = UsersView.NAME)
@NeededAnyPermission(values = {
		PermissionName.PERM_USER_SELECT_ANY
})
public class UsersView extends TreeGridView<User, UserService, UserForm> {
	public static final String NAME = "users";

	@Inject
	UserService userService;

	@Inject
	RoleService roleService;

	@Inject
	CustomerService customerService;

	@Override
	protected void init() {
		super.defaultComponents(i18n.get("UsersView.usersLabel"));
		initTable();
//
//		if (!UserUtil.getUser().getRole().getRoleName().equals(RoleName.ROLE_SUPER_ADMIN) && UserUtil.getUser().getVenues().size() == 0) {
//			editViewAndAddFormToWindow(UserUtil.getUser());
//		}
	}

	@Override
	protected void customizeView() {
		if (userService.userHasPermission(PermissionName.PERM_USER_CREATE_ANY)) {
			setAddButton(i18n.get("UsersView.addUserButton"));
		}
	}

	@Override
	protected void registerColumns() {
		List<Object> visibleColumns = new ArrayList<>(Arrays.asList(new Object[]{"displayName", "userName", "status", "remove"}));

		addGeneratedColumn("displayName", new EditCell(editClickListener));
		addGeneratedColumn("status", new StatusCell());
		addGeneratedColumn("remove", new RemoveCell<>(UserService.class));

		addContainerProperty("displayName", String.class, null);
		addContainerProperty("userName", String.class, null);
		addContainerProperty("status", EntityStatus.class, EntityStatus.ACTIVE);
		addContainerProperty("isEditCellEnabled", Boolean.class, false);
		addContainerProperty("isRemoveCellEnabled", Boolean.class, false);
		addContainerProperty("isVisible", Boolean.class, false);

		baseTreeTable.setColumnHeader("displayName", i18n.get("UsersView.displayName"));
		baseTreeTable.setColumnHeader("userName", i18n.get("UsersView.userName"));
		baseTreeTable.setColumnHeader("status", i18n.get("UsersView.status"));
		baseTreeTable.setColumnHeader("remove", "");

		setVisibleColumns(visibleColumns.toArray());
	}

	@Override
	protected User getFormEntity() {
		return new User();
	}

	@Override
	protected void generateForm(User tEntity) {
		form = new UserForm(tEntity, true);
	}

	@Override
	protected void setGridDataSource() {
		for (Role role : roleService.findAllWithRole(UserUtil.getCustomer(), RoleName.ROLE_SUPER_ADMIN, RoleName.ROLE_ADMIN)) {
			Item rootItem = hierarchicalContainer.addItem(role);

			String customerName = "";
			if (UserUtil.isSuperAdmin()) {
				customerName = role.getCustomer().getName() + " - ";
			}

			rootItem.getItemProperty("displayName").setValue(customerName + role.getCaption());
			List<User> parentUserList = userService.findAllByRole(role);
			for (User parentUser : parentUserList) {
				Item parentItem = hierarchicalContainer.addItem(parentUser.getId());
				parentItem.getItemProperty("displayName").setValue(parentUser.getDisplayName());
				parentItem.getItemProperty("userName").setValue(parentUser.getUserName());
				parentItem.getItemProperty("status").setValue(parentUser.getStatus());
				parentItem.getItemProperty("isVisible").setValue(Boolean.TRUE);
				if (userService.userHasPermission(PermissionName.PERM_USER_UPDATE_ANY)) {
					parentItem.getItemProperty("isEditCellEnabled").setValue(Boolean.TRUE);
				}
				if (userService.userHasPermission(PermissionName.PERM_USER_DELETE_ANY)) {
					parentItem.getItemProperty("isRemoveCellEnabled").setValue(Boolean.TRUE);
				}
				hierarchicalContainer.setParent(parentUser.getId(), role);
				baseTreeTable.setCollapsed(parentUser.getId(), false);
				baseTreeTable.setChildrenAllowed(parentUser.getId(), false);
			}
			baseTreeTable.setCollapsed(role, false);
		}
	}

	@Override
	public void windowClose(Window.CloseEvent closeEvent) {

	}

	@EventBusListenerMethod
	public void buttonClicked(EventBusAction eventBusAction) {
		if (eventBusAction.getAction().equals(Action.SAVE) || eventBusAction.getAction().equals(Action.DELETE)) {
			refreshTable();
		}
	}
}
