package hu.oroscom.c1801.webservice;

import hu.oroscom.c1801.service.TokenService;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.security.authentication.AuthenticationServiceException;
import org.springframework.security.authentication.LockedException;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.web.authentication.AbstractAuthenticationProcessingFilter;
import org.springframework.security.web.util.matcher.RequestMatcher;
import org.springframework.web.util.NestedServletException;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.text.MessageFormat;

/**
 * Created by kollerk on 15. 01. 30..
 */
public class CustomTokenAuthenticationFilter extends AbstractAuthenticationProcessingFilter {

	private static final String FILTER_APPLIED = "REST_API_AUTH_FILTER";
	private static final String X_AUTH_TOKEN = "X-AuthToken";

	public CustomTokenAuthenticationFilter(RequestMatcher requiresAuthenticationRequestMatcher) {
		super(requiresAuthenticationRequestMatcher);
	}

	@Override
	public Authentication attemptAuthentication(HttpServletRequest request, HttpServletResponse response) throws AuthenticationException, IOException, ServletException {

		String token = request.getHeader(X_AUTH_TOKEN);
		Authentication userAuthenticationToken = null;

		TokenProxy tokenProxy;
		try {
			tokenProxy = TokenService.getTokenProxyByToken(token);
			userAuthenticationToken = new UsernamePasswordAuthenticationToken(tokenProxy.getUserName(), tokenProxy.getPassword());
			if (userAuthenticationToken == null) {
				throw new AuthenticationServiceException(MessageFormat.format("Error | {0}", "Bad Token"));
			}
		} catch (Exception e) {
			e.printStackTrace();
			throw new AuthenticationServiceException(MessageFormat.format("Error | {0}", "Bad Token"));
		}

		return userAuthenticationToken;

	}

	@Override
	public void doFilter(ServletRequest req, ServletResponse res, FilterChain chain) throws IOException, ServletException {
		HttpServletRequest request = (HttpServletRequest) req;
		HttpServletResponse response = (HttpServletResponse) res;

		if (request.getAttribute(FILTER_APPLIED) != null) {
			chain.doFilter(request, response);
			return;
		}

		request.setAttribute(FILTER_APPLIED, Boolean.TRUE);

//		if(request.getParameter(actionParameter) !=null &&
//				request.getParameter(actionParameter).equals("logout")) {
//			SecurityContextHolder.clearContext();
//			return;
//		}

		if (!requiresAuthentication(request, response) || request.getMethod().equals("OPTIONS")) {
			chain.doFilter(request, response);
			return;
		}

		Authentication authResult;
		try {
			authResult = attemptAuthentication(request, response);
			if (authResult == null) {
				return;
			}
		} catch (AuthenticationException failed) {
			unsuccessfulAuthentication(request, response, failed);
			return;
		}

		try {
			successfulAuthentication(request, response, chain, authResult);
		} catch (NestedServletException e) {
			if (e.getCause() instanceof AccessDeniedException) {
				unsuccessfulAuthentication(request, response, new LockedException("Forbidden"));
			}
		}
	}

	/*@Override
	protected void successfulAuthentication(HttpServletRequest request, HttpServletResponse response,
											Authentication authResult) throws IOException, ServletException {
		SecurityContextHolder.getContext().setAuthentication(authResult);

		getSuccessHandler().onAuthenticationSuccess(request, response, authResult);
	}*/
}
