package hu.oroscom.c1801.widgetset;

import com.vaadin.ui.InlineDateField;
import hu.oroscom.c1801.widgetset.client.CustomInlineDateFieldState;
import hu.oroscom.c1801.widgetset.client.PageChangeRpc;
import org.joda.time.DateTime;

import java.util.*;

/**
 * Created by Norbert on 2014.11.28..
 */
public class CustomInlineDateField extends InlineDateField {


	private final Set<PageChangeListener> pageChangeListeners = new HashSet<>();

	private PageChangeRpc pageChangeRpc = new PageChangeRpc() {
		@Override
		public void selected(Date value) {
			DateTime baseDateTime = new DateTime(value);
			DateTime startDateTime = baseDateTime.withDayOfWeek(1);
			DateTime endDateTime = startDateTime.plusWeeks(6).minusDays(1);
			fireValueChangeEvent(startDateTime, endDateTime);
		}
	};

	public CustomInlineDateField() {
		registerRpc(pageChangeRpc);
	}

	@Override
	protected CustomInlineDateFieldState getState() {
		return (CustomInlineDateFieldState) super.getState();
	}

	public Date[] getDisabledDates() {
		return getDates(getState().disabledDates);
	}

	public void setDisabledDates(Date... dates) {
		getState().disabledDates = Arrays.asList(dates);
	}

	public Date[] getDisabledVenueDates() {
		return getDates(getState().disabledVenueDates);
	}

	public void setDisabledVenueDates(Date... dates) {
		getState().disabledVenueDates = Arrays.asList(dates);
	}

	public Date[] getDisabledEmployeeDates() {
		return getDates(getState().disabledEmployeeDates);
	}

	public void setDisabledEmployeeDates(Date... dates) {
		getState().disabledEmployeeDates = Arrays.asList(dates);
	}

	public Date[] getBookableDates() {
		return getDates(getState().bookableDates);
	}

	public void setBookableDates(Date... dates) {
		getState().bookableDates = Arrays.asList(dates);
	}

	public Date[] getBookableVenueDates() {
		return getDates(getState().bookableVenueDates);
	}

	public void setBookableVenueDates(Date... dates) {
		getState().bookableVenueDates = Arrays.asList(dates);
	}

	public Date[] getBookableEmployeeDates() {
		return getDates(getState().bookableEmployeeDates);
	}

	public void setBookableEmployeeDates(Date... dates) {
		getState().bookableEmployeeDates = Arrays.asList(dates);
	}

	public Date getStartDate() {
		return getState().startDate;
	}

	public void setStartDate(Date start) {
		getState().startDate = start;
	}

	public void setEndDate(Date end) {
		getState().endDate = end;
	}

	public Date getEndDate() {
		return getState().endDate;
	}

	protected void fireValueChangeEvent(DateTime start, DateTime end) {
		for (PageChangeListener listener : pageChangeListeners) {
			listener.pageChange(start, end);
		}
	}

	public void addPageChangeListener(PageChangeListener listener) {
		pageChangeListeners.add(listener);
	}

	public void removePageChangeListener(PageChangeListener listener) {
		pageChangeListeners.remove(listener);
	}

	private Date[] getDates(List<Date> dates) {
		if (dates == null || dates.isEmpty()) {
			return new Date[0];
		}
		return dates.toArray(new Date[dates.size()]);
	}
}
