package hu.oroscom.c1801.widgetset.client;

import com.vaadin.client.ApplicationConnection;
import com.vaadin.client.DateTimeService;
import com.vaadin.client.UIDL;
import com.vaadin.client.communication.RpcProxy;
import com.vaadin.client.communication.StateChangeEvent;
import com.vaadin.client.ui.datefield.AbstractDateFieldConnector;
import com.vaadin.shared.ui.Connect;
import com.vaadin.shared.ui.datefield.Resolution;
import hu.oroscom.c1801.widgetset.CustomInlineDateField;

import java.util.Date;
import java.util.List;

/**
 * Created by Norbert on 2014.11.28..
 */
@Connect(CustomInlineDateField.class)
public class CustomInlineDateFieldConnector extends AbstractDateFieldConnector {

	private final PageChangeRpc pageChangeRpc = RpcProxy.create(PageChangeRpc.class, this);
	private boolean isRpcCalled = false;

	@Override
	protected void init() {
		super.init();
		getWidget().addListener(new PageChangeListener() {
			@Override
			public void pageChange(Date value) {
				isRpcCalled = true;
				pageChangeRpc.selected(value);
			}
		});
	}

	@Override
	@SuppressWarnings("deprecation")
	public void updateFromUIDL(UIDL uidl, ApplicationConnection client) {
		super.updateFromUIDL(uidl, client);
		if (!isRealUpdate(uidl)) {
			return;
		}

		getWidget().calendarPanel.setShowISOWeekNumbers(getWidget()
				.isShowISOWeekNumbers());
		getWidget().calendarPanel.setDateTimeService(getWidget()
				.getDateTimeService());
		getWidget().calendarPanel.setResolution(getWidget()
				.getCurrentResolution());
		Date currentDate = getWidget().getCurrentDate();
		if (!isRpcCalled) {
			if (currentDate != null) {
				getWidget().calendarPanel.setDate(new Date(currentDate.getTime()));
			} else {
				getWidget().calendarPanel.setDate(null);
			}
		}

		if (getWidget().getCurrentResolution().getCalendarField() > Resolution.DAY
				.getCalendarField()) {
			getWidget().calendarPanel
					.setTimeChangeListener(new CustomVCalendarPanel.TimeChangeListener() {
						@Override
						public void changed(int hour, int min, int sec, int msec) {
							Date d = getWidget().getDate();
							if (d == null) {
								// date currently null, use the value from
								// calendarPanel
								// (~ client time at the init of the widget)
								d = (Date) getWidget().calendarPanel.getDate()
										.clone();
							}
							d.setHours(hour);
							d.setMinutes(min);
							d.setSeconds(sec);
							DateTimeService.setMilliseconds(d, msec);

							// Always update time changes to the server
							getWidget().calendarPanel.setDate(d);
							getWidget().updateValueFromPanel();
						}
					});
		}

		if (getWidget().getCurrentResolution().getCalendarField() <= Resolution.MONTH
				.getCalendarField()) {
			getWidget().calendarPanel
					.setFocusChangeListener(new CustomVCalendarPanel.FocusChangeListener() {
						@Override
						public void focusChanged(Date date) {
							Date date2 = new Date();
							if (getWidget().calendarPanel.getDate() != null) {
								date2.setTime(getWidget().calendarPanel
										.getDate().getTime());
							}
							/*
							 * Update the value of calendarPanel
                             */
							date2.setYear(date.getYear());
							date2.setMonth(date.getMonth());
							getWidget().calendarPanel.setDate(date2);
                            /*
                             * Then update the value from panel to server
                             */
							getWidget().updateValueFromPanel();
						}
					});
		} else {
			getWidget().calendarPanel.setFocusChangeListener(null);
		}

		// Update possible changes
		getWidget().calendarPanel.renderCalendar();
		isRpcCalled = false;
	}

	@Override
	public void onStateChanged(StateChangeEvent stateChangeEvent) {
		super.onStateChanged(stateChangeEvent);

		getWidget().setTabIndex(getState().tabIndex);
		getWidget().calendarPanel.setRangeStart(getState().rangeStart);
		getWidget().calendarPanel.setRangeEnd(getState().rangeEnd);

		if (stateChangeEvent.hasPropertyChanged("disabledDates")) {
			getWidget().setDisabledDates(getDays(getState().disabledDates));
		}

		if (stateChangeEvent.hasPropertyChanged("disabledVenueDates")) {
			getWidget().setDisabledVenueDates(getDays(getState().disabledVenueDates));
		}

		if (stateChangeEvent.hasPropertyChanged("disabledEmployeeDates")) {
			getWidget().setDisabledEmployeeDates(getDays(getState().disabledEmployeeDates));
		}

		if (stateChangeEvent.hasPropertyChanged("bookableDates")) {
			getWidget().setBookableDates(getDays(getState().bookableDates));
		}

		if (stateChangeEvent.hasPropertyChanged("bookableVenueDates")) {
			getWidget().setBookableVenueDates(getDays(getState().bookableVenueDates));
		}

		if (stateChangeEvent.hasPropertyChanged("bookableEmployeeDates")) {
			getWidget().setBookableEmployeeDates(getDays(getState().bookableEmployeeDates));
		}

		if (stateChangeEvent.hasPropertyChanged("startDate")) {
			getWidget().setStartDate(getState().startDate);
		}

		if (stateChangeEvent.hasPropertyChanged("endDate")) {
			getWidget().setEndDate(getState().endDate);
		}
	}

	@Override
	public CustomInlineDateFieldWidget getWidget() {
		return (CustomInlineDateFieldWidget) super.getWidget();
	}

	@Override
	public CustomInlineDateFieldState getState() {
		return (CustomInlineDateFieldState) super.getState();
	}

	private Date[] getDays(List<Date> days) {
		if (days == null || days.isEmpty()) {
			return null;
		} else {
			Date[] daysArray = new Date[days.size()];
			for (int i = 0; i < days.size(); i++) {
				daysArray[i] = days.get(i);
			}
			return daysArray;
		}
	}
}
