package hu.oroscom.c1801.widgetset.client;

import com.google.gwt.event.dom.client.DomEvent;
import com.vaadin.client.DateTimeService;
import com.vaadin.client.ui.VDateField;
import com.vaadin.shared.ui.datefield.Resolution;

import java.util.Date;

/**
 * Created by Norbert on 2014.11.28..
 */
public class CustomInlineDateFieldWidget extends VDateField {

	public final CustomVCalendarPanel calendarPanel;

	private Date startDate = null;
	private Date endDate = null;

	public CustomInlineDateFieldWidget() {
		super();
		calendarPanel = new CustomVCalendarPanel();
		calendarPanel.setParentField(this);
		add(calendarPanel);
		calendarPanel.setSubmitListener(new CustomVCalendarPanel.SubmitListener() {
			@Override
			public void onSubmit() {
				updateValueFromPanel();
			}

			@Override
			public void onCancel() {
				// TODO Auto-generated method stub

			}
		});
		calendarPanel.setFocusOutListener(new CustomVCalendarPanel.FocusOutListener() {
			@Override
			public boolean onFocusOut(DomEvent<?> event) {
				updateValueFromPanel();
				return false;
			}
		});
	}

	public void updateValueFromPanel() {

		// If field is invisible at the beginning, client can still be null when
		// this function is called.
		if (getClient() == null) {
			return;
		}

		Date date2 = calendarPanel.getDate();
		Date currentDate = getCurrentDate();
		if (currentDate == null || date2.getTime() != currentDate.getTime()) {
			setCurrentDate((Date) date2.clone());
			getClient().updateVariable(getId(), "year", date2.getYear() + 1900,
					false);
			if (getCurrentResolution().getCalendarField() > Resolution.YEAR
					.getCalendarField()) {
				getClient().updateVariable(getId(), "month",
						date2.getMonth() + 1, false);
				if (getCurrentResolution().getCalendarField() > Resolution.MONTH
						.getCalendarField()) {
					getClient().updateVariable(getId(), "day", date2.getDate(),
							false);
					if (getCurrentResolution().getCalendarField() > Resolution.DAY
							.getCalendarField()) {
						getClient().updateVariable(getId(), "hour",
								date2.getHours(), false);
						if (getCurrentResolution().getCalendarField() > Resolution.HOUR
								.getCalendarField()) {
							getClient().updateVariable(getId(), "min",
									date2.getMinutes(), false);
							if (getCurrentResolution().getCalendarField() > Resolution.MINUTE
									.getCalendarField()) {
								getClient().updateVariable(getId(), "sec",
										date2.getSeconds(), false);
								if (getCurrentResolution().getCalendarField() > Resolution.SECOND
										.getCalendarField()) {
									getClient().updateVariable(
											getId(),
											"msec",
											DateTimeService
													.getMilliseconds(date2),
											false);
								}
							}
						}
					}
				}
			}
			if (isImmediate()) {
				getClient().sendPendingVariableChanges();
			}
		}
	}

	public void setTabIndex(int tabIndex) {
		calendarPanel.getElement().setTabIndex(tabIndex);
	}

	public int getTabIndex() {
		return calendarPanel.getElement().getTabIndex();
	}

	public void setDisabledDates(Date... disabledDates) {
		calendarPanel.setDisabledDates(disabledDates);
	}

	public void setDisabledVenueDates(Date... disabledDates) {
		calendarPanel.setDisabledVenueDates(disabledDates);
	}

	public void setDisabledEmployeeDates(Date... disabledDates) {
		calendarPanel.setDisabledEmployeeDates(disabledDates);
	}

	public void setBookableDates(Date... bookableDates) {
		calendarPanel.setBookableDates(bookableDates);
	}

	public void setBookableVenueDates(Date... bookableDates) {
		calendarPanel.setBookableVenueDates(bookableDates);
	}

	public void setBookableEmployeeDates(Date... bookableDates) {
		calendarPanel.setBookableEmployeeDates(bookableDates);
	}

	public Date getStartDate() {
		return this.startDate;
	}

	public void setStartDate(Date date) {
		this.startDate = date;
		calendarPanel.setStartDate(date);
	}

	public Date getEndDate(Date date) {
		return this.endDate;
	}

	public void setEndDate(Date date) {
		this.endDate = date;
		calendarPanel.setEndDate(date);
	}

	public void addListener(PageChangeListener listener) {
		calendarPanel.getPageChangeListeners().add(listener);
	}

	public void removeListener(PageChangeListener listener) {
		calendarPanel.getPageChangeListeners().remove(listener);
	}
}
